package org.spring.heat.services;

import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Date;
import java.util.List;

import javax.servlet.ServletContext;
import javax.servlet.http.HttpSession;

import org.hibernate.Query;
import org.hibernate.Session;
import org.hibernate.SessionFactory;
import org.hibernate.Transaction;
import org.hibernate.heat.domain.BankCertificate;
import org.hibernate.heat.domain.BankInfo;
import org.hibernate.heat.domain.BuildingInfo;
import org.hibernate.heat.domain.CheckedInfo;
import org.hibernate.heat.domain.DueCharge;
import org.hibernate.heat.domain.FeemodifyInfo;
import org.hibernate.heat.domain.FeemodifyInfoDAO;
import org.hibernate.heat.domain.HeatsysSetting;
import org.hibernate.heat.domain.ProjectInfo;
import org.hibernate.heat.domain.StaffInfo;
import org.hibernate.heat.domain.StchgDaily;
import org.hibernate.heat.domain.StmdyDaily;
import org.hibernate.heat.domain.StmdyRange;
import org.hibernate.heat.domain.TaskassignInfo;
import org.hibernate.heat.domain.UnitPrice;
import org.hibernate.heat.domain.UsersInfo;
import org.hibernate.heat.domain.UsersInfoDAO;
import org.hibernate.heat.util.FileUploader;
import org.spring.heat.entity.BankCertInfo;
import org.spring.heat.entity.StaffPermission;
import org.spring.heat.models.ChangeInfo;
import org.spring.heat.models.ChangeList;
import org.spring.heat.models.Global;
import org.spring.heat.models.Result;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.ApplicationContext;
import org.springframework.orm.hibernate3.SessionFactoryUtils;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.context.support.WebApplicationContextUtils;

@Service("changeInfoService")
@Transactional
public class ChangeInfoService{
	
	@Autowired
	private UsersInfoDAO userInfoDAO;
	
	@Autowired
	private FeemodifyInfoDAO feemodifyInfoDAO;
	
	@Autowired
	private FileUploader fileUploader;
	//列出指定类型的变更信息
	public Result getChangeListWithType(
			String phase,
			String starttime,
			String endtime,
			String auditstart,
			String auditend,
			String queryStr,
			String requestman, 
			String auditman, 
			String type, 
			Long stfid, 
			Boolean isall, 
			int start, 
			int limit, 
			HttpSession session)
	{
		ServletContext servletContext = session.getServletContext();    
        ApplicationContext ctx = WebApplicationContextUtils.getWebApplicationContext(servletContext);
        SessionFactory sessionFactory = (SessionFactory) ctx.getBean("sessionFactory");
        Session hsession = SessionFactoryUtils.getSession(sessionFactory, true);
        Result result = new Result();	
		try{
			String conditions = "";
			String hql = "";
			
			if (!starttime.equals(""))
				conditions += " and fi.mdytime>=:starttime";
			
			if (!endtime.equals(""))
				conditions += " and fi.mdytime<=:endtime";
			
			String auditconditions = "";
			
			if (!auditstart.equals("") || !auditend.equals(""))
			{
				auditconditions += "(";
				if (!auditstart.equals(""))
					auditconditions += " fi.audittime>=:auditstart";
				if (!auditend.equals(""))
					auditconditions += " or fi.audittime<=:auditend";
				auditconditions += ")";
				
				auditconditions += "or (";
				if (!auditstart.equals(""))
					auditconditions += " fi.execaudittime>=:auditstart";
				if (!auditend.equals(""))
					auditconditions += " or fi.execaudittime<=:auditend";
				auditconditions += ")";
			}
			
			if (!auditconditions.equals(""))
				conditions += " and " + auditconditions;
			
			if (!type.equals(Global.ALL))
				conditions += " and fi.typename=:typename";
			
			if (isall==false)
				conditions += " and fi.staffInfoByRqtid.stfid = "+stfid;
			
			if (!phase.equals("") && !phase.equals(Global.ALL))
				conditions += " and fi.status=:status";
			
			if (!requestman.equals(""))
				conditions += " and fi.rqtname=:requestman";
			
			if (!auditman.equals(""))
				conditions += " and fi.chkname=:auditman";
			
			if (!queryStr.equals(""))
				conditions += " and (fi.rqtname like :querystr or fi.chkname like :querystr or fi.dealname like :querystr)";
			
			hql = "from FeemodifyInfo fi join fi.usersInfo ui where ui.isvalid=:isvalid and fi.isvalid=:isvalid " + conditions + " order by fi.feemdyid desc";
			Query query = hsession.createQuery(hql).setBoolean("isvalid", true).setFirstResult(start).setMaxResults(limit);
			
			if (!starttime.equals(""))
				query.setTimestamp("starttime", Global.secondformat.parse(starttime + " 00:00:00"));
			
			if (!endtime.equals(""))
				query.setTimestamp("endtime", Global.secondformat.parse(endtime + " 23:59:59"));
			
			if (!auditstart.equals(""))
				query.setTimestamp("auditstart", Global.secondformat.parse(auditstart + " 00:00:00"));
			
			if (!auditend.equals(""))
				query.setTimestamp("auditend", Global.secondformat.parse(auditend + " 23:59:59"));
			
			if (!queryStr.equals(""))
				query.setString("querystr", "%" + queryStr + "%");
			
			if (!phase.equals("") && !phase.equals(Global.ALL))
				query.setString("status", phase);
			
			if (!type.equals(Global.ALL))
				query.setString("typename", type);
			
			if (!requestman.equals(""))
				query.setString("requestman", requestman);
			
			if (!auditman.equals(""))
				query.setString("auditman", auditman);
			
			List feelist = query.list();
			
			List data = new ArrayList();
			Long genReason = null;
			for(int i=0;i<feelist.size();++i){
				Object[] info = (Object[]) feelist.get(i);
				FeemodifyInfo inf = (FeemodifyInfo)info[0];
				UsersInfo user = (UsersInfo)info[1];
				
				if (inf == null || user == null)
					throw new Exception("mdy info or related user is null!");
				
				data.add(
						new ChangeInfo(
								inf.getFeemdyid(),
								Global.secondformat.format(inf.getMdytime()),
								inf.getStaffInfoByRqtid().getStfid(),
								inf.getRqtname(),
								(inf.getStaffInfoByChkid()==null ? null : inf.getStaffInfoByChkid().getStfid()),
								inf.getChkname(),
								inf.getTypename(),
								inf.getOrivalue(),
								inf.getNewvalue(),
								inf.getAffect(),
								inf.getStatus(),
								user.getUsrid(),
								(inf.getGenreason() ? (long)1 : (long)0))
						);
			}		
			
			result.setData(data);
			
			hql = "select count(*) " + hql;
			query = hsession.createQuery(hql).setBoolean("isvalid", true);
			
			if (!starttime.equals(""))
				query.setTimestamp("starttime", Global.secondformat.parse(starttime + " 00:00:00"));
			
			if (!endtime.equals(""))
				query.setTimestamp("endtime", Global.secondformat.parse(endtime + " 23:59:59"));
			
			if (!auditstart.equals(""))
				query.setTimestamp("auditstart", Global.secondformat.parse(auditstart + " 00:00:00"));
			
			if (!auditend.equals(""))
				query.setTimestamp("auditend", Global.secondformat.parse(auditend + " 23:59:59"));
			
			if (!queryStr.equals(""))
				query.setString("querystr", "%" + queryStr + "%");
			
			if (!phase.equals("") && !phase.equals(Global.ALL))
				query.setString("status", phase);
			
			if (!type.equals(Global.ALL))
				query.setString("typename", type);
			
			if (!requestman.equals(""))
				query.setString("requestman", requestman);
			
			if (!auditman.equals(""))
				query.setString("auditman", auditman);
			
			List rows = query.list();
			
			if (rows.size() > 0)
				result.setTotalProperty((Long)rows.get(0));
			else
				result.setTotalProperty((long)0);
			
		}catch(Exception e){
			e.printStackTrace();
			result.setSuccess(false);
			result.setMessage("操作失败");
		}
		return result;
	}
	
	//列出变更信息（不分页）
	@SuppressWarnings("unchecked")
	public ChangeList getInfoList(Long usrid,Boolean isall){
		ChangeList list = new ChangeList();
		SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
		try{
			UsersInfo user = new UsersInfo();
			user.setUsrid(usrid);
			List feelist = this.feemodifyInfoDAO.findByProperty("usersInfo", user);
			List data = new ArrayList();
			Long genReason = null;
			for(Object info : feelist){
				FeemodifyInfo inf = (FeemodifyInfo)info;
				if(inf.getGenreason())
					genReason = (long)1;
				else 
					genReason = (long)0;
				String status = inf.getStatus();
				if(inf.getIsvalid()==true){//当前信息有效
					Boolean isCheck = status.equals(Global.CHECK);
					Boolean isExe = status.equals(Global.EXE);
					if (isall == false && (isCheck||isExe))
						continue;
					if(inf.getStaffInfoByChkid()==null)//当未审核时，审核人为空
						data.add(new ChangeInfo(inf.getFeemdyid(),sdf.format(inf.getMdytime()),inf.getStaffInfoByRqtid().getStfid(),inf.getRqtname(),
								null,inf.getChkname(),inf.getTypename(),inf.getOrivalue(),
								inf.getNewvalue(),inf.getAffect(),inf.getStatus(),usrid,genReason));
					else
						data.add(new ChangeInfo(inf.getFeemdyid(),sdf.format(inf.getMdytime()),inf.getStaffInfoByRqtid().getStfid(),inf.getRqtname(),
								inf.getStaffInfoByChkid().getStfid(),inf.getChkname(),inf.getTypename(),inf.getOrivalue(),
								inf.getNewvalue(),inf.getAffect(),inf.getStatus(),usrid,genReason));
				}		
			}
			list.setTotalProperty((long)data.size());
			list.setData(data);
			list.setMessage("");
			list.setSuccess(true);
			return list;
		}catch(Exception e){
			e.printStackTrace();
			list.setSuccess(false);
			list.setMessage("failure");
			return list;
		}
	}
	
	//变更请求（该请求中会发送所有字段，可以通过type获取真正修改的字段从而获取修改值）
	/*
	 * @RequestMapping(value="",method=RequestMethod.GET)
	 * public Result applyChange(HttpServletRequest request){
	 * Long usrid = request.getParameter("usrid");
	 * String typename = request.getParameter("typename");
	 * String type = request.getParameter("type");
	 * String newvalue = request.getParameter(type)+"";
	 * String newvalue = request.getParameter("oldVal");
	 * return service.ApplyChangeWithRequest(usrid,typename,type,genreason,newvalue,oldVal);
	 */
	public Result ApplyChangeWithRequest(Long usrid,// （用户ID）
			String typename,// （修改类型名，同变更信息字段）
			String type,// （修改值的名称）
			Long genreason,// （都为1）
			String newvalue,
			String oldVal,
			Long rqtid,
			String rqtname){
		Result result = null;
		try{
			FeemodifyInfo info = new FeemodifyInfo();
			info.setMdytime(Calendar.getInstance().getTime());
		
			if(rqtid!=null){
				StaffInfo staff = new StaffInfo();
				staff.setStfid(rqtid);
				info.setRqtname(rqtname);
				info.setStaffInfoByRqtid(staff);
			}
			info.setNewvalue(newvalue);
			info.setTypename(typename);
			info.setOrivalue(oldVal);
//			info.setAffect(affect);
			info.setStatus(Global.APPLY);
			info.setIsvalid(true);
			if (usrid != null) {
				UsersInfo temp = new UsersInfo();
				temp.setUsrid(usrid);
				info.setUsersInfo(temp);
			}
			this.feemodifyInfoDAO.attachDirty(info);
			result = Global.SUCCESS;
		}catch(Exception e){
			e.printStackTrace();
			result = Global.FAILURE;
		}
		return result;
	}
	
	public Result ApplyChangeNoRequest(
			Long usrid,// （用户ID）
			String typename,// （修改类型名，同变更信息字段）
			String type,// （修改值的名称）
			Long genreason,// （都为1）
			Long chgid,
			Long rqtid,
			String rqtname,
			String usrname,// （红色的选填，根据type的值判断取哪个）
			String usertype,String contracttype,String contractver,String contractpic,String phone,//
			Double area,Double realarea,Double feearea,String feetype,String upid,
			Double feerate,
			Double discount,
			Double reducefee,
			String heatstate,//
			Double heatbase,//（heatstate为热计量计费时才有）
			Double heatrate,//（heatstate为热计量计费时才有）
			String isaffect,// （是否影响应缴记录，影响为“on”，否则为空）
			String oldVal,
			StaffPermission sp,
			HttpSession session// (原始值，在type为heatstate且heatstate值为热计量费率时，值为“热计量基数_10_10”,以下划线分隔，第一个值为heatstate的值，第二个值为热计量基数，第三个数为热计量费率)
			){
		Result result = null;
		ServletContext servletContext = session.getServletContext();    
        ApplicationContext ctx = WebApplicationContextUtils.getWebApplicationContext(servletContext);
        SessionFactory sessionFactory = (SessionFactory) ctx.getBean("sessionFactory");
        Session hsession = SessionFactoryUtils.getSession(sessionFactory, true);
        Transaction transaction = hsession.beginTransaction();
        String hql = "";
        Query query;
		try{
			query = hsession.createQuery("from HeatsysSetting");
			HeatsysSetting heatsysSetting = (HeatsysSetting)query.list().get(0);
			
			FeemodifyInfo info = new FeemodifyInfo();
			info.setDealname(heatsysSetting.getCurdealname());
			info.setMdytime(Calendar.getInstance().getTime());
			if(rqtid!=null){
				StaffInfo staff = new StaffInfo();
				staff.setStfid(rqtid);
				info.setRqtname(rqtname);
				info.setStaffInfoByRqtid(staff);
			}
			
			UsersInfo user = null;
			if (usrid != null) {
				hql = "from UsersInfo as u where u.usrid=:usrid and u.isvalid=:isvalid";
				query = hsession.createQuery(hql);
				query.setBoolean("isvalid", true);
				query.setLong("usrid", usrid);
				user = (UsersInfo)query.list().get(0);
				info.setUsersInfo(user);
			}
			
//			typename = type+"_"+typename;//根据typename无法判断修改的那个字段
			info.setTypename(typename);
			//genreason 1代表信息变更 0代表计费变更
			if(genreason==1)
				info.setGenreason(true);
			else
				info.setGenreason(false);
			info.setOrivalue(oldVal);
			if(type.equals("usrname")){
				info.setNewvalue(usrname);
				info.setAffect(0.0);
			}else if(type.equals("usertype")){
				info.setNewvalue(usertype);
				info.setAffect(0.0);
			}else if(type.equals("contracttype")){
				info.setNewvalue(contracttype);
				info.setAffect(0.0);
			}else if(type.equals("contractver")){
				info.setNewvalue(contractver);
				info.setAffect(0.0);
			}else if(type.equals("contractpic")){
				info.setNewvalue(contractpic);
				info.setAffect(0.0);
			}else if(type.equals("phone")){
				info.setNewvalue(phone);
				info.setAffect(0.0);
			}else if(type.equals("area")){
				info.setNewvalue(area.toString());
				Double oldvalue = new Double(oldVal);
				Double affect = getFeeValue(area, user.getRealarea(),user.getFeearea(),user.getFeerate(),user.getFeetype())
						- getFeeValue(user.getArea(), user.getRealarea(),user.getFeearea(),user.getFeerate(),user.getFeetype());
				info.setAffect(affect);
			}else if(type.equals("realarea")){
				info.setNewvalue(realarea.toString());
				Double oldvalue = new Double(oldVal);
				Double affect = getFeeValue(user.getArea(),realarea,user.getFeearea(),user.getFeerate(),user.getFeetype())
						- getFeeValue(user.getArea(), user.getRealarea(),user.getFeearea(),user.getFeerate(),user.getFeetype());
				info.setAffect(affect);
			}else if(type.equals("feearea")){
				info.setNewvalue(feearea.toString());
				Double oldvalue = new Double(oldVal);
				Double affect = getFeeValue(user.getArea(),user.getRealarea(),feearea,user.getFeerate(),user.getFeetype())
						- getFeeValue(user.getArea(), user.getRealarea(),user.getFeearea(),user.getFeerate(),user.getFeetype());
				info.setAffect(affect);
			}else if(type.equals("feetype")){
				if(feetype.equals(Global.TYPEHEATSTATE)){//热计量
					feetype=feetype+"_"+heatbase+"_"+heatrate;
					info.setNewvalue(feetype);
					info.setAffect(0.0);
				}else{
					info.setNewvalue(feetype);
					Double affect = getFeeValue(user.getArea(),user.getRealarea(),user.getFeearea(),user.getFeerate(),feetype)
							- getFeeValue(user.getArea(), user.getRealarea(),user.getFeearea(),user.getFeerate(),user.getFeetype());
					info.setAffect(affect);
				}
			}else if(type.equals("upid")){
				info.setNewvalue(upid);
				Double oldrate = new Double(oldVal);
				Double affect = getFeeValue(user.getArea(), user.getRealarea(),user.getFeearea(),feerate,user.getFeetype())
						- getFeeValue(user.getArea(), user.getRealarea(),user.getFeearea(),user.getFeerate(),user.getFeetype());
				info.setAffect(affect);
			}else if(type.equals("heatstate")){
				info.setNewvalue(heatstate);
				info.setAffect(0.0);
			}else if(type.equals("discount")){
				info.setOrivalue(oldVal);
				info.setNewvalue(discount.toString());
				Double affect = getFeeValue(user.getArea(), user.getRealarea(),user.getFeearea(),user.getFeerate(),user.getFeetype())*discount;
				info.setAffect(affect);
			}else if(type.equals("reducefee")){
				Double affect = -reducefee;
				info.setOrivalue(oldVal);
				info.setNewvalue(affect.toString());
				info.setAffect(affect);
			}
			if(chgid!=null){
				DueCharge charge = new DueCharge();
				charge.setChgid(chgid);
				info.setDueCharge(charge);
				info.setIsaffect(true);
			}else{
				if(isaffect==null){
					info.setAffect(0.0);
					info.setIsaffect(false);
				}else
					info.setIsaffect(true);
			}
			
			if (chgid == null)
			{
				//信息变更申请次数
				
				StmdyDaily sd = getStmdyDaily(Global.nosecondformat.parse(Global.secondformat.format(Calendar.getInstance().getTime())),
						info.getUsersInfo().getProjectInfo(), sp.getStaffInfo(), info.getUsersInfo().getBuildingInfo(), hsession);
				sd.setImdyrqttimes((sd.getImdyrqttimes() == null ? 0 : sd.getImdyrqttimes()) + 1);
				hsession.saveOrUpdate(sd);
				
				StmdyRange sr = getStmdyRange(Global.nosecondformat.parse(Global.secondformat.format(Calendar.getInstance().getTime())),
						info.getUsersInfo().getProjectInfo(), sp.getStaffInfo(), info.getUsersInfo().getBuildingInfo(), hsession);
				sr.setImdyrqttimes((sr.getImdyrqttimes() == null ? 0 : sr.getImdyrqttimes()) + 1);
				hsession.saveOrUpdate(sr);
			}
			
			info.setStatus(Global.APPLY);
			info.setIsvalid(true);
			
			hsession.saveOrUpdate(info);
			transaction.commit();
			result = Global.SUCCESS;
		}catch(Exception e){
			e.printStackTrace();
			transaction.rollback();
			result = Global.FAILURE;
		}
		return result;
	}
	
	public StchgDaily getStchgDaily(Date stdate, ProjectInfo projectInfo, StaffInfo staffInfo, BuildingInfo buildingInfo, Session hsession)
	{
		Query query;
		if (staffInfo != null)
		{
			String hql = "from StchgDaily as st " +
					"where st.stdate=:stdate and st.staffInfo.stfid=:stfid and st.projectInfo.pjtid=:pjtid and st.buildingInfo.bldid=:bldid and st.isvalid=:isvalid";
			query = hsession.createQuery(hql);
			query.setBoolean("isvalid", true);
			query.setLong("stfid", staffInfo.getStfid());
			query.setLong("pjtid", projectInfo.getPjtid());
			query.setLong("bldid", buildingInfo.getBldid());
			query.setTimestamp("stdate", stdate);
			
		}
		else
		{
			String hql = "from StchgDaily as st " +
					"where st.stdate=:stdate and st.staffInfo.stfid=null and st.projectInfo.pjtid=:pjtid and st.buildingInfo.bldid=:bldid and st.isvalid=:isvalid";
			query = hsession.createQuery(hql);
			query.setBoolean("isvalid", true);
			query.setLong("pjtid", projectInfo.getPjtid());
			query.setLong("bldid", buildingInfo.getBldid());
			query.setTimestamp("stdate", stdate);
		}
		
		List list = query.list();
		
		if (list.size() > 0 )
		{
			return (StchgDaily)list.get(0);
		}
		else
		{
			StchgDaily sd = new StchgDaily();
			sd.setBuildingInfo(buildingInfo);
			sd.setProjectInfo(projectInfo);
			sd.setStdate(stdate);
			sd.setStaffInfo(staffInfo);
			return sd;
		}
	}
	
	public StmdyDaily getStmdyDaily(Date stdate, ProjectInfo projectInfo, StaffInfo staffInfo, BuildingInfo buildingInfo, Session hsession)
	{
		Query query;
		if (staffInfo != null)
		{
			String hql = "from StmdyDaily as st " +
					"where st.stdate=:stdate and st.staffInfo.stfid=:stfid and st.projectInfo.pjtid=:pjtid and st.buildingInfo.bldid=:bldid and st.isvalid=:isvalid";
			query = hsession.createQuery(hql);
			query.setBoolean("isvalid", true);
			query.setLong("stfid", staffInfo.getStfid());
			query.setLong("pjtid", projectInfo.getPjtid());
			query.setLong("bldid", buildingInfo.getBldid());
			query.setTimestamp("stdate", stdate);
			
		}
		else
		{
			String hql = "from StmdyDaily as st " +
					"where st.stdate=:stdate and st.staffInfo.stfid=null and st.projectInfo.pjtid=:pjtid and st.buildingInfo.bldid=:bldid and st.isvalid=:isvalid";
			query = hsession.createQuery(hql);
			query.setBoolean("isvalid", true);
			query.setLong("pjtid", projectInfo.getPjtid());
			query.setLong("bldid", buildingInfo.getBldid());
			query.setTimestamp("stdate", stdate);
		}
		
		List list = query.list();
		
		if (list.size() > 0 )
		{
			return (StmdyDaily)list.get(0);
		}
		else
		{
			StmdyDaily sd = new StmdyDaily();
			sd.setBuildingInfo(buildingInfo);
			sd.setProjectInfo(projectInfo);
			sd.setStdate(stdate);
			sd.setStaffInfo(staffInfo);
			return sd;
		}
	}
	
	public StmdyRange getStmdyRange(Date stdate, ProjectInfo projectInfo, StaffInfo staffInfo, BuildingInfo buildingInfo, Session hsession)
	{
		Query query;
		if (staffInfo != null)
		{
			String hql = "from StmdyRange as st " +
					"where st.stdate=:stdate and st.staffInfo.stfid=:stfid and st.projectInfo.pjtid=:pjtid and st.buildingInfo.bldid=:bldid and st.isvalid=:isvalid";
			query = hsession.createQuery(hql);
			query.setBoolean("isvalid", true);
			query.setLong("stfid", staffInfo.getStfid());
			query.setLong("pjtid", projectInfo.getPjtid());
			query.setLong("bldid", buildingInfo.getBldid());
			query.setTimestamp("stdate", stdate);
			
		}
		else
		{
			String hql = "from StmdyRange as st " +
					"where st.stdate=:stdate and st.staffInfo.stfid=null and st.projectInfo.pjtid=:pjtid and st.buildingInfo.bldid=:bldid and st.isvalid=:isvalid";
			query = hsession.createQuery(hql);
			query.setBoolean("isvalid", true);
			query.setLong("pjtid", projectInfo.getPjtid());
			query.setLong("bldid", buildingInfo.getBldid());
			query.setTimestamp("stdate", stdate);
		}
		
		List list = query.list();
		
		if (list.size() > 0 )
		{
			return (StmdyRange)list.get(0);
		}
		else
		{
			StmdyRange sd = new StmdyRange();
			sd.setBuildingInfo(buildingInfo);
			sd.setProjectInfo(projectInfo);
			sd.setStdate(stdate);
			sd.setStaffInfo(staffInfo);
			return sd;
		}
	}
	
	public void handleAuditFail(FeemodifyInfo info, Session hsession, StaffPermission sp) throws Exception
	{
		String hql;
		Query query;
		List list;
		
		StmdyDaily sd = getStmdyDaily(Global.nosecondformat.parse(Global.secondformat.format(Calendar.getInstance().getTime())),
				info.getUsersInfo().getProjectInfo(), sp.getStaffInfo(), info.getUsersInfo().getBuildingInfo(), hsession);
		
		StmdyRange sr = getStmdyRange(Global.nosecondformat.parse(Global.secondformat.format(Calendar.getInstance().getTime())),
				info.getUsersInfo().getProjectInfo(), sp.getStaffInfo(), info.getUsersInfo().getBuildingInfo(), hsession);
		
		
		if (info.getStatus() == Global.EXENOCHECK)
		{
			//费用变更审核次数
			sd.setFmdyaudtimes((sd.getFmdyaudtimes() == null ? 0 : sd.getFmdyaudtimes()) + 1);
			sr.setFmdyaudtimes((sr.getFmdyaudtimes() == null ? 0 : sr.getFmdyaudtimes()) + 1);
		}
		
		if (info.getStatus() == Global.APPLY)
		{
			//信息变更审核次数
			sd.setFmdyaudtimes((sd.getFmdyaudtimes() == null ? 0 : sd.getFmdyaudtimes()) + 1);
			sr.setFmdyaudtimes((sr.getFmdyaudtimes() == null ? 0 : sr.getFmdyaudtimes()) + 1);
		}
		
		hsession.saveOrUpdate(sd);
		hsession.saveOrUpdate(sr);
		
		//审核不通过
		info.setStatus(Global.NOTPASS);
		info.setStaffInfoByAudid(sp.getStaffInfo());
		info.setAudittime(Calendar.getInstance().getTime());
	}
	
	public void handleFeeMdyFinalAuditSuccess(FeemodifyInfo info, Session hsession, StaffPermission sp) throws Exception
	{
		Query query;
		List list;
		String hql;
		
		query = hsession.createQuery("from HeatsysSetting");
		HeatsysSetting heatsysSetting = (HeatsysSetting)query.list().get(0);
		
		StmdyDaily sdRequest = getStmdyDaily(Global.nosecondformat.parse(Global.secondformat.format(Calendar.getInstance().getTime())),
				info.getUsersInfo().getProjectInfo(), info.getStaffInfoByRqtid(), info.getUsersInfo().getBuildingInfo(), hsession);
		
		StmdyDaily sdAudit = getStmdyDaily(Global.nosecondformat.parse(Global.secondformat.format(Calendar.getInstance().getTime())),
				info.getUsersInfo().getProjectInfo(), sp.getStaffInfo(), info.getUsersInfo().getBuildingInfo(), hsession);
		
		StmdyRange srRequest = getStmdyRange(Global.nosecondformat.parse(Global.secondformat.format(Calendar.getInstance().getTime())),
				info.getUsersInfo().getProjectInfo(), info.getStaffInfoByRqtid(), info.getUsersInfo().getBuildingInfo(), hsession);
		
		StmdyRange srAudit = getStmdyRange(Global.nosecondformat.parse(Global.secondformat.format(Calendar.getInstance().getTime())),
				info.getUsersInfo().getProjectInfo(), sp.getStaffInfo(), info.getUsersInfo().getBuildingInfo(), hsession);
		
		
		//已审核减免
		if (info.getTypename().equals("减免额"))
		{
			sdAudit.setRemitaudmoney((sdAudit.getRemitaudmoney() == null ? 0 : sdAudit.getRemitaudmoney()) + info.getAffect());
			srAudit.setRemitaudmoney((srAudit.getRemitaudmoney() == null ? 0 : srAudit.getRemitaudmoney()) + info.getAffect());
			sdAudit.setRemitmoney((sdAudit.getRemitmoney() == null ? 0 : sdAudit.getRemitmoney()) - info.getAffect());
		}
		
		query = hsession.createQuery("from TaskassignInfo t where t.isvalid=:isvalid and t.dealname=:dealname and t.usersInfo.usrid=:usrid");
		query.setBoolean("isvalid", true);
		query.setString("dealname", heatsysSetting.getCurdealname());
		query.setLong("usrid", info.getUsersInfo().getUsrid());
		
		list = query.list();
		
		if (info.getDueCharge() != null)
		{
			if (list.size() == 0)
			{
				StchgDaily std = getStchgDaily(Global.nosecondformat.parse(Global.secondformat.format(Calendar.getInstance().getTime())),
						info.getUsersInfo().getProjectInfo(), null, info.getUsersInfo().getBuildingInfo(), hsession);
				
				if (info.getDueCharge().getDealname().equals(heatsysSetting.getCurdealname()))
				{
					std.setDuemoney((std.getDuemoney() == null ? 0 : std.getDuemoney()) + info.getAffect());
				}
				else
				{
					std.setDuemoney((std.getDuemoney() == null ? 0 : std.getDuemoney()) + info.getAffect());
				}
				hsession.saveOrUpdate(std);
			}
			else
			{
				TaskassignInfo taskAssignInfo = (TaskassignInfo)list.get(0);
				if (info.getDueCharge().getDealname().equals(heatsysSetting.getCurdealname()))
				{
					StchgDaily std = getStchgDaily(Global.nosecondformat.parse(Global.secondformat.format(Calendar.getInstance().getTime())),
							info.getUsersInfo().getProjectInfo(), taskAssignInfo.getStaffInfoByCurstaffid(), info.getUsersInfo().getBuildingInfo(), hsession);
					std.setDuemoneylast((std.getDuemoneylast() == null ? 0 : std.getDuemoneylast()) + info.getAffect());
					hsession.saveOrUpdate(std);
				}
				else
				{
					StchgDaily std = getStchgDaily(Global.nosecondformat.parse(Global.secondformat.format(Calendar.getInstance().getTime())),
							info.getUsersInfo().getProjectInfo(), taskAssignInfo.getStaffInfoByLaststaffid(), info.getUsersInfo().getBuildingInfo(), hsession);		
					std.setDuemoneylast((std.getDuemoneylast() == null ? 0 : std.getDuemoneylast()) + info.getAffect());
					hsession.saveOrUpdate(std);
				}
			}
		}
		else
			throw new Exception("no duecharge is related to this mdy info!");
		
		
		//已审核增加金额
		if (info.getAffect() > 0)
		{
			sdAudit.setMdyincaudmoney((sdAudit.getMdyincaudmoney() == null ? 0 : sdAudit.getMdyincaudmoney()) + info.getAffect());
			srAudit.setMdyincaudmoney((srAudit.getMdyincaudmoney() == null ? 0 : srAudit.getMdyincaudmoney()) + info.getAffect());
			sdAudit.setMdyincmoney((sdAudit.getMdyincmoney() == null ? 0 : sdAudit.getMdyincmoney()) - info.getAffect());
		}
		
		//已审核增加金额
		if (info.getAffect() < 0)
		{
			sdAudit.setMdydecaudmoney((sdAudit.getMdydecaudmoney() == null ? 0 : sdAudit.getMdydecaudmoney()) + info.getAffect());
			srAudit.setMdydecaudmoney((srAudit.getMdydecaudmoney() == null ? 0 : srAudit.getMdydecaudmoney()) + info.getAffect());
			sdAudit.setMdydecmoney((sdAudit.getMdydecmoney() == null ? 0 : sdAudit.getMdydecmoney()) - info.getAffect());
		}
		
		//费用变更审核次数
		sdAudit.setFmdyaudtimes((sdAudit.getFmdyaudtimes() == null ? 0 : sdAudit.getFmdyaudtimes()) + 1);
		srAudit.setFmdyaudtimes((srAudit.getFmdyaudtimes() == null ? 0 : srAudit.getFmdyaudtimes()) + 1);
		
		//费用变更被审核次数
		sdRequest.setFmdyaudbytimes((sdRequest.getFmdyaudbytimes() == null ? 0 : sdRequest.getFmdyaudbytimes()) + 1);
		srRequest.setFmdyaudbytimes((srRequest.getFmdyaudbytimes() == null ? 0 : srRequest.getFmdyaudbytimes()) + 1);
		sdRequest.setFmdyrqttimes((sdRequest.getFmdyrqttimes() == null ? 0 : sdRequest.getFmdyrqttimes()) - 1);
		
		//在实施后，审核
		hsession.saveOrUpdate(sdAudit);
		hsession.saveOrUpdate(sdRequest);
		hsession.saveOrUpdate(srRequest);
		hsession.saveOrUpdate(srAudit);
		info.setStatus(Global.EXE);
		updateCheckedTable(hsession,info,Global.EXENOCHECK2EXE);
		info.setExecaudittime(Calendar.getInstance().getTime());
		info.setStaffInfoByExecaudid(sp.getStaffInfo());
	}
	
	public void handleFeeMdyRequestAuditSuccess(FeemodifyInfo info, Session hsession, StaffPermission sp) throws Exception
	{
		Query query;
		List list;
		String hql;
		
		query = hsession.createQuery("from HeatsysSetting");
		HeatsysSetting heatsysSetting = (HeatsysSetting)query.list().get(0);
		
		//genreason==0
		info.setStatus(Global.EXENOCHECK);
		updateCheckedTable(hsession,info,Global.APPLY2EXENOCHECK);
		
		query = hsession.createQuery("from TaskassignInfo t where t.isvalid=:isvalid and t.dealname=:dealname and t.usersInfo.usrid=:usrid");
		query.setBoolean("isvalid", true);
		query.setString("dealname", heatsysSetting.getCurdealname());
		query.setLong("usrid", info.getUsersInfo().getUsrid());
		
		list = query.list();
		TaskassignInfo taskAssignInfo = (TaskassignInfo)list.get(0);
		
		
		//总减免金额
		if (info.getTypename().equals("减免额"))
		{
			StmdyDaily sd = getStmdyDaily(Global.nosecondformat.parse(Global.secondformat.format(Calendar.getInstance().getTime())),
					info.getUsersInfo().getProjectInfo(), sp.getStaffInfo(), info.getUsersInfo().getBuildingInfo(), hsession);
			
			StmdyRange sr = getStmdyRange(Global.nosecondformat.parse(Global.secondformat.format(Calendar.getInstance().getTime())),
					info.getUsersInfo().getProjectInfo(), sp.getStaffInfo(), info.getUsersInfo().getBuildingInfo(), hsession);
			
			sr.setRemitmoney((sd.getRemitmoney() == null ? 0 : sr.getRemitmoney()) + info.getAffect());
			sd.setRemitmoney((sd.getRemitmoney() == null ? 0 : sd.getRemitmoney()) + info.getAffect());
			hsession.saveOrUpdate(sd);
			hsession.saveOrUpdate(sr);
			
			if (taskAssignInfo == null)
			{
				StchgDaily std = getStchgDaily(Global.nosecondformat.parse(Global.secondformat.format(Calendar.getInstance().getTime())),
						info.getUsersInfo().getProjectInfo(), null, info.getUsersInfo().getBuildingInfo(), hsession);
				
				if (info.getDueCharge().getDealname().equals(heatsysSetting.getCurdealname()))
				{
					
					std.setImdyrqtmoney((std.getImdyrqtmoney() == null ? 0 : std.getImdyrqtmoney()) + info.getAffect());
				}
				else
				{
					std.setImdyrqtmoney((std.getImdyrqtmoney() == null ? 0 : std.getImdyrqtmoney()) + info.getAffect());
				}	
				hsession.saveOrUpdate(std);
			}
			else
			{
				if (info.getDueCharge().getDealname().equals(heatsysSetting.getCurdealname()))
				{
					StchgDaily std = getStchgDaily(Global.nosecondformat.parse(Global.secondformat.format(Calendar.getInstance().getTime())),
							info.getUsersInfo().getProjectInfo(), taskAssignInfo.getStaffInfoByCurstaffid(), info.getUsersInfo().getBuildingInfo(), hsession);
					std.setRemitmoney((std.getRemitmoney() == null ? 0 : std.getRemitmoney()) + info.getAffect());
					hsession.saveOrUpdate(std);
				}
				else
				{
					StchgDaily std = getStchgDaily(Global.nosecondformat.parse(Global.secondformat.format(Calendar.getInstance().getTime())),
							info.getUsersInfo().getProjectInfo(), taskAssignInfo.getStaffInfoByLaststaffid(), info.getUsersInfo().getBuildingInfo(), hsession);		
					std.setRemitmoney((std.getRemitmoney() == null ? 0 : std.getRemitmoney()) + info.getAffect());
					hsession.saveOrUpdate(std);
				}
			}
		}
		
		info.setExectime(Calendar.getInstance().getTime());
		info.setStaffInfoByExecid(sp.getStaffInfo());
	}
	
	public void handleInfoMdyRequestAuditSuccess(FeemodifyInfo info, Session hsession, StaffPermission sp) throws Exception
	{
		Query query;
		List list;
		String hql;
		
		query = hsession.createQuery("from HeatsysSetting");
		HeatsysSetting heatsysSetting = (HeatsysSetting)query.list().get(0);
		StmdyDaily sdRequest = getStmdyDaily(Global.nosecondformat.parse(Global.secondformat.format(Calendar.getInstance().getTime())),
				info.getUsersInfo().getProjectInfo(), info.getStaffInfoByRqtid(), info.getUsersInfo().getBuildingInfo(), hsession);
		
		StmdyDaily sdAudit = getStmdyDaily(Global.nosecondformat.parse(Global.secondformat.format(Calendar.getInstance().getTime())),
				info.getUsersInfo().getProjectInfo(), sp.getStaffInfo(), info.getUsersInfo().getBuildingInfo(), hsession);
		
		StmdyRange srRequest = getStmdyRange(Global.nosecondformat.parse(Global.secondformat.format(Calendar.getInstance().getTime())),
				info.getUsersInfo().getProjectInfo(), info.getStaffInfoByRqtid(), info.getUsersInfo().getBuildingInfo(), hsession);
		
		StmdyRange srAudit = getStmdyRange(Global.nosecondformat.parse(Global.secondformat.format(Calendar.getInstance().getTime())),
				info.getUsersInfo().getProjectInfo(), sp.getStaffInfo(), info.getUsersInfo().getBuildingInfo(), hsession);
		
		
		//genreason==1
		if(info.getIsaffect())
		{
			
			query = hsession.createQuery("from TaskassignInfo t where t.isvalid=:isvalid and t.dealname=:dealname and t.usersInfo.usrid=:usrid");
			query.setBoolean("isvalid", true);
			query.setString("dealname", heatsysSetting.getCurdealname());
			query.setLong("usrid", info.getUsersInfo().getUsrid());
			
			list = query.list();
			
			
			if (info.getDueCharge() != null)
			{
				if (list.size() == 0)
				{
					StchgDaily std = getStchgDaily(Global.nosecondformat.parse(Global.secondformat.format(Calendar.getInstance().getTime())),
							info.getUsersInfo().getProjectInfo(), null, info.getUsersInfo().getBuildingInfo(), hsession);
					std.setImdyrqtmoney((std.getImdyrqtmoney() == null ? 0 : std.getImdyrqtmoney()) + info.getAffect());
					hsession.saveOrUpdate(std);
				}
				else
				{
					TaskassignInfo taskAssignInfo = (TaskassignInfo)list.get(0);	
					if (info.getDueCharge().getDealname().equals(heatsysSetting.getCurdealname()))
					{
						StchgDaily std = getStchgDaily(Global.nosecondformat.parse(Global.secondformat.format(Calendar.getInstance().getTime())),
								info.getUsersInfo().getProjectInfo(), taskAssignInfo.getStaffInfoByCurstaffid(), info.getUsersInfo().getBuildingInfo(), hsession);
						std.setImdyrqtmoney((std.getImdyrqtmoney() == null ? 0 : std.getImdyrqtmoney()) + info.getAffect());
						hsession.saveOrUpdate(std);
					}
					else
					{
						StchgDaily std = getStchgDaily(Global.nosecondformat.parse(Global.secondformat.format(Calendar.getInstance().getTime())),
								info.getUsersInfo().getProjectInfo(), taskAssignInfo.getStaffInfoByLaststaffid(), info.getUsersInfo().getBuildingInfo(), hsession);		
						std.setImdyrqtmoney((std.getImdyrqtmoney() == null ? 0 : std.getImdyrqtmoney()) + info.getAffect());
						hsession.saveOrUpdate(std);
					}
				}
				
			}
			else
			{
				StchgDaily std = getStchgDaily(Global.nosecondformat.parse(Global.secondformat.format(Calendar.getInstance().getTime())),
						info.getUsersInfo().getProjectInfo(), null, info.getUsersInfo().getBuildingInfo(), hsession);
				
				std.setImdyrqtmoney((std.getImdyrqtmoney() == null ? 0 : std.getImdyrqtmoney()) + info.getAffect());
				hsession.saveOrUpdate(std);
			}
			
			//信息变更涉及金额
			sdAudit.setImdymoney((sdAudit.getImdymoney() == null ? 0 : sdAudit.getImdymoney()) + info.getAffect());
			srAudit.setImdymoney((srAudit.getImdymoney() == null ? 0 : srAudit.getImdymoney()) + info.getAffect());
			
			
			//已审核增加金额
			if (info.getAffect() > 0)
			{
				sdAudit.setMdyincmoney((sdAudit.getMdyincmoney() == null ? 0 : sdAudit.getMdyincmoney()) + info.getAffect());
				srAudit.setMdyincmoney((srAudit.getMdyincmoney() == null ? 0 : srAudit.getMdyincmoney()) + info.getAffect());
			}
			
			//已审核增加金额
			if (info.getAffect() < 0)
			{
				sdAudit.setMdydecmoney((sdAudit.getMdydecmoney() == null ? 0 : sdAudit.getMdydecmoney()) + info.getAffect());
				srAudit.setMdydecmoney((srAudit.getMdydecmoney() == null ? 0 : srAudit.getMdydecmoney()) + info.getAffect());
			}
			info.setStatus(Global.CHECKNOEXE);
			updateCheckedTable(hsession,info,Global.APPLY2CHECKNOEXE);
		}
		else
		{
			info.setStatus(Global.CHECK);
			updateCheckedTable(hsession,info,Global.APPLY2CHECK);
		}
		
		//信息变更被审核次数
		sdRequest.setImdyaudbytimes((sdRequest.getImdyaudbytimes() == null ? 0 : sdRequest.getImdyaudbytimes()) + 1);
		sdRequest.setImdyrqttimes((sdRequest.getImdyrqttimes() == null ? 0 : sdRequest.getImdyrqttimes()) - 1);
		srRequest.setImdyaudbytimes((srRequest.getImdyaudbytimes() == null ? 0 : srRequest.getImdyaudbytimes()) + 1);
		
		//信息变更被审核次数
		sdAudit.setImdyaudtimes((sdAudit.getImdyaudtimes() == null ? 0 : sdAudit.getImdyaudtimes()) + 1);
		srAudit.setImdyaudtimes((srAudit.getImdyaudtimes() == null ? 0 : srAudit.getImdyaudtimes()) + 1);
		
		hsession.saveOrUpdate(sdAudit);
		hsession.saveOrUpdate(sdRequest);
		info.setAudittime(Calendar.getInstance().getTime());
		info.setStaffInfoByAudid(sp.getStaffInfo());
	}
	
	//改动后的审核变更CheakChangeWithType
	public Result mdyAuditSuccess(Long id,Long checkStfid,String chkname,Boolean isSuccess,Long type,HttpSession session){
		Result result = null;
		ServletContext servletContext = session.getServletContext();    
		ApplicationContext ctx = WebApplicationContextUtils.getWebApplicationContext(servletContext);
		SessionFactory sessionFactory = (SessionFactory) ctx.getBean("sessionFactory");
		Session hsession = SessionFactoryUtils.getSession(sessionFactory, true);
		Transaction transaction = hsession.beginTransaction();
		String hql = "";
		Query query;
		List list;
		StaffPermission sp = (StaffPermission)session.getAttribute("staffinfo");
		try
		{
			FeemodifyInfo info = this.feemodifyInfoDAO.findById(id);
			Boolean genreason = info.getGenreason();
			if(isSuccess==false)
			{	
				handleAuditFail(info, hsession, sp);
			}
			else
			{
				if(type==1)
				{
					//在申请时，审核
					if(genreason)
					{
						handleInfoMdyRequestAuditSuccess(info, hsession, sp);
					}
					else
					{
						handleFeeMdyRequestAuditSuccess(info, hsession, sp);
					}
				}
				
				if(type==3)
				{
					handleFeeMdyFinalAuditSuccess(info, hsession, sp);
				}
			}	
			info.setStaffInfoByChkid(sp.getStaffInfo());
			info.setChkname(chkname);
			
			if (info.getStatus().equals(Global.CHECK) || info.getStatus().equals(Global.CHECKNOEXE))
			{
				UsersInfo user = info.getUsersInfo();
				String newvalue = info.getNewvalue();
				String typename = info.getTypename();
				updateUser(info,user,typename,newvalue);
				info.setUsersInfo(user);
				hsession.saveOrUpdate(user);
			}
			
			if (info.getStatus().equals(Global.EXE))
			{
				UsersInfo user = info.getUsersInfo();
				user.setNeedmoney(user.getNeedmoney() + info.getAffect());
				hsession.saveOrUpdate(user);
			}
			
			hsession.saveOrUpdate(info);
			transaction.commit();
			result = Global.SUCCESS;
		}
		catch(Exception e)
		{
			e.printStackTrace();
			transaction.rollback();
			result = Global.FAILURE;
		}
		return result;
	}
		
	
	//审核变更
	public Result CheakChange(Long id,Long checkid,String chkname,HttpSession session){
		Result result = null;
		ServletContext servletContext = session.getServletContext();    
        ApplicationContext ctx = WebApplicationContextUtils.getWebApplicationContext(servletContext);
        SessionFactory sessionFactory = (SessionFactory) ctx.getBean("sessionFactory");
        Session hsession = SessionFactoryUtils.getSession(sessionFactory, true);
        Transaction transaction = hsession.beginTransaction();
		try{
			FeemodifyInfo info = this.feemodifyInfoDAO.findById(id);
			Boolean genreason = info.getGenreason();
			if(genreason){//genreason==1
				if(info.getIsaffect()){
					info.setStatus(Global.CHECKNOEXE);
				}else{
					info.setStatus(Global.CHECK);
				}
			}else{//genreason==0
				info.setStatus(Global.EXENOCHECK);
			}
			StaffInfo staff = new StaffInfo();
			staff.setStfid(checkid);
			info.setStaffInfoByChkid(staff);
			info.setChkname(chkname);
			
			UsersInfo user = this.userInfoDAO.findById(info.getUsersInfo().getUsrid());
			String newvalue = info.getNewvalue();
			String typename = info.getTypename();
			updateUser(info,user,typename,newvalue);
			info.setUsersInfo(user);
			hsession.saveOrUpdate(user);
			hsession.saveOrUpdate(info);
			transaction.commit();
			result = Global.SUCCESS;
		}catch(Exception e){
			e.printStackTrace();
			transaction.rollback();
			result = Global.FAILURE;
		}
		return result;
	}
	
	//执行变更
	public Result ExeChange(Long id, Long chgid,HttpSession session){
		Result result = null;
		ServletContext servletContext = session.getServletContext();    
        ApplicationContext ctx = WebApplicationContextUtils.getWebApplicationContext(servletContext);
        SessionFactory sessionFactory = (SessionFactory) ctx.getBean("sessionFactory");
        Session hsession = SessionFactoryUtils.getSession(sessionFactory, true);
        Transaction transaction = hsession.beginTransaction();
        String hql = "";
        Query query;
        StaffPermission sp = (StaffPermission)session.getAttribute("staffinfo");
		try{
			query = hsession.createQuery("from HeatsysSetting");
			HeatsysSetting heatsysSetting = (HeatsysSetting)query.list().get(0);
			FeemodifyInfo info = this.feemodifyInfoDAO.findById(id);
			query = hsession.createQuery("from DueCharge d where d.chgid=" + chgid);
			DueCharge dueCharge = (DueCharge)query.list().get(0);
			
			query = hsession.createQuery("from TaskassignInfo t where t.isvalid=:isvalid and t.dealname=:dealname and t.usersInfo.usrid=:usrid");
			query.setBoolean("isvalid", true);
			query.setString("dealname", heatsysSetting.getCurdealname());
			query.setLong("usrid", info.getUsersInfo().getUsrid());
			
			List list = query.list();
			
			
			if (info.getDueCharge() == null)
			{
				if (list.size() != 0)
				{
					TaskassignInfo taskAssignInfo = (TaskassignInfo)list.get(0);
					
					StchgDaily sdNoAssign = getStchgDaily(Global.nosecondformat.parse(Global.secondformat.format(Calendar.getInstance().getTime())),
							info.getUsersInfo().getProjectInfo(), null, info.getUsersInfo().getBuildingInfo(), hsession);
	    			StchgDaily sdCurstf = getStchgDaily(Global.nosecondformat.parse(Global.secondformat.format(Calendar.getInstance().getTime())),
	    					info.getUsersInfo().getProjectInfo(), taskAssignInfo.getStaffInfoByCurstaffid(), info.getUsersInfo().getBuildingInfo(), hsession);
	    			StchgDaily sdLaststf;
	    			if (taskAssignInfo.getStaffInfoByCurstaffid().getStfid().equals(taskAssignInfo.getStaffInfoByLaststaffid().getStfid()))
	    				sdLaststf = sdCurstf;
	    			else
	    				sdLaststf = getStchgDaily(Global.nosecondformat.parse(Global.secondformat.format(Calendar.getInstance().getTime())),
	    						info.getUsersInfo().getProjectInfo(), new StaffInfo(taskAssignInfo.getStaffInfoByLaststaffid().getStfid()), info.getUsersInfo().getBuildingInfo(), hsession);
					
					sdNoAssign.setImdyrqtmoney((sdNoAssign.getImdyrqtmoney() == null ? 0 : sdNoAssign.getImdyrqtmoney()) - info.getAffect());
					hsession.saveOrUpdate(sdNoAssign);
					if (dueCharge.getDealname().equals(heatsysSetting.getCurdealname()))
					{
						sdCurstf.setImdyrqtmoney((sdCurstf.getImdyrqtmoney() == null ? 0 : sdCurstf.getImdyrqtmoney()) + info.getAffect());
						hsession.saveOrUpdate(sdCurstf);
					}
					else
					{
						sdLaststf.setImdyrqtmoney((sdLaststf.getImdyrqtmoney() == null ? 0 : sdLaststf.getImdyrqtmoney()) + info.getAffect());
						hsession.saveOrUpdate(sdLaststf);
					}		
				}
			}
			else
				throw new Exception("mdy info has connected!");
			
			
			info.setStatus(Global.EXENOCHECK);
			updateCheckedTable(hsession,info,Global.CHECKNOEXE2EXENOCHECK);
			UsersInfo user = info.getUsersInfo();
			String newvalue = info.getNewvalue();
			String typename = info.getTypename();
			updateUser(info,user,typename,newvalue);//直接实施,无需手动执行
			info.setDueCharge(dueCharge);
			hsession.saveOrUpdate(user);
			
			query = hsession.createQuery("from TaskassignInfo t where t.isvalid=:isvalid and t.dealname=:dealname and t.usersInfo.usrid=:usrid");
			
			
			StmdyDaily sd = getStmdyDaily(Global.nosecondformat.parse(Global.secondformat.format(Calendar.getInstance().getTime())),
					info.getUsersInfo().getProjectInfo(), info.getStaffInfoByRqtid(), info.getUsersInfo().getBuildingInfo(), hsession);
			
			StmdyRange sr = getStmdyRange(Global.nosecondformat.parse(Global.secondformat.format(Calendar.getInstance().getTime())),
					info.getUsersInfo().getProjectInfo(), info.getStaffInfoByRqtid(), info.getUsersInfo().getBuildingInfo(), hsession);
			
			sr.setFmdyrqttimes((sr.getFmdyrqttimes() == null ? 0 : sr.getFmdyrqttimes()) + 1);
			sd.setFmdyrqttimes((sd.getFmdyrqttimes() == null ? 0 : sd.getFmdyrqttimes()) + 1);
			hsession.saveOrUpdate(sd);
			hsession.saveOrUpdate(sr);

			info.setExectime(Calendar.getInstance().getTime());
			info.setStaffInfoByExecid(sp.getStaffInfo());
			
			hsession.saveOrUpdate(info);
			transaction.commit();
			result = Global.SUCCESS;
		}catch(Exception e){
			e.printStackTrace();
			transaction.rollback();
			result = Global.FAILURE;
		}
		return result;
	}

	//区分修改内容
	public void updateUser(FeemodifyInfo info,UsersInfo user,String type,String newvalue){
		if(type.equals(Global.USERNAME)){
			user.setUsrname(newvalue);
		}else if(type.equals(Global.USERTYPE)){
			user.setUsertype(newvalue);
		}else if(type.equals(Global.CONTRACTTYPE)){
			user.setContracttype(newvalue);
		}else if(type.equals(Global.CONTRACTVER)){
			user.setContractver(newvalue);
		}else if(type.equals(Global.CONTRACTPIC)){
			String tempPath = user.getContractpic();
			if (tempPath != null && !tempPath.equals("")) {
				this.fileUploader.deleteFile(tempPath);
			}
			user.setContractpic(newvalue);
		}else if(type.equals(Global.PHONE)){
			user.setPhone(newvalue);
		}
		
		if(type.equals(Global.AREA)){
			user.setArea(new Double(newvalue));
		}else if(type.equals(Global.REALAREA)){
			user.setRealarea(new Double(newvalue));
		}else if(type.equals(Global.FEEAREA)){
			user.setFeearea(new Double(newvalue));
		}else if(type.equals(Global.FEETYPE)){
			String[] content = newvalue.split("_");//在type为heatstate且heatstate值为热计量费率时，值为“热计量基数_10_10”
			if(content[0].equals(Global.TYPEHEATSTATE)){//热计量
				user.setFeetype(content[0]);
				user.setHeatbase(new Double(content[1]));
				user.setHeatrate(new Double(content[2]));
			}else{
				user.setFeetype(newvalue);
			}
			user.setFeetype(newvalue);
		}else if(type.equals(Global.UPID)){
			String[] content = newvalue.split("_");
			if(content[0]==null){
				UnitPrice unit = new UnitPrice();
				unit.setUpid(new Long(content[0]));
				user.setUnitPrice(unit);
			}
			user.setFeerate(new Double(content[1]));
		}else if(type.equals(Global.HEATSTATE)){
			user.setHeatstate(newvalue);
		}else if(type.equals(Global.DISCOUNT)){
			user.setDiscount(new Double(newvalue));
		}else if(type.equals(Global.REDUCEFEE)){
			user.setReducefee(new Double(newvalue));
		}	
	}
	
	public Double getFeeValue(Double area,Double realarea,Double feearea,Double feerate,String feetype){
		if (feetype == null)
			return 0.0;
		if(feetype.equals(Global.TYPEAREA))
			return area*feerate;
		else if(feetype.equals(Global.TYPEREALAREA))
			return realarea*feerate;
		else if(feetype.equals(Global.TYPEFEEAREA))
			return feearea*feerate;
		else if(feetype.equals(Global.TYPEHEATSTATE))
			return 0.0;//热计量不改变影响额
		else
			return 0.0;
	}
	public void updateCheckedTable(Session hsession,FeemodifyInfo info,String desp){
		CheckedInfo checked = new CheckedInfo();
		checked.setFeemodifyInfo(info);
		checked.setDesp(desp);
		checked.setIsreaded(false);
		checked.setIsvalid(true);
		hsession.saveOrUpdate(checked);
	}
	// new function 这三项的查询条件： 修改类型、申请日期（下拉筛选，时间区间），关键字模糊查询
	public Result searchWithType(String typename,String starttime,String endtime,String queryStr,String type,Boolean isall,Integer start,Integer limit,HttpSession session){
		ServletContext servletContext = session.getServletContext();    
        ApplicationContext ctx = WebApplicationContextUtils.getWebApplicationContext(servletContext);
        SessionFactory sessionFactory = (SessionFactory) ctx.getBean("sessionFactory");
        Session hsession = SessionFactoryUtils.getSession(sessionFactory, true);
        Result reslt = new Result();
        try{
        	if(typename==null||typename.equals("")||
				starttime==null||starttime.equals("")||
				endtime==null||endtime.equals("")||
				queryStr==null||queryStr.equals("")||
				type==null||type.equals("")){
				reslt.setSuccess(false);
				reslt.setMessage(Global.PARAMERROR);
				return reslt;
			}
			String conditions = " and fi.mdytime>=:starttime and fi.mdytime<=:endtime and fi.typename=:typename and " +
					"(fi.rqtname like '%"+queryStr+"%' or " +
					"fi.chkname like '%"+queryStr+"%' or " +
					"fi.orivalue like '%"+queryStr+"%' or " +
					"fi.newvalue like '%"+queryStr+"%' or " +
					"fi.dealname like '%"+queryStr+"%') ";
			if(isall==false){
				StaffPermission sp = (StaffPermission)session.getAttribute("staffinfo");
				Long stfid = sp.getStaffInfo().getStfid();
				conditions = " and fi.staffInfoByRqtid.stfid = "+stfid;
			}
			Query query = null;
			Query countQuery = null;
			if(!type.equals(Global.ALL)){
				query = hsession.createQuery("from FeemodifyInfo fi join fi.usersInfo ui where ui.isvalid=:isvalid and fi.isvalid=:isvalid and fi.status=:status "+conditions+" order by fi.feemdyid desc");
				query.setString("status", type);
				countQuery = hsession.createQuery("select count(fi) from FeemodifyInfo fi join fi.usersInfo ui where ui.isvalid=:isvalid and fi.isvalid=:isvalid and fi.status=:status "+conditions+" order by fi.feemdyid desc");
				countQuery.setString("status", type);
			}else{
				query = hsession.createQuery("from FeemodifyInfo fi join fi.usersInfo ui where ui.isvalid=:isvalid and fi.isvalid=:isvalid "+conditions+" order by fi.feemdyid desc");
				countQuery = hsession.createQuery("select count(fi) from FeemodifyInfo fi join fi.usersInfo ui where ui.isvalid=:isvalid and fi.isvalid=:isvalid and fi.status=:status "+conditions+" order by fi.feemdyid desc");
			}
			query.setBoolean("isvalid", true);
			query.setDate("starttime", Global.secondformat.parse(starttime));
			query.setDate("endtime", Global.secondformat.parse(endtime));
			query.setString("typename",typename);
			query.setFirstResult(start);
			query.setMaxResults(limit);
			countQuery.setBoolean("isvalid", true);
			countQuery.setDate("starttime", Global.secondformat.parse(starttime));
			countQuery.setDate("endtime", Global.secondformat.parse(endtime));
			countQuery.setString("typename",typename);
			List count = countQuery.list();
			reslt.setTotalProperty((Long)count.get(0));
			List feelist = query.list();
			List data = new ArrayList();
			Long genReason = null;
			for(int i=0;i<feelist.size();++i){
				Object[] info = (Object[]) feelist.get(i);
				FeemodifyInfo inf = (FeemodifyInfo)info[0];
				UsersInfo user = (UsersInfo)info[1];
				if(inf.getGenreason())
					genReason = (long)1;
				else 
					genReason = (long)0;
				if(inf.getStaffInfoByChkid()==null)//当未审核时，审核人为空
					data.add(new ChangeInfo(inf.getFeemdyid(),Global.secondformat.format(inf.getMdytime()),inf.getStaffInfoByRqtid().getStfid(),inf.getRqtname(),
						null,inf.getChkname(),inf.getTypename(),inf.getOrivalue(),
						inf.getNewvalue(),inf.getAffect(),inf.getStatus(),user.getUsrid(),genReason));
				else
					data.add(new ChangeInfo(inf.getFeemdyid(),Global.secondformat.format(inf.getMdytime()),inf.getStaffInfoByRqtid().getStfid(),inf.getRqtname(),
						inf.getStaffInfoByChkid().getStfid(),inf.getChkname(),inf.getTypename(),inf.getOrivalue(),
						inf.getNewvalue(),inf.getAffect(),inf.getStatus(),user.getUsrid(),genReason));
			}		
			reslt.setData(data);
        }catch(Exception e){
        	e.printStackTrace();
        	reslt.setSuccess(false);
        	reslt.setMessage(Global.FAILURESTR);
        }
        return reslt;
	}
	
	
	//历史变更记录： 申请人、修改类型、审核人、审核日期（下拉筛选，时间区间）、申请日期（下拉筛选，时间区间）、关键字模糊查询
	//new function
	public Result searchFeeModifyList(
			String rqtname,String typename,String chkname,String starttime,String endtime,String queryStr,Boolean isall,
			Integer start,Integer limit,HttpSession session){
		ServletContext servletContext = session.getServletContext();    
        ApplicationContext ctx = WebApplicationContextUtils.getWebApplicationContext(servletContext);
        SessionFactory sessionFactory = (SessionFactory) ctx.getBean("sessionFactory");
        Session hsession = SessionFactoryUtils.getSession(sessionFactory, true);
        Result reslt = new Result();
        try{
        	if(rqtname==null||rqtname.equals("")||
        	   chkname==null||chkname.equals("")||
        	 starttime==null||starttime.equals("")||
			   endtime==null||endtime.equals("")||
			  queryStr==null||queryStr.equals("")||
			  typename==null||typename.equals("")){
				reslt.setSuccess(false);
				reslt.setMessage(Global.PARAMERROR);
				return reslt;
			}
        	String conditions = " and fi.mdytime>=:starttime and fi.mdytime<=:endtime and fi.typename=:typename and " +
        			"fi.rqtname like '%"+rqtname+"%' and " +
        			"fi.typename=:typename and " +
        			"fi.chkname like '%"+chkname+"%' and " +
        			"(fi.orivalue like '%"+queryStr+"%' or " +
					"fi.newvalue like '%"+queryStr+"%' or " +
					"fi.status like '%"+queryStr+"%' or " +
					"fi.dealname like '%"+queryStr+"%') ";
			if(isall==false){
				StaffPermission sp = (StaffPermission)session.getAttribute("staffinfo");
				Long stfid = sp.getStaffInfo().getStfid();
				conditions = " and fi.staffInfoByRqtid.stfid = "+stfid;
			}
			
			Query query = hsession.createQuery("from FeemodifyInfo fi join fi.usersInfo ui where ui.isvalid=:isvalid and fi.isvalid=:isvalid "+conditions+" order by fi.feemdyid desc");
			query.setBoolean("isvalid", true);
			query.setString("typename", typename);
			query.setDate("starttime", Global.secondformat.parse(starttime));
			query.setDate("endtime", Global.secondformat.parse(endtime));
			query.setFirstResult(start);
			query.setMaxResults(limit);
			
			Query countQuery = hsession.createQuery("select count(fi) from FeemodifyInfo fi join fi.usersInfo ui where ui.isvalid=:isvalid and fi.isvalid=:isvalid "+conditions+" order by fi.feemdyid desc");
			countQuery.setBoolean("isvalid", true);
			countQuery.setString("typename", typename);
			countQuery.setDate("starttime", Global.secondformat.parse(starttime));
			countQuery.setDate("endtime", Global.secondformat.parse(endtime));
			
			List count = countQuery.list();
			reslt.setTotalProperty((Long)count.get(0));
			
			List feelist = query.list();
			List data = new ArrayList();
			Long genReason = null;
			for(int i=0;i<feelist.size();++i){
				Object[] info = (Object[]) feelist.get(i);
				FeemodifyInfo inf = (FeemodifyInfo)info[0];
				UsersInfo user = (UsersInfo)info[1];
				if(inf.getGenreason())
					genReason = (long)1;
				else 
					genReason = (long)0;
				if(inf.getStaffInfoByChkid()==null)//当未审核时，审核人为空
					data.add(new ChangeInfo(inf.getFeemdyid(),Global.secondformat.format(inf.getMdytime()),inf.getStaffInfoByRqtid().getStfid(),inf.getRqtname(),
						null,inf.getChkname(),inf.getTypename(),inf.getOrivalue(),
						inf.getNewvalue(),inf.getAffect(),inf.getStatus(),user.getUsrid(),genReason));
				else
					data.add(new ChangeInfo(inf.getFeemdyid(),Global.secondformat.format(inf.getMdytime()),inf.getStaffInfoByRqtid().getStfid(),inf.getRqtname(),
						inf.getStaffInfoByChkid().getStfid(),inf.getChkname(),inf.getTypename(),inf.getOrivalue(),
						inf.getNewvalue(),inf.getAffect(),inf.getStatus(),user.getUsrid(),genReason));
			}		
			reslt.setData(data);
        }catch(Exception e){
        	e.printStackTrace();
        	reslt.setSuccess(false);
        	reslt.setMessage(Global.FAILURESTR);
        }
        return reslt;
	}
}