package com.glodon.paas.document.service.impl;

import java.io.IOException;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.Date;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.concurrent.Callable;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;

import org.apache.commons.lang.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;

import com.glodon.paas.account.core.bean.Contact;
import com.glodon.paas.account.core.bean.Member;
import com.glodon.paas.account.core.bean.Org;
import com.glodon.paas.consts.StringConst;
import com.glodon.paas.document.api.bean.File;
import com.glodon.paas.document.api.bean.Invitation;
import com.glodon.paas.document.api.bean.MemberView;
import com.glodon.paas.document.api.bean.Project;
import com.glodon.paas.document.api.bean.ProjectMember;
import com.glodon.paas.document.api.bean.ProjectMemberAttendees;
import com.glodon.paas.document.api.bean.ProjectPrivilege;
import com.glodon.paas.document.api.bean.ProjectRelation;
import com.glodon.paas.document.api.bean.ProjectRole;
import com.glodon.paas.document.api.bean.ProjectTemplate;
import com.glodon.paas.document.api.bean.ProjectTemplatePrivilege;
import com.glodon.paas.document.api.bean.ProjectTemplateResource;
import com.glodon.paas.document.api.bean.Revision;
import com.glodon.paas.document.api.bean.StringUtil;
import com.glodon.paas.document.api.bean.User;
import com.glodon.paas.document.api.cons.DocumentConstant;
import com.glodon.paas.document.cache.DocumentPaasUserCache;
import com.glodon.paas.document.cache.DocumentPaasUserEhcache;
import com.glodon.paas.document.dao.InvitationDao;
import com.glodon.paas.document.dao.ProjectDao;
import com.glodon.paas.document.dao.ProjectMemberAttendeesDao;
import com.glodon.paas.document.dao.ProjectMemberDao;
import com.glodon.paas.document.dao.ProjectPrivilegeDao;
import com.glodon.paas.document.dao.ProjectRoleDao;
import com.glodon.paas.document.dao.ProjectTemplateDao;
import com.glodon.paas.document.dao.ProjectTemplatePrivilegeDao;
import com.glodon.paas.document.dao.ProjectTemplateResourceDao;
import com.glodon.paas.document.exception.DocumentErrorCode;
import com.glodon.paas.document.exception.DocumentPaasDBException;
import com.glodon.paas.document.exception.DocumentPaasNotFoundException;
import com.glodon.paas.document.exception.DocumentPaasSystemException;
import com.glodon.paas.document.exception.DocumentPaasValidateException;
import com.glodon.paas.document.service.FileService;
import com.glodon.paas.document.service.PrivilegeService;
import com.glodon.paas.document.service.ProjectService;
import com.glodon.paas.document.service.RevisionService;
import com.glodon.paas.document.service.util.MailPropertiesUtil;
import com.glodon.paas.document.service.util.RoleMessages;
import com.glodon.paas.document.service.util.WebService;
import com.glodon.paas.document.service.util.WebServiceNotification;
import com.glodon.paas.document.util.Deteminator;
import com.glodon.paas.notification.dto.MailMessageParam;
import com.glodon.paas.storage.core.Storage;
import com.glodon.paas.storage.core.StorageFile;
import com.glodon.paas.util.DateUtil;
import com.glodon.paas.util.TokenGenerator;
import com.glodon.paas.util.UUIDUtil;

@Service("projectService")
public class ProjectServiceImpl implements ProjectService {

    @Autowired
    private ProjectTemplateDao projectTemplateDao;
    @Autowired
    private InvitationDao invitationDao;
    @Autowired
    private ProjectMemberDao projectMemberDao;
    @Autowired
    private WebService webService;
    @Autowired
    private FileService fileService;
    @Autowired
    private Storage storage;
    @Autowired
    private ProjectDao projectDao;
    @Autowired
    private ProjectRoleDao projectRoleDao;
    @Autowired
    private ProjectPrivilegeDao projectPrivilegeDao;
    @Autowired
    private ProjectMemberAttendeesDao attendeesDao;
    @Autowired
    private ProjectTemplateResourceDao projectTemplateResourceDao;
    @Autowired
    private ProjectTemplatePrivilegeDao projectTemplatePrivilegeDao;
    @Autowired
    private RevisionService revisionService;
    @Autowired
    private PrivilegeService privilegeService;
    @Autowired
    private DocumentPaasUserCache userCache;
    @Autowired
    @Qualifier(value = "urlDeteminator")
    private Deteminator<String> urlDeteminator;

    @Value("${project.quota}")
    private int projectQuota = 1;

    private static final Logger LOGGER = LoggerFactory.getLogger(ProjectServiceImpl.class);

    @Override
    public Project createProject(String userId, String email, String name, String description) {
        if (StringUtils.isBlank(name)) {
            throw new DocumentPaasValidateException(DocumentErrorCode.EMPTY_PROJECT_NAME,
                    "Project name should not be empty!", new Exception("Project name is empty!"));
        }
        Project pe = projectDao.getProjectByName(userId, name);
        if (pe != null) {
            throw new DocumentPaasValidateException(DocumentErrorCode.DUPLICATE_PROJECT,
                    "There is a project with the same name!", new Exception("Project name is duplicate!"));
        }
        Date now = new Date();
        // project
        Project project = new Project();
        project.setId(UUIDUtil.getUUIDStr());
        project.setOwnerId(userId);
        project.setIsDeleted(0);
        project.setName(name);
        project.setDescription(description);

        User user = new User();
        user.setId(userId);
        File folder = fileService.createProjectFolder(userId, project.getId());
        project.setFolderId(folder.getId());
        // 管理员 角色
        ProjectRole ownerRole = new ProjectRole();
        ownerRole.setId(UUIDUtil.getUUIDStr());
        ownerRole.setRoleName(RoleMessages.getString("ProjectServiceImpl.Creator"));
        ownerRole.setProjectId(project.getId());
        ownerRole.setCreator(user);
        ownerRole.setIsEditable(0);
        ownerRole.setIsVisible(1);
        ownerRole.setUpdator(user);
        ownerRole.setIsDefault(0);
        ownerRole.setIsDeleted(0);
        ownerRole.setCreateTime(now);
        ownerRole.setUpdateTime(now);
        // 未分配 角色
        ProjectRole defaultRole = new ProjectRole();
        defaultRole.setId(UUIDUtil.getUUIDStr());
        defaultRole.setRoleName(RoleMessages.getString("ProjectServiceImpl.Undispatched"));
        defaultRole.setProjectId(project.getId());
        defaultRole.setIsDefault(1);
        defaultRole.setIsEditable(0);
        defaultRole.setIsVisible(1);
        defaultRole.setCreator(user);
        defaultRole.setUpdator(user);
        defaultRole.setIsDeleted(0);
        defaultRole.setCreateTime(now);
        defaultRole.setUpdateTime(now);
        // 添加 默认成员
        ProjectMember member = new ProjectMember();
        member.setId(UUIDUtil.getUUIDStr());
        member.setProjectId(project.getId());
        member.setRoleId(ownerRole.getId());
        member.setStatus(1);
        // member.setMemberType(0);
        // member.setUserId(userId);
        member.setMemberId(userId);
        member.setRootId(userId);
        member.setCreateTime(now);
        member.setUpdateTime(now);
        member.setEditable(1);
        // Add attendees
        ProjectMemberAttendees attendee = new ProjectMemberAttendees();
        attendee.setId(UUIDUtil.getUUIDStr());
        attendee.setMemberId(member.getId());
        attendee.setStatus(1);
        attendee.setEditable(1);
        attendee.setUserId(userId);
        attendee.setCreateTime(now);
        attendee.setUpdateTime(now);
        attendee.setCreator(userId);
        attendee.setUpdator(userId);
        // 管理员角色默认权限，读，写，删除，进入
        ProjectPrivilege ownerRoleResource = new ProjectPrivilege();
        ownerRoleResource.setId(UUIDUtil.getUUIDStr());
        ownerRoleResource.setFileId(folder.getId());
        ownerRoleResource.setPrivilegeStr(DocumentConstant.FILE_READ + StringConst.SEPARATOR_COMMA
                + DocumentConstant.FILE_WRITE + StringConst.SEPARATOR_COMMA + DocumentConstant.FILE_DELETE
                + StringConst.SEPARATOR_COMMA + DocumentConstant.FILE_ENTER);
        ownerRoleResource.setRole(ownerRole);
        ownerRoleResource.setUpdateTime(now);
        ownerRoleResource.setUpdatorId(userId);
        // 未分配人员默认权限， 进入
        ProjectPrivilege defaultRoleResource = new ProjectPrivilege();
        defaultRoleResource.setId(UUIDUtil.getUUIDStr());
        defaultRoleResource.setFileId(folder.getId());
        defaultRoleResource.setPrivilegeStr(DocumentConstant.FILE_ENTER);
        defaultRoleResource.setRole(defaultRole);
        defaultRoleResource.setUpdateTime(now);
        defaultRoleResource.setUpdatorId(userId);
        createProject(project, folder, ownerRole, defaultRole, member, attendee, ownerRoleResource, defaultRoleResource);
        // add member in account desynchronized
        //        addProjectRelation(userId, userId, member.getProjectId(), member.getRoleId(), userId,false);
        projectDao.addProjectRelation(UUIDUtil.getUUIDStr(), userId, member.getProjectId(), member.getRoleId(), userId);

        return project;
        // projectDto.setFiles(new ArrayList<File>());
        // projectDto.getFiles().add(File.getDto(folder, false));
        // return projectDto;
    }

    @Override
    public void addProjectRelation(final String loginId, final String userId, final String projectId,
            final String roleId, final String orgId) {
        projectDao.addProjectRelation(UUIDUtil.getUUIDStr(), userId, projectId, roleId, orgId);
    }

    //    private void addProjectRelationAsync(final String loginId, final String userId,
    //            final String projectId, final String roleId, final String orgId) {
    //        ExecutorService threadPool = Executors.newCachedThreadPool();
    //        try {
    //            threadPool.submit(new Callable<Object>() {
    //
    //                @Override
    //                public Object call() throws Exception {
    //                    webService.addProjectRole(userId, userId, projectId,
    //                            roleId, userId);
    //                    return null;
    //                }
    //
    //            });
    //        } finally {
    //            threadPool.shutdown();
    //        }
    //    }

    @Override
    public void checkUserProjectQuota(String userId) {
        User user = getUserFromCache(userId);
        if (user != null) {
            List<ProjectRelation> projects = projectDao.checkQuota(userId);
            int projectCount = (projects != null && !projects.isEmpty() ? projects.size() : 0);
            if (projectCount >= projectQuota) {
                throw new DocumentPaasValidateException(DocumentErrorCode.USER_PROJECT_QUOTA_EXCEED,
                        "User can only create " + projectQuota + " project when in trial service!", new Exception(
                                "User project quota exceed!"));
            }
        }
    }

    @Override
    public boolean deleteProject(String userId, String projectId) {
        Project entity = projectDao.get(projectId);
        if (entity != null) {
            fileService.purge(entity.getFolderId());
            return deleteProject(projectId);
        }
        return false;
    }

    @Override
    public Project updateProject(String userId, Project project) {
        String name = project.getName();
        if (StringUtils.isBlank(name)) {
            throw new DocumentPaasValidateException(DocumentErrorCode.EMPTY_PROJECT_NAME,
                    "Project name should not be empty!", new Exception("Project name is empty!"));
        }
        Project dbProject = projectDao.get(project.getId());
        if (dbProject == null) {
            throw new DocumentPaasNotFoundException(DocumentErrorCode.DATA_NOT_FOUND, "can not find project["
                    + project.getId() + "]", null);
        }
        Project pe = projectDao.getProjectByName(userId, name);
        if (pe != null && !pe.getId().equals(project.getId())) {
            throw new DocumentPaasValidateException(DocumentErrorCode.DUPLICATE_PROJECT,
                    "There is a project with the same name!", new Exception("Project name is duplicate!"));
        }
        dbProject.setName(name);
        dbProject.setDescription(project.getDescription());
        dbProject.setUpdateTime(new Date());
        updateProject(dbProject);
        //        String folderId = dbProject.getFolderId();
        //        fileService.rename(userId, folderId, name);
        return dbProject;
    }

    @Override
    public ProjectRole createRole(String userId, String projectId, String roleName) {
        Project project = projectDao.get(projectId);
        if (project == null) {
            throw new DocumentPaasNotFoundException(DocumentErrorCode.DATA_NOT_FOUND, "can not find project["
                    + projectId + "]", null);
        }
        Date now = new Date();
        ProjectRole role = new ProjectRole();
        User creator = new User();
        creator.setId(userId);
        role.setCreator(creator);
        role.setUpdator(creator);
        role.setId(UUIDUtil.getUUIDStr());
        role.setProjectId(projectId);
        role.setIsDefault(0);
        role.setIsVisible(1);
        role.setIsEditable(1);
        role.setRoleName(roleName);
        role.setCreateTime(now);
        role.setUpdateTime(now);
        // check the same name role
        boolean result = projectRoleDao.checkSameRole(role);
        if (result) {
            throw new DocumentPaasValidateException(DocumentErrorCode.DUPLICATE_ROLE,
                    "There is a role with the same name!", new Exception("Role name is duplicate!"));
        }
        // 新角色 根目录 有进入权限
        ProjectPrivilege resource = new ProjectPrivilege();
        resource.setId(UUIDUtil.getUUIDStr());
        resource.setFileId(project.getFolderId());
        resource.setPrivilegeStr(DocumentConstant.FILE_ENTER);
        resource.setRole(role);
        resource.setUpdateTime(new Date());
        resource.setUpdatorId(userId);
        createRole(role, resource);
        return role;
    }

    @Override
    public ProjectRole getProjectRole(String roleId) {
        return projectRoleDao.get(roleId);
    }

    @Override
    public List<ProjectRole> getProjectRoles(String userId, String projectId) {
        Project project = projectDao.get(projectId);
        if (project == null) {
            throw new DocumentPaasNotFoundException(DocumentErrorCode.DATA_NOT_FOUND, "can not find project["
                    + projectId + "]", null);
        }
        List<ProjectRole> projectRoles = projectRoleDao.getRoles(projectId);
        return projectRoles;
    }

    @Override
    public ProjectRole updateRole(String userId, String roleId, String name) {
        if (StringUtils.isBlank(roleId) || StringUtils.isBlank(name)) {
            throw new DocumentPaasValidateException(DocumentErrorCode.VALIDATE_ERROR, "roleId or role name is empty.",
                    null);
        }
        ProjectRole role = projectRoleDao.get(roleId);
        if (role == null) {
            throw new DocumentPaasValidateException(DocumentErrorCode.DATA_NOT_FOUND, "can not find Role[" + roleId
                    + "]", null);
        }
        Project project = projectDao.get(role.getProjectId());
        if (project == null) {
            throw new DocumentPaasNotFoundException(DocumentErrorCode.DATA_NOT_FOUND, "can not find project["
                    + role.getProjectId() + "]", null);
        }
        // Check the duplicate name role
        ProjectRole roleForCheck = new ProjectRole();
        roleForCheck.setProjectId(role.getProjectId());
        roleForCheck.setRoleName(name);
        boolean result = projectRoleDao.checkSameRole(roleForCheck);
        if (result) {
            throw new DocumentPaasValidateException(DocumentErrorCode.DUPLICATE_ROLE,
                    "There is a role with the same name!", new Exception("Role name is duplicate!"));
        }
        if (role.isEditable()) {
            ProjectRole dbRole = projectRoleDao.get(roleId);
            dbRole.setRoleName(name);
            User updator = new User();
            updator.setId(userId);
            dbRole.setUpdator(updator);
            dbRole.setUpdateTime(new Date());
            projectRoleDao.updateRole(dbRole);
            return dbRole;
        } else {
            throw new DocumentPaasSystemException(DocumentErrorCode.DOCUMENT_ERROR,
                    "system defined Role, can not update.", null);
        }
    }

    @Override
    public void deleteRole(String userId, String roleId) {
        ProjectRole role = projectRoleDao.get(roleId);
        if (role == null) {
            throw new DocumentPaasNotFoundException(DocumentErrorCode.DATA_NOT_FOUND, "can not find Role[" + roleId
                    + "]", null);
        }
        Project project = projectDao.get(role.getProjectId());
        if (project == null) {
            throw new DocumentPaasNotFoundException(DocumentErrorCode.DATA_NOT_FOUND, "can not find project["
                    + role.getProjectId() + "]", null);
        }
        if (role.isEditable()) {
            deleteRole(roleId);
            // Get default role id
            ProjectRole defaultRole = projectRoleDao.getDefaultRole(role.getProjectId());
            updateProjectRelation(userId, role.getProjectId(), roleId, defaultRole.getId(), false);
        } else {
            throw new DocumentPaasSystemException(DocumentErrorCode.DOCUMENT_ERROR,
                    "system defined Role, can not delete.", null);
        }
    }

    private void updateProjectRelation(final String userId, final String projectId, final String roleId,
            final String newRoleId, boolean async) {
        if (async) {
            updateProjectRelationAsync(userId, projectId, roleId, newRoleId);
        } else {
            projectDao.updateProjectRelation(projectId, roleId, newRoleId);
        }
    }

    private void updateProjectRelationAsync(final String userId, final String projectId, final String roleId,
            final String newRoleId) {
        ExecutorService threadPool = Executors.newCachedThreadPool();
        try {
            threadPool.submit(new Callable<Object>() {

                @Override
                public Object call() throws Exception {
                    webService.updateProjectRole(userId, projectId, roleId, newRoleId);
                    return null;
                }

            });
        } finally {
            threadPool.shutdown();
        }
    }

    @Override
    public boolean updateMemberStatus(String memberId, int status) {
        return projectMemberDao.updateMemberStatus(memberId, status);
    }

    @Override
    public Project getProject(String projectId) {
        Project dbProject = projectDao.get(projectId);
        if (dbProject == null) {
            throw new DocumentPaasNotFoundException(DocumentErrorCode.DATA_NOT_FOUND, "can not find project["
                    + projectId + "]", null);
        }
        return dbProject;
    }

    @Override
    public void insertInvitation(Invitation entity) {
        invitationDao.insert(entity);
    }

    @Override
    public boolean confirmInvitation(String userId, String token) {
        return projectMemberDao.acceptMember(userId, token);
    }

    @Override
    public List<Project> getProjectsByIds(List<String> ids) {
        return projectDao.getProjects(ids);
    }

    @Override
    public List<ProjectMember> getMembers(String projectId, String roleId) {
        return projectMemberDao.getMembers(projectId, roleId);
    }

    @Override
    public ProjectMember getConfirmedMember(String userId, String token) {
        Invitation entity = invitationDao.getInvitation(userId, token);
        if (entity != null) {
            ProjectMember projectMemberEntity = projectMemberDao.get(entity.getMemberId());
            return projectMemberEntity;
        }
        return null;
    }

    @Override
    public ProjectMember getMemberById(String memberId) {
        return projectMemberDao.get(memberId);
    }

    @Override
    public ProjectRole getDefaultRole(String projectId) {
        return projectRoleDao.getDefaultRole(projectId);
    }

    @Override
    public boolean updateRoleOfMember(String memberId, String projectId, String oldRoleId, String newRoleId) {
        return projectMemberDao.updateRoleOfMember(memberId, projectId, oldRoleId, newRoleId) > 0;
    }

    @Override
    public List<MemberView> inviteMembers(MemberView member, User owner, String basePath) {
        // Get role id
        String roleId = member.getRoleId();
        // Get project id
        String projectId = member.getProjectId();
        // if(member.getMembers()!=null&&!member.getMembers().isEmpty()){
        //
        // roleId = member.getMembers().get(0).getRoleId();
        // projectId = member.getMembers().get(0).getProjectId();
        // }
        // If the role id is empty, replace with default role
        if (StringUtils.isEmpty(roleId)) {
            ProjectRole role = getDefaultRole(projectId);
            roleId = role.getId();
        }
        // step1: insert into member table
        Map<String, ProjectMember> statusMap = new HashMap<String, ProjectMember>();
        // Get project information
        Project project = projectDao.get(projectId);
        if (project == null) {
            throw new DocumentPaasValidateException(DocumentErrorCode.PROJECT_NOT_EXIST, "Project is not exist!",
                    new Exception("Project is not exist!"));
        }
        // here is for multiple inviting in future
        List<MemberView> members = new ArrayList<MemberView>();
        // if is group,get organization members
        List<Org> orgList = new ArrayList<Org>();
        // Invite logic
        inviteMember(member, owner, project, basePath, orgList, statusMap);
        members.add(member);
        // Get user information
        // get members
        List<MemberView> result = new ArrayList<MemberView>();
        for (MemberView memberView : members) {
            if (!memberView.isOrg()) {
                // If the member is not a organization
                String userId = getUserId(memberView);
                ;

                // Get user info
                User user = getUserFromCache(userId);

                // Combine detail info for member
                ProjectMember pm = statusMap.get(user.getIdStr());
                memberView.setId(pm.getId());
                memberView.setUserId(user.getIdStr());
                memberView.setStatus(pm.getStatus());
                memberView.setMemberCount(0);
                memberView.setName(memberView.getAccount());
                if (user != null) {
                    memberView.setName(user.getDisplayName());
                }
                result.add(memberView);
            }
        }
        //        if (!orgList.isEmpty()) {
        //            // If the member is an organization, do special process
        //            List<MemberView> orgMembers = new ArrayList<MemberView>();
        //            for (Org org : orgList) {
        //                // Get full path of the organization
        //                Org fullOrg = webService.getOrgPath(org.getId(), owner.getIdStr());
        //                // Convert organization to MemberView object with namenode attribute
        //                this.convertOrgWithNameNode(fullOrg, org, statusMap, orgMembers);
        //            }
        //            // Combine detail info for organization
        //            setDetailForMemberView(orgMembers, statusMap, projectId, roleId);
        //            result.addAll(orgMembers);
        //        }
        return result;
    }

    private String getUserId(MemberView mv) {
        String userId = "";
        if (mv != null) {
            if (!StringUtil.isBlank(mv.getOrgId())) {
                // if member is an email or other single user
                userId = mv.getUserId();
            } else {
                // if member is in an organization
                userId = mv.getAccount();
            }
        }
        return userId;
    }

    private List<User> getUsersFromCache(Set<String> userIds) {
        List<User> users = new ArrayList<User>();

        if (userCache == null) {
            userCache = new DocumentPaasUserEhcache();
        }
        Set<String> ids = new HashSet<String>();
        for (String id : userIds) {
            User user = userCache.hint(id);
            if (user == null) {
                ids.add(id);
            } else {
                users.add(user);
            }
        }
        List<User> userList = new ArrayList<User>();
        if (!ids.isEmpty()) {
            userList = webService.getUsers(ids);
            if (userList != null && !userList.isEmpty()) {
                users.addAll(userList);
                for (User user : userList) {
                    userCache.putUser(user);
                }
            }
        }
        return users;
    }

    private User getUserFromCache(String userId) {
        User user = null;
        if (userCache == null) {
            userCache = new DocumentPaasUserEhcache();
        }
        if (user == null) {
            user = userCache.get(userId);
        }
        //remove this api
        //      if (user == null) {
        //          user = userCache.getInactiveUser(userId);
        //      }
        return user;
    }

    private void inviteMember(MemberView member, User owner, Project project, String basePath, List<Org> orgList,
            Map<String, ProjectMember> statusMap) {
        if (member.isOrg()) {
            // insert into member database
            //            ProjectMember entity = new ProjectMember();
            //            entity.setCreateTime(DateUtil.getNow());
            //            entity.setId(UUIDUtil.getUUIDStr());
            //            entity.setProjectId(member.getProjectId());
            //            entity.setRoleId(member.getRoleId());
            //            // if the member is an organization,accept the invitation automatically
            //            entity.setStatus(1);
            //            entity.setIsOrg(1);
            //            entity.setMemberId(member.getUserId());
            //            entity.setUpdateTime(entity.getCreateTime());
            //            // Set root id
            //            Org rootOrg = webService.getOrgPath(member.getUserId(), owner.getIdStr());
            //            entity.setRootId(rootOrg.getId());
            //            ProjectMember pm = projectMemberDao.getMember(member.getProjectId(), member.getRoleId(),
            //                                                          entity.getMemberId(), null);
            //            if (pm != null) {
            //                throw PaasServerExceptionFactory.validateException("Member is already exist!",
            //                                                                   new Exception("Member is already exist!")).setErrorCode(ErrorCode.MEMBER_EXIST);
            //            }
            //            insertMember(entity);
            //            // Get organization members
            //            Org org = webService.getOrgById(member.getUserId(), owner.getIdStr(), true);
            //            orgList.add(org);
            //            List<Member> unInvitedMemList = new ArrayList<Member>();
            //            // Get members of the organization
            //            this.getUninvitedOrgMember(member.getProjectId(), member.getRoleId(), org, unInvitedMemList);
            //            this.processUninvitedMembers(entity, member, owner, project, basePath, unInvitedMemList, statusMap);
            //            this.processSubOrgs(entity, member, org);
        } else {
            // begin check if email/mobile is registered.
            User user = null;
            String userId = getUserId(member);
            // define if the user is in organization
            boolean isInOrg = false;
            // if the member is new in Organization
            String memberId = "";
            // define the status of user
            boolean isInActiveUser = false;
            user = getUserFromCache(userId);
            if (!StringUtil.isBlank(member.getOrgId())) {
                memberId = member.getOrgId();
                isInOrg = true;
            } else {
                if (user == null) {
                    //chage this to create viretual user api
                    //                    user = webService.createInactiveUser(member.getAccount());
                    user = webService.createVitualUser(member.getAccount());
                    isInActiveUser = true;
                }
                if (user.isVirtual()) {
                    isInActiveUser = true;
                }
                memberId = user.getIdStr();
            }
            // check member: if the user is already invited ,throw exception
            ProjectMember pm = projectMemberDao.getMember(member.getProjectId(), member.getRoleId(), memberId, null);
            if (pm != null && pm.getIsOrg() != 1) {
                throw new DocumentPaasValidateException(DocumentErrorCode.MEMBER_EXIST, "Member is already exist!",
                        new Exception("Member is already exist!"));
            }
            String attendeeMemberId = "";
            userId = user.getIdStr();
            String attendeeId = UUIDUtil.getUUIDStr();
            ProjectMember entity = new ProjectMember();
            if (!isInOrg) {
                // If the member is not in an organization, create a new project member
                // insert into member database
                entity.setCreateTime(new Date());
                entity.setId(UUIDUtil.getUUIDStr());
                entity.setProjectId(member.getProjectId());
                entity.setRoleId(member.getRoleId());
                entity.setStatus(0);
                entity.setIsOrg(0);
                // end
                entity.setMemberId(user.getIdStr());
                // If the member is a single person,set root id with user id
                entity.setRootId(user.getIdStr());
                entity.setUpdateTime(entity.getCreateTime());
                // TODO insert member at last
                // insertMember(entity);
                attendeeMemberId = entity.getId();
                // set member Id =attendee id for view in page
                // Copy a member entity for view in page
                ProjectMember viewEntity = new ProjectMember();
                viewEntity = copy2ViewEntity(entity);
                viewEntity.setId(attendeeId);
                statusMap.put(entity.getMemberId(), viewEntity);
            } else {
                // If the member is in an organization and the organization is already in the project
                attendeeMemberId = pm.getId();
                // Set the status=0 for view in the front page
                pm.setStatus(0);
                // Set project member id = attendee id
                pm.setId(attendeeId);
                statusMap.put(userId, pm);
            }
            // Add the member in attendees
            ProjectMemberAttendees attendees = new ProjectMemberAttendees();
            attendees.setId(attendeeId);
            attendees.setMemberId(attendeeMemberId);
            attendees.setStatus(0);
            attendees.setUserId(userId);
            attendees.setCreator(owner.getIdStr());
            attendees.setUpdator(owner.getIdStr());
            attendees.setCreateTime(new Date());
            attendees.setUpdateTime(new Date());
            // Check if the member is already accepted in the project
            if (this.checkAcceptedMember(member.getProjectId(), userId)) {
                attendees.setStatus(1);
                entity.setStatus(1);
                // add project id into this user project list
                //webService.addProjectRole(owner.getIdStr(), userId, member.getProjectId(), member.getRoleId(),
                //                          statusMap.get(userId).getMemberId());
                this.addProjectRelation(owner.getIdStr(), userId, member.getProjectId(), member.getRoleId(), statusMap
                        .get(userId).getMemberId());
                // Set project member Status for view
                statusMap.get(userId).setStatus(1);
            } else {
                // Define email parameters
                Map<String, Object> param = new HashMap<String, Object>();
                param.put("owner", owner.getDisplayName());
                param.put("projectName", project.getName());
                // Generate inviting token
                String token = TokenGenerator.generate(user.getEmail() + member.getRoleId() + member.getProjectId()
                        + new Date());
                String queryString = "?userId=" + userId + "&token=" + token;
                String link = urlDeteminator.detemine(basePath) + "/project/" + member.getProjectId()
                        + "/member/acception" + queryString;
                param.put("link", link);

                // Step2: send invitation mail
                if (!isInActiveUser) {
                    sendMail(user.getEmail(), param, MailPropertiesUtil.getString("project.mail.inviteMember.title"),
                            MailPropertiesUtil.getString("project.mail.inviteMember.template"));
                } else {
                    // If the member is an inactive user, also send an active link
                    param.put("activeLink", basePath + "/index?userId=" + userId + "&email=" + user.getEmail());
                    sendMail(user.getEmail(), param, MailPropertiesUtil.getString("project.mail.inviteMember.title"),
                            MailPropertiesUtil.getString("project.mail.inviteMember.Activetemplate"));
                }
                // 站内通知
                WebServiceNotification.processUninvitedMembers(owner, userId, project, token, link + "&isNotification");

                // Step3: Insert invite history
                Invitation invitation = new Invitation();
                invitation.setId(UUIDUtil.getUUIDStr());
                invitation.setAccount(userId);
                invitation.setToken(token);
                invitation.setStatus(0);
                invitation.setMemberId(attendeeMemberId);
                invitation.setCreateTime(DateUtil.getNow());
                invitation.setUpdateTime(invitation.getCreateTime());
                insertInvitation(invitation);
            }
            // TODO insert project member if the member is not in an organization
            if (!isInOrg) {
                insertMember(entity);
            }
            attendeesDao.insert(attendees);
        }
    }

    private ProjectMember copy2ViewEntity(ProjectMember entity) {
        ProjectMember viewEntity = new ProjectMember();
        viewEntity.setCreateTime(entity.getCreateTime());
        viewEntity.setId(entity.getId());
        viewEntity.setProjectId(entity.getProjectId());
        viewEntity.setRoleId(entity.getRoleId());
        viewEntity.setStatus(entity.getStatus());
        viewEntity.setIsOrg(entity.getIsOrg());
        // end
        viewEntity.setMemberId(entity.getMemberId());
        // If the member is a single person,set root id with user id
        viewEntity.setRootId(entity.getRootId());
        viewEntity.setUpdateTime(entity.getCreateTime());
        return viewEntity;
    }

    private void processSubOrgs(ProjectMember entity, MemberView member, Org org) {
        List<String> subOrgIdList = new ArrayList<String>();
        getSubOrgId(org, subOrgIdList);
        if (!subOrgIdList.isEmpty()) {
            String[] ids = subOrgIdList.toArray(new String[subOrgIdList.size()]);
            processInvitedOrgMember(entity.getId(), member.getProjectId(), member.getRoleId(), ids);
        }
    }

    private void getSubOrgId(Org org, List<String> subOrgIdList) {
        if (org != null) {
            if (org.getSubOrgs() != null && !org.getSubOrgs().isEmpty()) {
                for (Org subOrg : org.getSubOrgs()) {
                    subOrgIdList.add(subOrg.getId());
                    getSubOrgId(subOrg, subOrgIdList);
                }
            }
        }
    }

    /**
     * Process uninvited member in this role
     * 
     * @param entity
     * @param member
     * @param owner
     * @param project
     * @param basePath
     * @param unInvitedMemList
     * @param statusMap
     */
    private void processUninvitedMembers(ProjectMember entity, MemberView member, User owner, Project project,
            String basePath, List<Member> unInvitedMemList, Map<String, ProjectMember> statusMap) {
        if (!unInvitedMemList.isEmpty()) {
            // Invite each member
            for (Member mem : unInvitedMemList) {
                ProjectMemberAttendees attendees = new ProjectMemberAttendees();
                attendees.setId(UUIDUtil.getUUIDStr());
                attendees.setMemberId(entity.getId());
                // Set status 0:invited
                attendees.setStatus(0);
                attendees.setUserId(mem.getUserId());
                attendees.setCreator(owner.getIdStr());
                attendees.setUpdator(owner.getIdStr());
                attendees.setCreateTime(entity.getCreateTime());
                attendees.setUpdateTime(entity.getCreateTime());
                // check the member is already accepted in the project
                if (checkAcceptedMember(member.getProjectId(), mem.getUserId())) {
                    // add project id into this user project list
                    addProjectRelation(owner.getIdStr(), mem.getUserId(), member.getProjectId(), member.getRoleId(),
                            entity.getMemberId());
                    attendees.setStatus(1);
                } else {
                    // Set mail parameters
                    Map<String, Object> param = new HashMap<String, Object>();
                    param.put("owner", owner.getDisplayName());
                    param.put("projectName", project.getName());
                    // Generate inviting token
                    String token = TokenGenerator.generate(mem.getEmail() + member.getRoleId() + member.getProjectId()
                            + new Date());
                    String queryString = "?userId=" + attendees.getUserId() + "&token=" + token;
                    String link = urlDeteminator.detemine(basePath) + "/project/" + member.getProjectId()
                            + "member/acception" + queryString;
                    param.put("link", link);
                    // Step2: send invitation mail
                    boolean mailResult = sendMail(mem.getEmail(), param,
                            MailPropertiesUtil.getString("project.mail.inviteMember.title"),
                            MailPropertiesUtil.getString("project.mail.inviteMember.template"));
                    // 站内通知
                    WebServiceNotification.processUninvitedMembers(owner, member.getUserId(), project, token, link
                            + "&isNotification");

                    // Step3: Insert invite history
                    Invitation invitation = new Invitation();
                    invitation.setId(UUIDUtil.getUUIDStr());
                    invitation.setAccount(attendees.getUserId());
                    invitation.setToken(token);
                    invitation.setStatus(0);
                    invitation.setMemberId(entity.getId());
                    invitation.setCreateTime(DateUtil.getNow());
                    invitation.setUpdateTime(invitation.getCreateTime());
                    insertInvitation(invitation);
                    // set invitation id
                    attendees.setInvitationId(invitation.getId());
                    if (!mailResult) {
                        entity.setStatus(-1);
                    }
                }
                attendeesDao.insert(attendees);
                // remember status of attendee
                ProjectMember attendeeEntity = new ProjectMember();
                attendeeEntity.setId(attendeeEntity.getId());
                attendeeEntity.setMemberId(attendees.getUserId());
                attendeeEntity.setStatus(attendees.getStatus());
                statusMap.put(attendees.getUserId(), attendeeEntity);
            }
        }
    }

    /**
     * Check the user is already a member of the project
     * 
     * @param projectId project id
     * @param userId user id
     * @return true/false
     */
    private boolean checkAcceptedMember(String projectId, String userId) {
        return attendeesDao.checkAcceptedMember(projectId, userId) > 0;
    }

    private boolean checkInvited(String projectId, String roleId, String orgId, String userId) {
        return attendeesDao.checkInvited(projectId, roleId, orgId, userId) > 0;
    }

    /**
     * Set detail info for memberview
     * 
     * @param orgMembers members in organization
     * @param statusMap member status
     * @param projectId project id
     * @param roleId role id
     */
    private void setDetailForMemberView(List<MemberView> orgMembers, Map<String, ProjectMember> statusMap,
            String projectId, String roleId) {
        for (MemberView mem : orgMembers) {
            if (mem.isOrg()) {
                mem.setProjectId(projectId);
                mem.setRoleId(roleId);
                mem.setStatus(1);
                mem.setUserId(mem.getId());
                mem.setEditable(true);
            } else {
                ProjectMember pm = statusMap.get(mem.getUserId());
                String memberId = "";
                if (pm != null) {
                    mem.setStatus(pm.getStatus());
                } else {
                    // Get added attendee info
                    ProjectMemberAttendees pmEntity = getAttendeeByUserId(projectId, roleId, mem.getUserId());
                    pm = projectMemberDao.get(pmEntity.getMemberId());
                    mem.setStatus(pmEntity.getStatus());
                }
                mem.setId(pm.getId());
                memberId = pm.getMemberId();
                User user = webService.getUser(memberId);
                if (user != null) {
                    mem.setName(user.getDisplayName());
                } else {
                    mem.setName(mem.getAccount());
                }
                mem.setUserId(memberId);
                mem.setProjectId(projectId);
                mem.setRoleId(roleId);
                mem.setEditable(true);
            }
            if (mem.getMemberViews() != null && !mem.getMemberViews().isEmpty()) {
                setDetailForMemberView(mem.getMemberViews(), statusMap, projectId, roleId);
            }
        }
    }

    /**
     * Send email when invite a member into project
     * 
     * @param sendTo accept mail address
     * @param title mail tile
     * @param template mail content template
     * @return true/false
     */
    private boolean sendMail(String sendTo, Map<String, Object> param, String title, String template) {
        if (StringUtil.isBlank(sendTo))
            return true;
        MailMessageParam mailContent = new MailMessageParam();
        mailContent.setTo(sendTo.split(";"));
        mailContent.setSubject(title);
        mailContent.setTemplateLocation(template);
        mailContent.setTemplateParam(param);
        int result = WebServiceNotification.sendMail(mailContent, true);
        return result == 200;
    }

    @Override
    public boolean reInvite(ProjectMember member, User owner, String basePath) {
        // get project member object by :project id, role id, account id
        member = projectMemberDao.getMember(member.getProjectId(), member.getRoleId(), member.getUserId(), null);
        // step1: update status in member table
        updateMemberStatus(member.getId(), 0);
        String projectId = member.getProjectId();
        Project project = projectDao.get(projectId);
        if (project == null) {
            throw new DocumentPaasValidateException(DocumentErrorCode.PROJECT_NOT_EXIST, "Project is not exist!",
                    new Exception("Project is not exist!"));
        }
        // step2: send invite email
        Map<String, Object> param = new HashMap<String, Object>();
        param.put("owner", owner.getDisplayName());
        param.put("projectName", project.getName());
        // begin get user
        User user = getUserFromCache(member.getUserId());
        // end
        String token = TokenGenerator.generate(user.getEmail() + member.getRoleId() + projectId + new Date());
        String queryString = "?userId=" + member.getUserId() + "&token=" + token;
        param.put("link", basePath + queryString);
        boolean result = sendMail(user.getEmail(), param,
                MailPropertiesUtil.getString("project.mail.inviteMember.title"),
                MailPropertiesUtil.getString("project.mail.inviteMember.template"));
        if (result) {
            // Step3:Insert invite history
            Invitation invitation = new Invitation();
            invitation.setId(UUIDUtil.getUUIDStr());
            invitation.setToken(token);
            invitation.setMemberId(member.getId());
            invitation.setAccount(member.getUserId());
            invitation.setCreateTime(new Date());
            invitation.setUpdateTime(invitation.getCreateTime());
            insertInvitation(invitation);
            ProjectMemberAttendees attendees = attendeesDao.getAttendeeByUIDandPMID(member.getUserId(), member.getId());
            // update attendee invitationId
            attendeesDao.updateAttendeeInvitation(attendees.getId(), invitation.getId());
        }
        return result;
    }

    @Override
    public List<MemberView> showMembers(String projectId, String roleId, String ownerId) {
        List<ProjectMember> members = new ArrayList<ProjectMember>();
        // step1: list all members
        members = getMembers(projectId, roleId);
        // step1.1:get attendees：
        List<ProjectMember> projectMemberList = new ArrayList<ProjectMember>();
        for (ProjectMember member : members) {
            // Get all attendees
            List<ProjectMemberAttendees> attendeesList = attendeesDao.getAttendees(member.getId());
            if (!attendeesList.isEmpty()) {
                for (ProjectMemberAttendees entity : attendeesList) {
                    ProjectMember mem = new ProjectMember();
                    mem.setId(entity.getId());
                    mem.setProjectId(member.getProjectId());
                    mem.setRoleId(member.getRoleId());
                    mem.setIsOrg(member.getIsOrg());
                    if (member.getIsOrg() == 1) {
                        mem.setGroupId(member.getMemberId());
                    }
                    mem.setUserId(entity.getUserId());
                    mem.setStatus(entity.getStatus());
                    mem.setEditable(entity.getEditable());
                    projectMemberList.add(mem);
                }
            } else {
                member.setGroupId(member.getMemberId());
                projectMemberList.add(member);
            }
        }
        List<ProjectMember> singleMember = new ArrayList<ProjectMember>();
        Map<Org, List<ProjectMember>> resultMap = new HashMap<Org, List<ProjectMember>>();
        // step2: check membertype=group,compare members with group in account
        Map<String, List<ProjectMember>> orgMember = new HashMap<String, List<ProjectMember>>();
        // Get all users
        Set<String> ids = new HashSet<String>();
        for (ProjectMember member : projectMemberList) {
            if (member.getUserId() != null) {
                ids.add(member.getUserId());
            }
        }
        // Store the user object in a temporary map
        List<User> users = getUsersFromCache(ids);
        Map<String, User> userMap = new HashMap<String, User>();
        for (User user : users) {
            userMap.put(user.getIdStr(), user);
        }
        // Store the project member by organization id
        for (ProjectMember member : projectMemberList) {
            if (StringUtils.isEmpty(member.getGroupId()) && member.getIsOrg() != 1) {
                member.setMemberName(userMap.get(member.getUserId()).getDisplayName());
                singleMember.add(member);
            } else {
                if (orgMember.containsKey(member.getGroupId())) {
                    orgMember.get(member.getGroupId()).add(member);
                } else {
                    List<ProjectMember> memb = new ArrayList<ProjectMember>();
                    memb.add(member);
                    orgMember.put(member.getGroupId(), memb);
                }
            }
        }
        // Merge members with the users in the account
        //        for (Map.Entry<String, List<ProjectMember>> entry : orgMember.entrySet()) {
        //            Org org = webService.getOrgById(entry.getKey(), ownerId, true);
        //            List<Member> orgMembers = new ArrayList<Member>();
        //            List<Member> exitsMembers = new ArrayList<Member>();
        //            // Get all members of the organization
        //            getOrgMember(org, orgMembers);
        //            List<ProjectMember> mems = entry.getValue();
        //            // Define members not exist.
        //            List<ProjectMember> unExistMem = new ArrayList<ProjectMember>();
        //            // Define members exist.
        //            List<ProjectMember> existMem = new ArrayList<ProjectMember>();
        //            if (org != null && orgMembers.isEmpty()) {
        //                // if there are no members in this organization in the project
        //                // Just add this organization as a member
        //                existMem.add(entry.getValue().get(0));
        //            } else {
        //                for (ProjectMember member : mems) {
        //                    // Get user id
        //                    User user = userMap.get(member.getUserId());
        //                    for (Member mem : orgMembers) {
        //                        if (user != null && mem.getUserId().equals(user.getIdStr())) {
        //                            exitsMembers.add(mem);
        //                            member.setMemberName(user.getDisplayName());
        //                            existMem.add(member);
        //                        } else {
        //                            unExistMem.add(member);
        //                        }
        //                    }
        //                }
        //                // Get new member in the organization, these members are not invited
        //                orgMembers.removeAll(exitsMembers);
        //                // remove deleted member
        //                if (!unExistMem.isEmpty()) {
        //                    // mems.removeAll(unExistMem);
        //                    for (ProjectMember member : unExistMem) {
        //                        removeMember(member.getId(), false);
        //                    }
        //                }
        //                // add new member from org
        //                if (!orgMembers.isEmpty()) {
        //                    Set<String> idSet = new HashSet<String>();
        //                    for (Member member : orgMembers) {
        //                        idSet.add(member.getUserId());
        //                    }
        //                    List<User> userList = webService.getUserByIds(idSet);
        //                    for (User user : userList) {
        //                        if (user.getEmail() != null) {
        //                            userMap.put(user.getIdStr(), user);
        //                        }
        //                    }
        //                    for (Member member : orgMembers) {
        //                        ProjectMember mem = new ProjectMember();
        //                        User user = userMap.get(member.getUserId());
        //                        mem.setGroupId(mems.get(0).getGroupId());
        //                        mem.setMemberName(user.getDisplayName());
        //                        mem.setProjectId(mems.get(0).getProjectId());
        //                        mem.setRoleId(mems.get(0).getRoleId());
        //                        // mem.setMemberType(mems.get(0).getMemberType());
        //                        // Set member status -1:uninvited
        //                        mem.setStatus(-1);
        //                        mem.setUserId(user.getIdStr());
        //                        existMem.add(mem);
        //                    }
        //                }
        //            }
        //            resultMap.put(org, existMem);
        //        }
        // put all members not in an organization in a virtual organization: single
        Org singleOrg = new Org();
        singleOrg.setId("Single");
        resultMap.put(singleOrg, singleMember);
        List<MemberView> resultList = convert2ViewObject(resultMap, ownerId, userMap);
        resultList = mergeMemberViewList(resultList);
        return resultList;
    }

    /**
     * merge member in the same organization to a seperated memberview.
     * 
     * @param memberViewList
     * @return memberViewList
     */
    private List<MemberView> mergeMemberViewList(List<MemberView> memberViewList) {
        List<MemberView> finalResult = new ArrayList<MemberView>();
        Map<String, MemberView> viewMap = new HashMap<String, MemberView>();
        for (MemberView member : memberViewList) {
            if (member.isOrg()) {
                // If the member is in an organization
                MemberView baseMember = null;
                // Get base member view object
                if (viewMap.containsKey(member.getId())) {
                    baseMember = viewMap.get(member.getId());
                } else {
                    viewMap.put(member.getId(), member);
                    continue;
                }
                // Merge with base member view object
                merge(member, baseMember);
                viewMap.put(member.getId(), baseMember);
            } else {
                // If the member is a single member
                finalResult.add(member);
            }
        }
        // Convert to member view list
        for (Map.Entry<String, MemberView> entry : viewMap.entrySet()) {
            finalResult.add(entry.getValue());
        }
        return finalResult;
    }

    /**
     * Merge with base member view
     * 
     * @param source source member view
     * @param base base member view
     * @return true/false
     */
    private boolean merge(MemberView source, MemberView base) {
        // member equals condition:
        // If member is org: base.orgId=source.orgId,base.id=source.id
        // If member is not org:base.userId=source.userid,base.originalorgId=source.originalorgId
        // if (((base.isOrg() && source.isOrg()) || ((base.isOrg() == false && source.isOrg() == false) &&
        // base.getUserId().equals(source.getUserId())))
        // && (((base.getId() == null && source.getId() == null) ||((base.getId()!=null&&source.getId()!=null)&&
        // base.getId().equals(source.getId())))
        // && ((base.getOrgId() == null && source.getOrgId() == null) ||
        // (base.getOrgId()!=null&&source.getOrgId()!=null&&base.getOrgId().equals(base.getOrgId()))))) {
        if ((base.isOrg() && source.isOrg()
                && ((base.getId() == null && source.getId() == null) || base.getId().equals(source.getId())) && ((base
                .getOrgId() == null && source.getOrgId() == null) || base.getOrgId().equals(source.getOrgId())))
                || ((!base.isOrg() && !source.isOrg()) && base.getUserId().equals(source.getUserId()) && base
                        .getOriginalOrgId().equals(source.getOriginalOrgId()))) {
            if (source.isOrg()) {
                base.setNameNode((source.isNameNode() == true || base.isNameNode() == true));
                base.setEditable((source.isEditable() == true || base.isEditable() == true));
            }
            base.setStatus(source.getStatus() <= base.getStatus() ? base.getStatus() : source.getStatus());
            if (source.getMemberViews() != null && !source.getMemberViews().isEmpty()) {
                if (base.getMemberViews() == null || base.getMemberViews().isEmpty()) {
                    List<MemberView> children = new ArrayList<MemberView>();
                    children.addAll(source.getMemberViews());
                    base.setMemberViews(children);
                    // Set member count
                    base.setMemberCount(base.getMemberCount() + children.size());
                } else {
                    for (MemberView sourceChild : source.getMemberViews()) {
                        boolean result = false;
                        for (MemberView baseChild : base.getMemberViews()) {
                            result = result || merge(sourceChild, baseChild);
                        }
                        if (!result) {
                            base.getMemberViews().add(sourceChild);
                            // Set member count
                            base.setMemberCount(base.getMemberCount() + 1);
                        }
                    }
                }
            }
            return true;
        }
        return false;
    }

    /**
     * Get members in organization
     * 
     * @param org organization
     * @param members result list
     */
    private void getUninvitedOrgMember(String projectId, String roleId, Org org, List<Member> unIvitedMembers) {
        if (org != null) {
            if (org.getMembers() != null) {
                // If the organization has members
                for (Member mem : org.getMembers()) {
                    if (!checkInvited(projectId, roleId, mem.getOrgId(), mem.getUserId())) {
                        unIvitedMembers.add(mem);
                    }
                }
            }
            List<Org> subOrgs = org.getSubOrgs();
            if (subOrgs != null) {
                // If the organization has sub organizations
                for (Org subOrg : subOrgs) {
                    getUninvitedOrgMember(projectId, roleId, subOrg, unIvitedMembers);
                }
            }
        }
    }

    /**
     * Get members in organization
     * 
     * @param org organization
     * @param members result list
     */
    private void getOrgMember(Org org, List<Member> members) {
        if (org != null) {
            if (org.getMembers() != null) {
                // If the organization has members
                members.addAll(org.getMembers());
            }
            List<Org> subOrgs = org.getSubOrgs();
            if (subOrgs != null) {
                // If the organization has sub organizations
                for (Org subOrg : subOrgs) {
                    getOrgMember(subOrg, members);
                }
            }
        }
    }

    /**
     * Convert project member to member view
     * 
     * @param memberMap project members of the organization
     * @param ownerId owner
     * @param userMap users
     * @return member view list
     */
    private List<MemberView> convert2ViewObject(Map<Org, List<ProjectMember>> memberMap, String ownerId,
            Map<String, User> userMap) {
        // convert List to map
        Map<Org, Map<String, ProjectMember>> tempMap = new HashMap<Org, Map<String, ProjectMember>>();
        for (Map.Entry<Org, List<ProjectMember>> entry : memberMap.entrySet()) {
            Map<String, ProjectMember> pmMap = new HashMap<String, ProjectMember>();
            for (ProjectMember member : entry.getValue()) {
                pmMap.put(member.getUserId(), member);
            }
            tempMap.put(entry.getKey(), pmMap);
        }
        List<MemberView> members = new ArrayList<MemberView>();
        for (Map.Entry<Org, Map<String, ProjectMember>> entry : tempMap.entrySet()) {
            if ("Single".equals(entry.getKey().getId())) {
                // Process single project member
                for (Map.Entry<String, ProjectMember> subEntry : entry.getValue().entrySet()) {
                    MemberView member = new MemberView();
                    member.setId(subEntry.getValue().getId());
                    member.setName(subEntry.getValue().getMemberName());
                    member.setOrg(false);
                    member.setProjectId(subEntry.getValue().getProjectId());
                    member.setRoleId(subEntry.getValue().getRoleId());
                    // begin get user
                    User user = userMap.get(subEntry.getValue().getUserId());
                    // end
                    member.setAccount(user.getEmail());
                    // set user id
                    member.setUserId(user.getIdStr());
                    member.setStatus(subEntry.getValue().getStatus());
                    member.setMemberCount(0);
                    member.setEditable(subEntry.getValue().getEditable() == 0);
                    members.add(member);
                }
            } else {
                // begin get full org
                //                Org org = webService.getOrgPath(entry.getKey().getId(), ownerId);
                //                // end
                //                MemberView mem = new MemberView();
                //                mem = convert(org, entry.getKey(), tempMap, ownerId, userMap);
                //                members.add(mem);
            }
        }
        return members;
    }

    /**
     * merge organization added in the project with the full organization and set the detail of the members
     * 
     * @param org full organization from account
     * @param topOrg organization added in project
     * @param memberMap project member stored by organization id
     * @param ownerId
     * @param userMap user info map
     * @return member view
     */
    //    private MemberView convert(Org org, Org topOrg, Map<Org, Map<String, ProjectMember>> memberMap, String ownerId,
    //                               Map<String, User> userMap) {
    //        Integer memberCount = 0;
    //        Map<String, ProjectMember> pmMap = memberMap.get(topOrg);
    //        MemberView member = new MemberView();
    //        // get first member
    //        ProjectMember first = pmMap.entrySet().iterator().next().getValue();
    //        member.setId(org.getId());
    //        member.setName(org.getName());
    //        member.setOrg(true);
    //        // Set project Id 2013/03/04/19:06
    //        // member.setProjectId(first.getId());
    //        member.setProjectId(first.getProjectId());
    //        member.setRoleId(first.getRoleId());
    //        member.setStatus(first.getStatus());
    //        member.setNameNode(false);
    //        member.setEditable(false);
    //        if (org.getId().equals(topOrg.getId())) {
    //            member.setNameNode(true);
    //            // if the name node is true, this node can be removed
    //            member.setEditable(true);
    //            member.setStatus(1);
    //            if (!StringUtil.isBlank(first.getMemberId())) {
    //                member.setId(first.getMemberId());
    //            }
    //            org = topOrg;
    //        }
    //        List<MemberView> subMembers = new ArrayList<MemberView>();
    //        if (org.getMembers() != null && !org.getMembers().isEmpty()) {
    //            for (Member mem : org.getMembers()) {
    //                ProjectMember pm = pmMap.get(mem.getUserId());
    //                MemberView subMem = new MemberView();
    //                subMem.setId(pm.getId());
    //                subMem.setUserId(pm.getUserId());
    //                subMem.setName(pm.getMemberName());
    //                subMem.setOrg(false);
    //                subMem.setProjectId(pm.getProjectId());
    //                subMem.setRoleId(pm.getRoleId());
    //                // begin get user
    //                User usr = userMap.get(pm.getUserId());
    //                // end
    //                subMem.setAccount(usr.getEmail());
    //                subMem.setStatus(pm.getStatus());
    //                subMem.setMemberCount(0);
    //                subMem.setOrgId(pm.getGroupId());
    //                subMem.setEditable(pm.getEditable() == 0);
    //                // Stored original organization id
    //                subMem.setOriginalOrgId(mem.getOrgId());
    //                subMembers.add(subMem);
    //                memberCount++;
    //            }
    //        }
    //        if (org.getSubOrgs() != null && !org.getSubOrgs().isEmpty()) {
    //            for (Org subOrg : org.getSubOrgs()) {
    //                memberCount++;
    //                if (subOrg != null && subOrg.getId() != null) {
    //                    subMembers.add(convert(subOrg, topOrg, memberMap, ownerId, userMap));
    //                }
    //            }
    //        }
    //        member.setMemberCount(memberCount);
    //        member.setMemberViews(subMembers);
    //        return member;
    //    }
    @Override
    public List<Map<String, String>> showSuggestion(String userId) {
        List<Contact> communicateList = webService.getContactList(userId);
        //        List<Org> orgs = webService.getInstitution(userId);
        //        List<Member> members = new ArrayList<Member>();
        //        getOrgMembers(orgs, members);
        Set<String> ids = new HashSet<String>();
        // index map
        Map<String, Contact> map = new HashMap<String, Contact>();
        if (communicateList != null) {
            for (Contact communicate : communicateList) {
                ids.add(communicate.getContactUserId().toString());
            }
            for (Contact item : communicateList) {
                map.put(item.getContactUserId().toString(), item);
            }
        }

        //        for (Member member : members) {
        //            if (member.getUserId() != null) {
        //                ids.add(member.getUserId());
        //            }
        //        }

        // get all users
        List<User> users = webService.getUsers(ids);
        List<Map<String, String>> userlist = new ArrayList<Map<String, String>>();
        for (User item : users) {
            if (item != null) {
                Map<String, String> userInfo = new HashMap<String, String>();
                String tempProperty = null;
                // displayName
                if (map != null && map.get(item.getId()) != null) {
                    tempProperty = map.get(item.getId()).getContactUserDisplay();
                }
                if (StringUtils.isNotBlank(tempProperty)) {
                    userInfo.put("displayName", tempProperty);
                } else {
                    userInfo.put("displayName", item.getDisplayName());
                }
                // accountName
                userInfo.put("accountName", item.getAccountName());
                // set userId
                userInfo.put("accountId", item.getIdStr());
                userlist.add(userInfo);
            }
        }
        // Sort the suggestions order by first character
        Collections.sort(userlist, new Comparator<Map<String, String>>() {

            @Override
            public int compare(Map<String, String> o1, Map<String, String> o2) {
                if (o1.get("accountName").toCharArray()[0] > o2.get("accountName").toCharArray()[0]) {
                    return 1;
                } else if (o1.get("accountName").toCharArray()[0] < o2.get("accountName").toCharArray()[0]) {
                    return -1;
                }
                return 0;
            }
        });
        return userlist;
    }

    /**
     * Get all members of all organizations
     * 
     * @param orgs organization list
     * @param members result member list
     */
    //    private void getOrgMembers(List<Org> orgs, List<Member> members) {
    //        for (Org org : orgs) {
    //            getOrgMember(org, members);
    //        }
    //    }
    //
    //    @Override
    //    public List<MemberView> getOrg(String userId, String projectId, String roleId) {
    //        List<Org> orgs = webService.getInstitution(userId);
    //        List<MemberView> resultList = new ArrayList<MemberView>();
    //        convertOrg2MemberView(orgs, resultList, false);
    //        // Get organization added.
    //        List<ProjectMember> orgList = projectMemberDao.getAddedOrgs(projectId, roleId);
    //        Map<String, Integer> statusMap = new HashMap<String, Integer>();
    //        // Get project member by projectId and roleId
    //        for (ProjectMember entity : orgList) {
    //            if (entity != null) {
    //                statusMap.put(entity.getMemberId(), 1);
    //            }
    //        }
    //        setMemberStatus(resultList, statusMap, projectId, roleId);
    //        return resultList;
    //    }

    /**
     * Set status for each member
     * 
     * @param memberList members
     * @param statusMap status
     * @param projectId project id
     * @param roleId role id
     */
    private void setMemberStatus(List<MemberView> memberList, Map<String, Integer> statusMap, String projectId,
            String roleId) {
        for (MemberView member : memberList) {
            member.setProjectId(projectId);
            member.setRoleId(roleId);
            if (member.isOrg()) {
                member.setStatus(statusMap.get(member.getId()) == null ? -1 : statusMap.get(member.getId()));
            }
            if (member.getMemberViews() != null && !member.getMemberViews().isEmpty()) {
                setMemberStatus(member.getMemberViews(), statusMap, projectId, roleId);
            }
        }
    }

    /**
     * Convert organization to member view
     * 
     * @param orgs
     * @param resultList
     * @param needMembers
     */
    private void convertOrg2MemberView(List<Org> orgs, List<MemberView> resultList, boolean needMembers) {
        for (Org org : orgs) {
            MemberView member = new MemberView();
            member.setId(org.getId());
            member.setName(org.getName());
            member.setOrg(true);
            // Set uerid = orgId,if member is org
            member.setUserId(org.getId());
            member.setEditable(true);
            int count = 0;
            List<MemberView> subResultList = new ArrayList<MemberView>();
            if (needMembers && org.getMembers() != null && !org.getMembers().isEmpty()) {
                count = count + org.getMembers().size();
                for (Member subMem : org.getMembers()) {
                    MemberView subMember = new MemberView();
                    subMember.setAccount(subMem.getEmail());
                    subMember.setId(subMem.getUserId());
                    subMember.setName(subMem.getName());
                    subMember.setOrg(false);
                    subMember.setUserId(subMem.getUserId());
                    subMember.setEditable(true);
                    subResultList.add(subMember);
                }
            }
            if (org.getSubOrgs() != null && !org.getSubOrgs().isEmpty()) {
                count = count + org.getSubOrgs().size();
                convertOrg2MemberView(org.getSubOrgs(), subResultList, needMembers);
            }
            member.setMemberCount(count);
            member.setMemberViews(subResultList);
            resultList.add(member);
        }
    }

    /**
     * Convert organization to member view with status and name node attribute
     * 
     * @param fullOrg full organization instruct
     * @param sourceOrg organization added
     * @param statusMap status
     * @param resultList result list
     * @return member view
     */
    private void convertOrgWithNameNode(Org fullOrg, Org sourceOrg, Map<String, ProjectMember> statusMap,
            List<MemberView> resultList) {
        MemberView member = new MemberView();
        member.setId(fullOrg.getId());
        member.setName(fullOrg.getName());
        member.setOrg(true);
        // Set uerid = orgId,if member is org
        member.setUserId(fullOrg.getId());
        member.setEditable(false);
        if (fullOrg.getId().equals(sourceOrg.getId())) {
            // if the organization id equals the id in the full organization, set name node true
            member.setNameNode(true);
            member.setEditable(true);
            fullOrg = sourceOrg;
        }
        int count = 0;
        List<MemberView> subResultList = new ArrayList<MemberView>();
        if (fullOrg.getMembers() != null && !fullOrg.getMembers().isEmpty()) {
            count = count + fullOrg.getMembers().size();
            for (Member subMem : fullOrg.getMembers()) {
                MemberView subMember = new MemberView();
                subMember.setAccount(subMem.getEmail());
                subMember.setId(subMem.getUserId());
                subMember.setName(subMem.getName());
                subMember.setOrg(false);
                subMember.setUserId(subMem.getUserId());
                // if the member is invited in another sub organization,set status invited.
                subMember.setStatus(statusMap.get(subMem.getUserId()) == null ? 0 : statusMap.get(subMem.getUserId())
                        .getStatus());
                subMember.setEditable(true);
                subResultList.add(subMember);
            }
        }
        if (fullOrg.getSubOrgs() != null && !fullOrg.getSubOrgs().isEmpty()) {
            count = count + fullOrg.getSubOrgs().size();
            for (Org subOrg : fullOrg.getSubOrgs()) {
                convertOrgWithNameNode(subOrg, sourceOrg, statusMap, subResultList);
            }
        }
        member.setMemberCount(count);
        member.setMemberViews(subResultList);
        resultList.add(member);
    }

    @Override
    public List<MemberView> showProjectMembers(String projectId, String ownerId) {
        List<ProjectMember> members = new ArrayList<ProjectMember>();
        // step1: list all members
        members = getMembers(projectId, null);
        // convert project member to memberview
        List<MemberView> resultList = new ArrayList<MemberView>();
        // set user id
        Set<String> ids = new HashSet<String>();
        for (ProjectMember projectMember : members) {
            MemberView member = new MemberView();
            member.setId(projectMember.getUserId());
            ids.add(projectMember.getUserId());
            member.setProjectId(projectMember.getProjectId());
            member.setRoleId(projectMember.getRoleId());
            resultList.add(member);
        }
        // Get user list
        List<User> users = webService.getUsers(ids);
        Map<String, User> userMap = new HashMap<String, User>();
        for (User user : users) {
            userMap.put(user.getIdStr(), user);
        }
        for (MemberView mem : resultList) {
            if (userMap.get(mem.getId()) != null) {
                mem.setName(userMap.get(mem.getId()).getDisplayName());
            }
            mem.setAccount(userMap.get(mem.getId()).getEmail());
        }
        return resultList;
    }

    @Override
    public boolean updateInvitationStatus(String email, String token) {
        return invitationDao.updateInvitationStatus(email, token) > 0;
    }

    @Override
    public ProjectMemberAttendees getAttendeeById(String id) {
        return attendeesDao.get(id);
    }

    @Override
    public ProjectMember getMember(String projectId, String roleId, String memberId) {
        return projectMemberDao.getMember(projectId, roleId, memberId, null);
    }

    @Override
    public boolean setInvitaionInvalid(String id) {
        return invitationDao.setInvitaionInvalid(id) > 0;
    }

    public void setFileService(FileService fileService) {
        this.fileService = fileService;
    }

    public void setWebService(WebService webService) {
        this.webService = webService;
    }

    @Override
    public List<ProjectTemplate> getAllTemplates(String userId, String hasCommon) {
        return projectTemplateDao.getAllTemplates(userId, hasCommon);
    }

    @Override
    public List<ProjectTemplateResource> getTemplateResource(String templateId) {
        return projectTemplateResourceDao.getTemplateResource(templateId);
    }

    @Override
    public void createResourceByTemplate(File rootfile, String templateId, Project project, String userId)
            throws IOException {
        List<ProjectTemplateResource> resources = getTemplateResource(templateId);
        Map<String, String> roleMap = new HashMap<String, String>();
        boolean hasFolder = false;
        int folderCount = 0;
        if (resources != null && !resources.isEmpty()) {
            hasFolder = true;
        }
        for (ProjectTemplateResource resource : resources) {
            if (resource.getType() == 0) {
                folderCount++;
                // Process folder
                String fileId = fileService.createFolder(userId, project.getFolderId(), resource.getPath()).getId();
                //                try {
                //                    fileId = fileService.getFileId(project.getFolderId(), resource.getPath());
                //                } catch (Exception e) {
                //                    LOGGER.info("The file is not found.exception:" + e.getMessage());
                //                }
                //                if (StringUtil.isEmpty(fileId)) {
                //                    // The folder is not created.Create the folder
                //                    File file = fileService.createFolder(userId, project.getFolderId(), resource.getPath());
                //                    fileId = file.getId();
                //                }
                if (resource.getPrivileges() != null) {
                    for (ProjectTemplatePrivilege privilege : resource.getPrivileges()) {
                        String roleId = "";
                        // Set role
                        if (StringUtil.isEmpty(roleMap.get(privilege.getRoleName()))) {
                            ProjectRole role = createRole(userId, project.getId(), privilege.getRoleName());
                            roleId = role.getId();
                            roleMap.put(privilege.getRoleName(), roleId);
                        } else {
                            roleId = roleMap.get(privilege.getRoleName());
                        }
                        // Set privilege
                        privilegeService.savePrivilege(userId, fileId, roleId, privilege.getPrivilege());
                    }
                }
            } else {
                // Get file from storage
                //                InputStream input = null;
                StorageFile storageFile = storage.getFile("/project_template/" + templateId + "/" + resource.getPath());
                //                try {
                //                    input = storageFile.getInputStream();
                //                } catch (IOException e1) {
                //                	throw new DocumentPaasNotFoundException(DocumentErrorCode.DATA_NOT_FOUND, "can not find file[" + resource.getPath() + "]", null);
                //                }
                //                // process file
                //                int index = resource.getPath().lastIndexOf(StringConst.SEPARATOR_SLASH);
                //                String path = "";
                //                // Get folder
                //                String fileId = null;
                //                if (index != -1) {
                //                    path = resource.getPath().substring(0, index);
                //                    try {
                //                        fileId = fileService.getFileId(project.getFolderId(), path);
                //                    } catch (Exception e) {
                //                        LOGGER.info("The file is not found.exception:" + e.getMessage());
                //                    }
                //                    if (StringUtil.isEmpty(fileId)) {
                //                        // The folder is not created.Create the folder
                //                        File file = fileService.createFolder(userId, project.getFolderId(), path);
                //                        fileId = file.getId();
                //                    }
                //                } else {
                //                    // if the file is in the root folder
                //                    fileId = project.getFolderId();
                //                }
                //                String fileName = resource.getPath().substring(resource.getPath().lastIndexOf(StringConst.SEPARATOR_SLASH) + 1);
                // Upload file:inputstream
                fileService.uploadFile(userId, project.getFolderId(), resource.getPath(), 0, storageFile.length(),
                        storageFile.getInputStream());
            }
        }
        rootfile.setHasFolder(hasFolder);
        rootfile.setFolderCount(folderCount);
    }

    /*
     * (non-Javadoc)
     * @see com.glodon.paas.document.service.ProjectService#removeMemberAndAttendee(java.lang.String, java.lang.String,
     * boolean)
     */
    @Override
    public boolean removeMemberAndAttendee(String memberId, String attendeeId, boolean deleteMember) {
        if (StringUtil.isEmpty(attendeeId)) {
            removeMember(memberId, deleteMember);
        } else {
            if (deleteMember) {
                projectMemberDao.delete(memberId);
            }
            attendeesDao.removeAttedees(memberId, attendeeId);
        }
        return true;
    }

    /*
     * (non-Javadoc)
     * @see com.glodon.paas.document.service.ProjectService#createTemplate(java.lang.String,
     * com.glodon.paas.document.api.bean.ProjectTemplate, java.lang.String)
     */
    @Override
    public ProjectTemplate createTemplate(String projectId, String templateName, String ownerId) {
        Project project = this.getProject(projectId);
        ProjectTemplate entity = new ProjectTemplate();
        entity.setId(UUIDUtil.getUUIDStr());
        entity.setName(templateName);
        entity.setOwnerId(ownerId);
        User user = new User();
        user.setId(ownerId);
        entity.setCreator(user);
        entity.setUpdator(user);
        entity.setStatus(0);
        entity.setCreateTime(new Date());
        entity.setUpdateTime(entity.getCreateTime());
        // insert template begin
        projectTemplateDao.insert(entity);
        // end
        createTemplate(entity.getId(), projectId, project.getFolderId(), ownerId, null);
        return entity;
    }

    private void createTemplate(String templateId, String projectId, String folderId, String creator, String parentPath) {
        List<File> files = fileService.getFiles(folderId);
        if (parentPath == null) {
            parentPath = "";
        }
        User user = new User();
        user.setId(creator);
        for (File file : files) {
            String path = "".equals(parentPath) ? file.getFullName() : parentPath + StringConst.SEPARATOR_SLASH
                    + file.getFullName();
            if (file.isFolder()) {
                ProjectTemplateResource resourceEntity = new ProjectTemplateResource();
                resourceEntity.setId(UUIDUtil.getUUIDStr());
                resourceEntity.setTemplateId(templateId);
                resourceEntity.setType(0);
                // Get folder path
                // String path = fileService.getPath(projectId, file.getId(), false);
                // resourceEntity.setPath(path.substring(path.indexOf(StringConst.SEPARATOR_SLASH)+1));
                resourceEntity.setPath(path);
                resourceEntity.setCreator(user);
                resourceEntity.setUpdator(user);
                resourceEntity.setCreateTime(new Date());
                resourceEntity.setUpdateTime(resourceEntity.getCreateTime());
                // insert template resource begin
                projectTemplateResourceDao.insert(resourceEntity);
                // end
                // get roles
                List<ProjectRole> roles = getProjectRoles(creator, projectId);
                for (ProjectRole role : roles) {
                    if (!role.isDefault() && role.isEditable()) {
                        // ignore default role and owner
                        List<ProjectPrivilege> resources = privilegeService.getFileRolePrivilege(creator, file.getId(),
                                role.getId());
                        for (ProjectPrivilege resource : resources) {
                            ProjectTemplatePrivilege privilege = new ProjectTemplatePrivilege();
                            privilege.setId(UUIDUtil.getUUIDStr());
                            privilege.setResourceId(resourceEntity.getId());
                            privilege.setRoleName(role.getRoleName());
                            privilege.setPrivilege(resource.getPrivilegeStr());
                            privilege.setCreator(user);
                            privilege.setUpdator(user);
                            privilege.setCreateTime(DateUtil.getNow());
                            privilege.setUpdateTime(resourceEntity.getCreateTime());
                            // Insert privileges
                            projectTemplatePrivilegeDao.insert(privilege);
                        }
                    }
                }
                createTemplate(templateId, projectId, file.getId(), creator, path);
            } else {
                // process file
                // File fileRevision = File.getDto(fileService.getFile(file.getId()), false);
                Revision revisionEntity = revisionService.getRevision(file.getId(), file.getRevisionNumber());
                // copy file by storage: storage key begin
                String originalStorageId = revisionEntity.getFileId();
                String storagePath = "/project_template/" + templateId + "/" + path;
                //                                     + (path.lastIndexOf(StringConst.SEPARATOR_SLASH) == -1 ? path : path.substring(0, path.lastIndexOf(StringConst.SEPARATOR_SLASH)));
                storage.copyBySourceId(originalStorageId, storagePath);
                // end
                ProjectTemplateResource resourceEntity = new ProjectTemplateResource();
                resourceEntity.setId(UUIDUtil.getUUIDStr());
                resourceEntity.setTemplateId(templateId);
                resourceEntity.setType(1);
                resourceEntity.setPath(path);
                //                                       + (StringUtil.isEmpty(file.getExtension()) ? "" : "." + file.getExtension()));
                //                resourceEntity.setFileKey(file.getDigest());
                resourceEntity.setCreator(user);
                resourceEntity.setUpdator(user);
                resourceEntity.setCreateTime(DateUtil.getNow());
                resourceEntity.setUpdateTime(resourceEntity.getCreateTime());
                // insert template resource begin
                projectTemplateResourceDao.insert(resourceEntity);
            }
        }
    }

    //    private String convertArray2String(String[] array) {
    //        StringBuffer sb = new StringBuffer();
    //        for (int i = 0; i < array.length; i++) {
    //            if (i == 0) {
    //                sb.append(array[i]);
    //            } else {
    //                sb.append(StringConst.SEPARATOR_COMMA).append(array[i]);
    //            }
    //        }
    //        return sb.toString();
    //    }

    //    /*
    //     * (non-Javadoc)
    //     * @see com.glodon.paas.document.service.ProjectService#checkAdministrator(java.lang.String, java.lang.String)
    //     */
    //    @Override
    //    public boolean checkAdministrator(String projectId, String userId) {
    //        return projectRoleDao.checkAdministrator(projectId, userId) > 0;
    //    }

    /*
     * (non-Javadoc)
     * @see com.glodon.paas.document.service.ProjectService#checkSameTemplateName(java.lang.String, java.lang.String)
     */
    @Override
    public boolean checkSameTemplateName(String userId, String templateName) {
        return projectTemplateDao.checkSameTemplateName(userId, templateName) > 0;
    }

    /*
     * (non-Javadoc)
     * @see
     * com.glodon.paas.document.service.ProjectService#updateTemplate(com.glodon.paas.document.api.bean.ProjectTemplate,
     * java.lang.String)
     */
    @Override
    public boolean updateTemplate(ProjectTemplate template, String userId) {
        ProjectTemplate entity = new ProjectTemplate();
        entity.setId(template.getId());
        entity.setName(template.getName());
        User user = new User();
        user.setId(userId);
        entity.setUpdator(user);
        entity.setUpdateTime(new Date());
        return projectTemplateDao.updateTemplate(entity) > 0;
    }

    /*
     * (non-Javadoc)
     * @see com.glodon.paas.document.service.ProjectService#getTemplate(java.lang.String)
     */
    @Override
    public ProjectTemplate getTemplate(String id) {
        return projectTemplateDao.get(id);
    }

    public String createProject(Project project, File folder, ProjectRole role, ProjectRole defaultRole,
            ProjectMember member, ProjectMemberAttendees attendee, ProjectPrivilege ownerRoleResource,
            ProjectPrivilege defaultRoleResource) {
        projectDao.insert(project);
        //        fileService.insertFile(folder);
        projectMemberDao.insert(member);
        attendeesDao.insert(attendee);
        projectRoleDao.insert(role);
        projectRoleDao.insert(defaultRole);
        projectPrivilegeDao.insert(ownerRoleResource);
        projectPrivilegeDao.insert(defaultRoleResource);
        return project.getId();
    }

    public boolean deleteProject(String id) {
        boolean result = false;
        try {
            result = projectDao.delete(id) > 0;
            if (result) {
                projectDao.deleteProjectRelation(id);
            }
        } catch (Exception e) {
            LOGGER.error(e.getMessage(), e);
            throw new DocumentPaasDBException(DocumentErrorCode.PERSISTENCE_ERROR, e.getLocalizedMessage(), e);
        }
        return result;
    }

    public boolean updateProject(Project entity) {
        boolean result = false;
        try {
            result = projectDao.updateProject(entity) > 0;
        } catch (Exception e) {
            LOGGER.error(e.getMessage(), e);
            throw new DocumentPaasDBException(DocumentErrorCode.PERSISTENCE_ERROR, e.getLocalizedMessage(), e);
        }
        return result;
    }

    public String createRole(ProjectRole role, ProjectPrivilege resource) {
        projectRoleDao.insert(role);
        projectPrivilegeDao.insert(resource);
        return role.getId();
    }

    public boolean deleteRole(String roleId) {
        boolean result = false;
        try {
            result = projectRoleDao.delete(roleId) > 0;
        } catch (Exception e) {
            LOGGER.error(e.getMessage(), e);
            throw new DocumentPaasDBException(DocumentErrorCode.PERSISTENCE_ERROR, e.getMessage(), e);
        }
        return result;
    }

    @Override
    public String insertMember(ProjectMember entity) {
        String result = "";
        try {
            result = projectMemberDao.insert(entity);
        } catch (Exception e) {
            LOGGER.error(e.getMessage(), e);
            throw new DocumentPaasDBException(DocumentErrorCode.PERSISTENCE_ERROR, e.getMessage(), e);
        }
        return result;
    }

    @Override
    public boolean removeMember(String memberId, boolean deleteProjectMember) {
        if (deleteProjectMember) {
            projectMemberDao.delete(memberId);
        }
        attendeesDao.removeAttedees(memberId, null);
        return true;
    }

    public void processInvitedOrgMember(String newMemberId, String projectId, String roleId, String[] orgIds) {
        // step1: move attendees to new member
        attendeesDao.updateAttendeeMemberId(newMemberId, projectId, roleId, orgIds);
        // step2: remove old members
        projectMemberDao.removeMembers(projectId, roleId, orgIds);
    }

    public ProjectMemberAttendees getAttendeeByUserId(String projectId, String roleId, String userId) {
        return attendeesDao.getAttendeeByUserId(projectId, roleId, userId);
    }

    @Override
    public List<Invitation> getInvitaionsOfUser(String userId, String token) {
        Invitation invitation = invitationDao.getInvitation(userId, token);
        if (invitation == null) {
            throw new DocumentPaasValidateException(DocumentErrorCode.INVALID_INVITATION, "Invalid invitation.",
                    new Exception("Invalid invitation."));
        }
        ProjectMember member = projectMemberDao.get(invitation.getMemberId());
        return invitationDao.getInvitaionsOfUser(member.getProjectId(), userId);
    }

    @Override
    public boolean deleteTemplate(String templateId) {
        // delete template privilege
        projectTemplatePrivilegeDao.deleteByTemplateId(templateId);
        // delete template resource
        projectTemplateResourceDao.deleteByTemplateId(templateId);
        // delete template
        return projectTemplateDao.deleteTemplate(templateId) > 0;
    }

    @Override
    public int deleteProjectRelation(String userId, String projectId, String roleId, String orgId) {
        return projectDao.deleteProjectRelation(userId, projectId, roleId, orgId);
    }

    @Override
    public List<ProjectRelation> getProjectRelationsByUserId(String userId) {
        return projectDao.selectProjectRelationsByUserId(userId);
    }

    public void setProjectDao(ProjectDao projectDao) {
        this.projectDao = projectDao;
    }

    public void setProjectMemberDao(ProjectMemberDao projectMemberDao) {
        this.projectMemberDao = projectMemberDao;
    }

    public void setAttendeesDao(ProjectMemberAttendeesDao attendeesDao) {
        this.attendeesDao = attendeesDao;
    }

    public void setProjectRoleDao(ProjectRoleDao projectRoleDao) {
        this.projectRoleDao = projectRoleDao;
    }

    public List<ProjectPrivilege> loadUserResource(String userId) {
        return projectPrivilegeDao.selectByUser(userId);
    }

    public void setInvitationDao(InvitationDao invitationDao) {
        this.invitationDao = invitationDao;
    }

    public void setProjectTemplateDao(ProjectTemplateDao projectTemplateDao) {
        this.projectTemplateDao = projectTemplateDao;
    }

    public void setPrivilegeService(PrivilegeService privilegeService) {
        this.privilegeService = privilegeService;
    }

    public void setProjectTemplateResourceDao(ProjectTemplateResourceDao projectTemplateResourceDao) {
        this.projectTemplateResourceDao = projectTemplateResourceDao;
    }

    public void setProjectTemplatePrivilegeDao(ProjectTemplatePrivilegeDao projectTemplatePrivilegeDao) {
        this.projectTemplatePrivilegeDao = projectTemplatePrivilegeDao;
    }

    public void setProjectPrivilegeDao(ProjectPrivilegeDao projectPrivilegeDao) {
        this.projectPrivilegeDao = projectPrivilegeDao;
    }

    public void setUserCache(DocumentPaasUserCache userCache) {
        this.userCache = userCache;
    }

    public List<Long> getAllAttendeesIdForActivity(String projectId, File file) {
        if (file == null)
            return null;
        List<Long> userList = new ArrayList<Long>();
        List<Long> resultList = new ArrayList<Long>();
        userList = projectMemberDao.getAllAttendeesId(projectId);
        String fileId = null;
        if (file.isFolder()) {
            fileId = file.getId();
        } else {
            fileId = file.getParentId();
        }
        for (Long userId : userList) {
            Set<String> privilegs = privilegeService.getFilePrivileges(String.valueOf(userId), fileId);
            if (privilegs.contains("r")) {
                resultList.add(userId);
            }
        }

        return resultList;
    }

    public void setUrlDeteminator(Deteminator<String> urlDeteminator) {
        this.urlDeteminator = urlDeteminator;
    }
}
