package org.spring.heat.services;

import java.math.BigDecimal;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Timestamp;
import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Date;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import javax.servlet.ServletContext;
import javax.servlet.http.HttpSession;

import org.hibernate.Query;
import org.hibernate.Session;
import org.hibernate.SessionFactory;
import org.hibernate.Transaction;
import org.hibernate.heat.domain.BuildingInfo;
import org.hibernate.heat.domain.BuildingInfoDAO;
import org.hibernate.heat.domain.CommunityInfo;
import org.hibernate.heat.domain.CommunityInfoDAO;
import org.hibernate.heat.domain.CountyInfo;
import org.hibernate.heat.domain.DueCharge;
import org.hibernate.heat.domain.HeatsysSetting;
import org.hibernate.heat.domain.MachinesetInfo;
import org.hibernate.heat.domain.MachinesetInfoDAO;
import org.hibernate.heat.domain.OperationInfo;
import org.hibernate.heat.domain.ProjectInfo;
import org.hibernate.heat.domain.ProjectInfoDAO;
import org.hibernate.heat.domain.StaffInfo;
import org.hibernate.heat.domain.StchgDaily;
import org.hibernate.heat.domain.StchgSnapshot;
import org.hibernate.heat.domain.UnitInfo;
import org.hibernate.heat.domain.UnitInfoDAO;
import org.hibernate.heat.domain.UnitPrice;
import org.hibernate.heat.domain.UnitPriceDAO;
import org.hibernate.heat.domain.UsersInfo;
import org.hibernate.heat.domain.UsersInfoDAO;
import org.hibernate.heat.domain.UsersStatus;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.spring.heat.entity.MachineInfo;
import org.spring.heat.entity.ResultForMachine;
import org.spring.heat.entity.StaffPermission;
import org.spring.heat.entity.UserInfo;
import org.spring.heat.models.ChangeList;
import org.spring.heat.models.Global;
import org.spring.heat.models.ListCount;
import org.spring.heat.models.Result;
import org.spring.heat.models.StatisData;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.ApplicationContext;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.RowCallbackHandler;
import org.springframework.orm.hibernate3.SessionFactoryUtils;
import org.springframework.stereotype.Service;
import org.springframework.transaction.PlatformTransactionManager;
import org.springframework.transaction.TransactionStatus;
import org.springframework.transaction.support.DefaultTransactionDefinition;
import org.springframework.web.context.support.WebApplicationContextUtils;

@Service
public class UserManagerService{
	
	@Autowired
	private UsersInfoDAO dao;
	
	@Autowired
	private BuildingInfoDAO buildingDAO;
	
	@Autowired
	private CommunityInfoDAO communityDAO;
	
	@Autowired
	private ProjectInfoDAO projectInfoDAO;
	
	@Autowired
	private MachinesetInfoDAO machinesetInfoDAO;
	
	@Autowired
	private UnitPriceDAO unitPriceDAO;
	
	@Autowired
	private UnitInfoDAO unitInfoDAO;
	
	private JdbcTemplate jdbcTemplate;
	
	private PlatformTransactionManager jdbcTransactionManager;
	
	public PlatformTransactionManager getJdbcTransactionManager() {
		return jdbcTransactionManager;
	}
	public void setJdbcTransactionManager(
			PlatformTransactionManager jdbcTransactionManager) {
		this.jdbcTransactionManager = jdbcTransactionManager;
	}
	
	public JdbcTemplate getJdbcTemplate() {
		return jdbcTemplate;
	}
	public void setJdbcTemplate(JdbcTemplate jdbcTemplate) {
		this.jdbcTemplate = jdbcTemplate;
	}
	private static final Logger log = LoggerFactory.getLogger(UserManagerService.class);
	/*获取项目信息*/
	public  ArrayList<ArrayList<Object>> getProjectInfo(){
		log.debug("UserManagerService.getProjectInfo");
		ArrayList<ArrayList<Object>> list = new ArrayList<ArrayList<Object>>();
		String sql = "select pjtid,pjtname from project_info where isvalid = 1";
		List rows = this.getJdbcTemplate().queryForList(sql);
		Iterator it = rows.iterator();
		while(it.hasNext()) {      
			Map map = (Map) it.next();   
			ArrayList<Object> temp = new ArrayList<Object>();
			temp.add(((BigDecimal)(map.get("pjtid"))).longValue());
			temp.add((map.get("pjtname")));
			list.add(temp);
		} 
		return list;
	}
	/*获取社区信息*/
	public  ArrayList<ArrayList<Object>> getCommunityInfo(){
		log.debug("UserManagerService.getCommunityInfo");
		ArrayList<ArrayList<Object>> list = new ArrayList<ArrayList<Object>>();
		String sql = "select cmtid,cmtname from community_info where isvalid = 1";
		List rows = this.getJdbcTemplate().queryForList(sql);
		Iterator it = rows.iterator();
		while(it.hasNext()) {      
			Map map = (Map) it.next();   
			ArrayList<Object> temp = new ArrayList<Object>();
			temp.add(((BigDecimal)(map.get("cmtid"))).longValue());
			temp.add((map.get("cmtname")));
			list.add(temp);
		} 
		return list;
	}
	
	/*获取机组信息*/
	public  ResultForMachine getMachineInfo(Long bldid){
		log.debug("UserManagerService.getMachineInfo");
		ResultForMachine reslt = new ResultForMachine();
		
		String sql = "select m.mchid,mchname from machineset_info m,building_info b " +
				"where b.bldid = "+bldid+" and b.mchid = m.mchid and m.isvalid =1 and b.isvalid =1";
		List rows = this.getJdbcTemplate().queryForList(sql);
		if(rows.size()==0 || rows == null){
			reslt.setSuccess(false);
			reslt.setMessage("楼栋未关联机组");
			reslt.setData(null);
			return reslt;
		}
		MachineInfo machine = new MachineInfo();
		Map map = (Map)rows.get(0);
		machine.setMchid(((BigDecimal)map.get("mchid")).longValue());
		machine.setMchname((String)map.get("mchname"));
		reslt.setSuccess(true);
		reslt.setMessage("");
		reslt.setData(machine);
		return reslt;
	}
	
	/*根据社区号，获取此社区所有楼栋*/
	public  ArrayList<ArrayList<Object>> getBuildingInfo(Long cmtid){
		log.debug("UserManagerService.getBuildingInfo");
		ArrayList<ArrayList<Object>> list = new ArrayList<ArrayList<Object>>();
		String sql = "select bldid,bldname  from building_info where isvalid = 1 and  cmtid="+cmtid;
		List rows = this.getJdbcTemplate().queryForList(sql);
		Iterator it = rows.iterator();
		while(it.hasNext()) {      
			Map map = (Map) it.next();   
			ArrayList<Object> temp = new ArrayList<Object>();
			temp.add(((BigDecimal)(map.get("bldid"))).longValue());
			temp.add((map.get("bldname")));
			list.add(temp);
		} 
		return list;
	}
	/*根据楼栋号，获取此楼栋所有单元*/
	public  ArrayList<ArrayList<Object>> getUnitInfo(Long bldid){
		log.debug("UserManagerService.getUnitInfo");
		ArrayList<ArrayList<Object>> list = new ArrayList<ArrayList<Object>>();
		String sql = "select untid,untname  from unit_info  where isvalid =1 and bldid="+bldid;
		List rows = this.getJdbcTemplate().queryForList(sql);
		Iterator it = rows.iterator();
		while(it.hasNext()) {      
			Map map = (Map) it.next();   
			ArrayList<Object> temp = new ArrayList<Object>();
			temp.add(((BigDecimal)(map.get("untid"))).longValue());
			temp.add((map.get("untname")));
			list.add(temp);
		} 
		return list;
	}
	
	public StchgDaily getStchgDaily(Date stdate, ProjectInfo projectInfo, StaffInfo staffInfo, BuildingInfo buildingInfo, Session hsession)
	{
		Query query;
		if (staffInfo != null)
		{
			String hql = "from StchgDaily as st " +
					"where st.stdate=:stdate and st.staffInfo.stfid=:stfid and st.projectInfo.pjtid=:pjtid and st.buildingInfo.bldid=:bldid and st.isvalid=:isvalid";
			query = hsession.createQuery(hql);
			query.setBoolean("isvalid", true);
			query.setLong("stfid", staffInfo.getStfid());
			query.setLong("pjtid", projectInfo.getPjtid());
			query.setLong("bldid", buildingInfo.getBldid());
			query.setTimestamp("stdate", stdate);
			
		}
		else
		{
			String hql = "from StchgDaily as st " +
					"where st.stdate=:stdate and st.staffInfo.stfid=null and st.projectInfo.pjtid=:pjtid and st.buildingInfo.bldid=:bldid and st.isvalid=:isvalid";
			query = hsession.createQuery(hql);
			query.setBoolean("isvalid", true);
			query.setLong("pjtid", projectInfo.getPjtid());
			query.setLong("bldid", buildingInfo.getBldid());
			query.setTimestamp("stdate", stdate);
		}
		
		List list = query.list();
		
		if (list.size() > 0 )
		{
			return (StchgDaily)list.get(0);
		}
		else
		{
			StchgDaily sd = new StchgDaily();
			sd.setBuildingInfo(buildingInfo);
			sd.setProjectInfo(projectInfo);
			sd.setStdate(stdate);
			sd.setStaffInfo(staffInfo);
			return sd;
		}
	}
	
	/*新建或者更新用户*/	
	public Result insertUserSession(Long usrid,String usertype,String address,String usrname,
			String phone,String startdate,String contractdate,String contracttype,String contractver,String contractpic,
			String idpic,String houseidpic,String housepic,String desp,Double area,Double realarea,Double feearea,
			String feetype,Double feerate,Double discount,Double reducefee,String heatstate,Double heatbase,Double heatrate,
			String housetype,Double uncheckbalance,Double balance,Double uncheckmoney,Double money,Double needmoney,String lastdate,Long pjtid,
			Long cmtid,Long bldid,Long untid,Long mchid,String genAccount, String upid, HttpSession session){
		Result result = new Result();
		ServletContext servletContext = session.getServletContext();    
        ApplicationContext ctx = WebApplicationContextUtils.getWebApplicationContext(servletContext);
        SessionFactory sessionFactory = (SessionFactory) ctx.getBean("sessionFactory");
        Session hsession = SessionFactoryUtils.getSession(sessionFactory, true);
        Transaction transaction = hsession.beginTransaction();
        String hql;
        try{
        	Query query = hsession.createQuery("from HeatsysSetting");
			HeatsysSetting heatsysSetting = (HeatsysSetting)query.list().get(0);
        	DateFormat format = new SimpleDateFormat("yyyy-MM-dd");         
        	UsersInfo user = new UsersInfo();
        	user.setUsrid(usrid);
			user.setUsertype(usertype);
			user.setAddress(address);
			user.setUsrname(usrname);
			user.setPhone(phone);
			user.setStartdate(format.parse(startdate));
			user.setContractdate(format.parse(contractdate));
			user.setContracttype(contracttype);
			user.setContractver(contractver);
			user.setContractpic(contractpic);
			user.setIdpic(idpic);
			user.setHouseidpic(houseidpic);
			user.setHousepic(housepic);
			user.setDesp(desp);
			user.setArea(area);
			user.setRealarea(realarea);
			user.setFeearea(feearea);
			user.setFeetype(feetype);
			user.setFeerate(feerate);
			user.setDiscount(discount);
			user.setReducefee(reducefee);
			user.setHeatstate(heatstate);
			user.setHeatbase(heatbase);
			user.setHeatrate(heatrate);
			user.setHousetype(housetype);
			user.setBalance(balance);
			user.setUncheckbalance(uncheckbalance);
			user.setMoney(money);
			user.setUncheckmoney(uncheckmoney);
			if(genAccount==null||!genAccount.equals("on")){
				user.setNeedmoney(needmoney);
			}else if(genAccount.equals("on")){
				user.setNeedmoney(feearea*feerate);
			}else
				user.setNeedmoney(needmoney);
			user.setLastdate((lastdate == null) ? null : format.parse(lastdate));
			ProjectInfo project = new ProjectInfo(pjtid);
			user.setProjectInfo(project);
			
			query = hsession.createQuery("from UsersInfo as u where u.buildingInfo.bldid=:bldid and u.communityInfo.cmtid=:cmtid and u.unitInfo.untid=:untid and u.isvalid=:isvalid and u.address=:address");
			query.setLong("bldid", bldid);
			query.setLong("cmtid", cmtid);
			query.setLong("untid", untid);
			query.setString("address", address);
			query.setBoolean("isvalid", true);
			List rows = query.list();
			if(rows.size()!=0)
			{
				throw new Exception("房间号在该楼栋中已经存在！");
			}
			
			CommunityInfo community = new CommunityInfo(cmtid);
			user.setCommunityInfo(community);
			BuildingInfo building = new BuildingInfo(bldid);
			user.setBuildingInfo(building);
			UnitInfo unit = new UnitInfo(untid);
			user.setUnitInfo(unit);
			MachinesetInfo machineset = new MachinesetInfo(mchid);
			user.setMachinesetInfo(machineset);
			UnitPrice unitprice = new UnitPrice(Long.parseLong(upid));
			user.setUnitPrice(unitprice);
			user.setCurnotpay(genAccount.equals("on") ? (long)1:(long)0);
			user.setLastnotpay((long)0);
			user.setIsvalid(true);
			hsession.saveOrUpdate(user);
			
			
			if(genAccount!=null&&genAccount.equals("on")){//表示sheng cheng xin zhang qi
				DueCharge dueCharge = new DueCharge();
				dueCharge.setCharge(user.getFeerate()*user.getArea());
				dueCharge.setDealname(heatsysSetting.getCurdealname());
				dueCharge.setChgtype(Global.DUECHARGECHGTYPE);
				dueCharge.setArea(user.getArea());
				dueCharge.setRate(user.getFeerate());
				dueCharge.setReducechg(user.getReducefee());
				dueCharge.setStatus(Global.DUECHARGESTATUS);
				dueCharge.setMoney((double)0);
				dueCharge.setUncheckmoney((double)0);
				dueCharge.setLastchgtime(null);
				dueCharge.setIsvalid(true);
				dueCharge.setUsersInfo(user);
				hsession.saveOrUpdate(dueCharge);
				
				//insert into users_status
				
				UsersStatus usersStatus = new UsersStatus();
				usersStatus.setIsvalid(true);
				usersStatus.setStatus(Global.USERCURNOPAY);
				usersStatus.setUsersInfo(user);
				usersStatus.setStatusdate(Calendar.getInstance().getTime());
				hsession.saveOrUpdate(usersStatus);
				
				//新建用户
				StchgSnapshot stPreNoassign = null;
				if (!Global.nosecondformat.parse(Global.secondformat.format(Calendar.getInstance().getTime())).equals(heatsysSetting.getCurdealdata()))
				{
					hql = "from StchgSnapshot as st " +
							"where st.stdate=:stdate and st.staffInfo.stfid=null and st.projectInfo.pjtid=:pjtid and st.buildingInfo.bldid=:bldid and st.isvalid=:isvalid";
					query = hsession.createQuery(hql);
					query.setBoolean("isvalid", true);
					query.setLong("pjtid", user.getProjectInfo().getPjtid());
					query.setLong("bldid", user.getBuildingInfo().getBldid());
					Calendar cd = Calendar.getInstance();
					cd.add(Calendar.DATE, -1);
					query.setTimestamp("stdate", Global.nosecondformat.parse(Global.secondformat.format(cd.getTime())));
					List sslist = query.list();
					
					if (sslist.size() > 0)
					{
						stPreNoassign = (StchgSnapshot)sslist.get(0);
					}
				}
				
				StchgDaily sd = getStchgDaily(Global.nosecondformat.parse(Global.secondformat.format(Calendar.getInstance().getTime())),
						user.getProjectInfo(), null, user.getBuildingInfo(), hsession);
				
				
				if (sd.getDuemoneyall() == 0)
					sd.setDuemoneyall((stPreNoassign == null ? 0 : stPreNoassign.getDuemoney()) + ((sd.getDuemoney() == null ? 0 : sd.getDuemoney()) + user.getFeerate()*user.getArea()));
				else
					sd.setDuemoneyall((sd.getDuemoney() == null ? 0 : sd.getDuemoney()) + user.getFeerate()*user.getArea());
				if (sd.getDueuserall() == 0)
					sd.setDueuserall((stPreNoassign == null ? 0 : stPreNoassign.getDueuser()) + ((sd.getDueuser() == null ? 0 : sd.getDueuser()) + 1));
				else
					sd.setDueuserall((sd.getDueuser() == null ? 0 : sd.getDueuser()) + 1);
				
				sd.setDueuser((sd.getDueuser() == null ? 0 : sd.getDueuser()) + 1);
				sd.setDueuserlast((sd.getDueuserlast() == null ? 0 : sd.getDueuserlast()) + 1);
				sd.setDuemoney((sd.getDuemoney() == null ? 0 : sd.getDuemoney()) + user.getFeerate()*user.getArea());
				sd.setPartialusers((sd.getPartialusers() == null ? 0 : sd.getPartialusers()) + 1);
				sd.setArea((sd.getArea() == null ? 0 : sd.getArea()) + user.getArea());
				hsession.saveOrUpdate(sd);
			}
        	
			transaction.commit();
			result = Global.SUCCESS;
		}catch(Exception e){
			e.printStackTrace();
			transaction.rollback();
			result = Global.FAILURE;
		}
		return result;
	}

	/*删除记录*/	
	public Result deleteUser(Long usrid){
		log.debug("UserManagerService.deleteUser");
		String sql = "update users_info set isvalid = 0 where isvalid = 1 and usrid = "+usrid; 
		int rows = this.getJdbcTemplate().update(sql);
		if(0==rows){
			Result NO_USER = new Result(false,"failure,no this user");
			return NO_USER;
		}
		else
			return Global.SUCCESS;
	}
	
	public Result listUserTaskComboSearchHiber(Integer start,
			Integer limit,
			String queryStr,
			Integer cmtid,
			Integer bldid,
			String date,
			String type,
			Boolean alltask,
			Integer pjtid,
			HttpSession session)
	{
		ServletContext servletContext = session.getServletContext();    
        ApplicationContext ctx = WebApplicationContextUtils.getWebApplicationContext(servletContext);
        SessionFactory sessionFactory = (SessionFactory) ctx.getBean("sessionFactory");
        Session hsession = SessionFactoryUtils.getSession(sessionFactory, true);
        Transaction transaction = hsession.beginTransaction();
        String hql;
        Query query;
        List list = new ArrayList();
        Result result = new Result();
        try
        {
        	query = hsession.createQuery("from HeatsysSetting");
			HeatsysSetting heatsysSetting = (HeatsysSetting)query.list().get(0);
			
			hql = "from UsersStatus u left outer join u.usersInfo.taskassignInfos ti where u.usid in (select max(u.usid) from UsersStatus u where statusdate<:statusdate group by u.usersInfo.usrid)";
			
			if (cmtid != null && cmtid != 0)
			{
				hql += " and u.usersInfo.communityInfo.cmtid=:cmtid";
			}
			
			if (bldid != null && bldid != 0)
			{
				hql += " and u.usersInfo.buildingInfo.bldid=:bldid";
			}
			
			if(!alltask){
				hql += " and not (ti.staffInfoByLaststaffid.stfid is not null and ti.staffInfoByCurstaffid.stfid is not null and ti.dealname=:dealname)";
			}
			
			if(!(type.equals(Global.USERALL))){
				hql += " and u.status=:status";
			}
			
			if (queryStr != null && !queryStr.trim().isEmpty())
				hql += " and (u.usersInfo.usrname like '%"+queryStr+"%' or u.usersInfo.usertype like '%"+queryStr+"%' or u.usersInfo.phone like '%"+queryStr+"%' or u.usersInfo.locationcode like '%"+queryStr+"%')";
			
			hql += " and u.usersInfo.projectInfo.pjtid=:pjtid order by u.usersInfo.usrid asc";
			
			query = hsession.createQuery(hql).setFirstResult(start).setMaxResults(limit);
			Calendar cal = Calendar.getInstance();
		    cal.setTime(Global.nosecondformat.parse(date));
		    cal.add(cal.DATE, 1);
			query.setTimestamp("statusdate", cal.getTime());
			query.setLong("pjtid", pjtid);
			
			if (cmtid != null && cmtid != 0)
			{
				query.setLong("cmtid", cmtid);
			}
			
			if (bldid != null && bldid != 0)
			{
				query.setLong("bldid", bldid);
			}
			
			if(!alltask){
				query.setString("dealname", heatsysSetting.getCurdealname());
			}
			
			if(!(type.equals(Global.USERALL))){
				query.setString("status", type);
			}
			
			List rows = query.list();
			for (int i = 0; i < rows.size(); i++)
			{
				Object[] obj = (Object[])rows.get(i);
				UsersStatus usersStatus = (UsersStatus)obj[0];
				UserInfo temp = new UserInfo();
				temp.setUsrid(usersStatus.getUsersInfo().getUsrid());
				temp.setUsertype(usersStatus.getUsersInfo().getUsertype());
				temp.setAddress(usersStatus.getUsersInfo().getAddress());
				temp.setUsrname(usersStatus.getUsersInfo().getUsrname());
				temp.setPhone(usersStatus.getUsersInfo().getPhone());
				//
				temp.setStartdate(usersStatus.getUsersInfo().getStartdate() == null ?
						null : Global.nosecondformat.format(usersStatus.getUsersInfo().getStartdate()));
				temp.setContractdate(usersStatus.getUsersInfo().getContractdate() == null ?
						null : Global.nosecondformat.format(usersStatus.getUsersInfo().getContractdate()));
				//
				temp.setContracttype(usersStatus.getUsersInfo().getContracttype());
				temp.setContractver(usersStatus.getUsersInfo().getContractver());
				temp.setContractpic(usersStatus.getUsersInfo().getContractpic());
				temp.setIdpic(usersStatus.getUsersInfo().getIdpic());
				temp.setHouseidpic(usersStatus.getUsersInfo().getHouseidpic());
				temp.setHousepic(usersStatus.getUsersInfo().getHousepic());
				temp.setDesp(usersStatus.getUsersInfo().getDesp());
				temp.setArea(usersStatus.getUsersInfo().getArea());
				temp.setRealarea(usersStatus.getUsersInfo().getRealarea());
				temp.setFeearea(usersStatus.getUsersInfo().getFeearea());
				temp.setFeetype(usersStatus.getUsersInfo().getFeetype());
				temp.setFeerate(usersStatus.getUsersInfo().getFeerate());
				temp.setDiscount(usersStatus.getUsersInfo().getDiscount());
				temp.setReducefee(usersStatus.getUsersInfo().getReducefee());
				temp.setHeatstate(usersStatus.getUsersInfo().getHeatstate());
				temp.setHeatbase(usersStatus.getUsersInfo().getHeatbase());
				temp.setHeatrate(usersStatus.getUsersInfo().getHeatrate());
				temp.setHousetype(usersStatus.getUsersInfo().getHousetype());
				temp.setBalance(usersStatus.getUsersInfo().getBalance());
				temp.setMoney(usersStatus.getUsersInfo().getMoney());
				temp.setNeedmoney(usersStatus.getUsersInfo().getNeedmoney());
				//
				temp.setLastdate(usersStatus.getUsersInfo().getLastdate() == null ?
						null : Global.nosecondformat.format(usersStatus.getUsersInfo().getLastdate()));
				//
				temp.setPjtname(usersStatus.getUsersInfo().getProjectInfo() == null ? 
						null : usersStatus.getUsersInfo().getProjectInfo().getPjtname());
				temp.setCmtname(usersStatus.getUsersInfo().getCommunityInfo() == null ? 
						null : usersStatus.getUsersInfo().getCommunityInfo().getCmtname());
				temp.setBldname(usersStatus.getUsersInfo().getBuildingInfo() == null ? 
						null : usersStatus.getUsersInfo().getBuildingInfo().getBldname());
				temp.setUntname(usersStatus.getUsersInfo().getUnitInfo() == null ? 
						null : usersStatus.getUsersInfo().getUnitInfo().getUntname());
				temp.setMchname(usersStatus.getUsersInfo().getMachinesetInfo() == null ? 
						null : usersStatus.getUsersInfo().getMachinesetInfo().getMchname());
				
				temp.setLastnotpay(usersStatus.getUsersInfo().getLastnotpay());
				temp.setCurnotpay(usersStatus.getUsersInfo().getCurnotpay());
				temp.setUpid(usersStatus.getUsersInfo().getUnitPrice().getUpid());
				temp.setUncheckbalance(usersStatus.getUsersInfo().getUncheckbalance());
				temp.setUncheckmoney(usersStatus.getUsersInfo().getUncheckmoney());
				temp.setUnchecksave(usersStatus.getUsersInfo().getUncheckbalance());
				list.add(temp);
			}
			
			result.setData(list);
			
			hql = "select count(*) " + hql;
			
			query = hsession.createQuery(hql);
			cal = Calendar.getInstance();
		    cal.setTime(Global.nosecondformat.parse(date));
		    cal.add(cal.DATE, 1);
			query.setTimestamp("statusdate", cal.getTime());
			query.setLong("pjtid", pjtid);
			
			if (cmtid != null && cmtid != 0)
			{
				query.setLong("cmtid", cmtid);
			}
			
			if (bldid != null && bldid != 0)
			{
				query.setLong("bldid", bldid);
			}
			
			if(!alltask){
				query.setString("dealname", heatsysSetting.getCurdealname());
			}
			
			if(!(type.equals(Global.USERALL))){
				query.setString("status", type);
			}
			
			rows = query.list();
			
			if (rows.size() > 0)
				result.setTotalProperty((Long)rows.get(0));
			else
				result.setTotalProperty((long)0);
			
        }catch(Exception e){
			e.printStackTrace();			
			result.setSuccess(false);
			result.setMessage("failure");
		}
		return result;
	}
	
	public Result listUserComboSearchHiber(Integer start,
			Integer limit,
			String queryStr,
			Integer cmtid,
			Integer bldid,
			String date,
			String type,
			Boolean all,
			Integer pjtid,
			HttpSession session)
	{
		ServletContext servletContext = session.getServletContext();    
        ApplicationContext ctx = WebApplicationContextUtils.getWebApplicationContext(servletContext);
        SessionFactory sessionFactory = (SessionFactory) ctx.getBean("sessionFactory");
        Session hsession = SessionFactoryUtils.getSession(sessionFactory, true);
        Transaction transaction = hsession.beginTransaction();
        String hql;
        Query query;
        List list = new ArrayList();
        Result result = new Result();
        try
        {
        	query = hsession.createQuery("from HeatsysSetting");
			HeatsysSetting heatsysSetting = (HeatsysSetting)query.list().get(0);
			
			hql = "from UsersStatus u left outer join u.usersInfo.taskassignInfos ti where u.usid in (select max(u.usid) from UsersStatus u where statusdate<:statusdate group by u.usersInfo.usrid)";
			
			if (cmtid != null && cmtid != 0)
			{
				hql += " and u.communityInfo.cmtid=:cmtid";
			}
			
			if (bldid != null && bldid != 0)
			{
				hql += " and u.buildingInfo.bldid=:bldid";
			}
			
			if(!all)
			{
				hql += " and u.usersInfo.projectInfo.pjtid is null";
			}
			
			if(!(type.equals(Global.USERALL))){
				hql += " and u.status=:status";
			}
			
			if (queryStr != null && !queryStr.trim().isEmpty())
				hql += " and (u.usersInfo.usrname like '%"+queryStr+"%' or u.usersInfo.usertype like '%"+queryStr+"%' or u.usersInfo.phone like '%"+queryStr+"%' or u.usersInfo.locationcode like '%"+queryStr+"%')";
			
			hql += " order by u.usersInfo.usrid asc";
			
			query = hsession.createQuery(hql);
			Calendar cal = Calendar.getInstance();
			cal.setTime(Global.nosecondformat.parse(date));
		    cal.add(cal.DATE, 1);
			query.setTimestamp("statusdate", cal.getTime());
			
			if (cmtid != null && cmtid != 0)
			{
				query.setLong("cmtid", cmtid);
			}
			
			if (bldid != null && bldid != 0)
			{
				query.setLong("bldid", bldid);
			}
			
			if(!(type.equals(Global.USERALL))){
				query.setString("status", type);
			}
			
			List rows = query.list();
			for (int i = 0; i < rows.size(); i++)
			{
				Object[] obj = (Object[])rows.get(i);
				UsersStatus usersStatus = (UsersStatus)obj[0];
				UserInfo temp = new UserInfo();
				temp.setUsrid(usersStatus.getUsersInfo().getUsrid());
				temp.setUsertype(usersStatus.getUsersInfo().getUsertype());
				temp.setAddress(usersStatus.getUsersInfo().getAddress());
				temp.setUsrname(usersStatus.getUsersInfo().getUsrname());
				temp.setPhone(usersStatus.getUsersInfo().getPhone());
				//
				temp.setStartdate(usersStatus.getUsersInfo().getStartdate() == null ?
						null : Global.nosecondformat.format(usersStatus.getUsersInfo().getStartdate()));
				temp.setContractdate(usersStatus.getUsersInfo().getContractdate() == null ?
						null : Global.nosecondformat.format(usersStatus.getUsersInfo().getContractdate()));
				//
				temp.setContracttype(usersStatus.getUsersInfo().getContracttype());
				temp.setContractver(usersStatus.getUsersInfo().getContractver());
				temp.setContractpic(usersStatus.getUsersInfo().getContractpic());
				temp.setIdpic(usersStatus.getUsersInfo().getIdpic());
				temp.setHouseidpic(usersStatus.getUsersInfo().getHouseidpic());
				temp.setHousepic(usersStatus.getUsersInfo().getHousepic());
				temp.setDesp(usersStatus.getUsersInfo().getDesp());
				temp.setLocationcode((usersStatus.getUsersInfo().getLocationcode()));
				temp.setArea(usersStatus.getUsersInfo().getArea());
				temp.setRealarea(usersStatus.getUsersInfo().getRealarea());
				temp.setFeearea(usersStatus.getUsersInfo().getFeearea());
				temp.setFeetype(usersStatus.getUsersInfo().getFeetype());
				temp.setFeerate(usersStatus.getUsersInfo().getFeerate());
				temp.setDiscount(usersStatus.getUsersInfo().getDiscount());
				temp.setReducefee(usersStatus.getUsersInfo().getReducefee());
				temp.setHeatstate(usersStatus.getUsersInfo().getHeatstate());
				temp.setHeatbase(usersStatus.getUsersInfo().getHeatbase());
				temp.setHeatrate(usersStatus.getUsersInfo().getHeatrate());
				temp.setHousetype(usersStatus.getUsersInfo().getHousetype());
				temp.setBalance(usersStatus.getUsersInfo().getBalance());
				temp.setMoney(usersStatus.getUsersInfo().getMoney());
				temp.setNeedmoney(usersStatus.getUsersInfo().getNeedmoney());
				//
				temp.setLastdate(usersStatus.getUsersInfo().getLastdate() == null ?
						null : Global.nosecondformat.format(usersStatus.getUsersInfo().getLastdate()));
				//
				temp.setPjtname(usersStatus.getUsersInfo().getProjectInfo() == null ? 
						null : usersStatus.getUsersInfo().getProjectInfo().getPjtname());
				temp.setCmtname(usersStatus.getUsersInfo().getCommunityInfo() == null ? 
						null : usersStatus.getUsersInfo().getCommunityInfo().getCmtname());
				temp.setBldname(usersStatus.getUsersInfo().getBuildingInfo() == null ? 
						null : usersStatus.getUsersInfo().getBuildingInfo().getBldname());
				temp.setUntname(usersStatus.getUsersInfo().getUnitInfo() == null ? 
						null : usersStatus.getUsersInfo().getUnitInfo().getUntname());
				temp.setMchname(usersStatus.getUsersInfo().getMachinesetInfo() == null ? 
						null : usersStatus.getUsersInfo().getMachinesetInfo().getMchname());
				
				temp.setLastnotpay(usersStatus.getUsersInfo().getLastnotpay());
				temp.setCurnotpay(usersStatus.getUsersInfo().getCurnotpay());
				temp.setUpid(usersStatus.getUsersInfo().getUnitPrice().getUpid());
				temp.setUncheckbalance(usersStatus.getUsersInfo().getUncheckbalance());
				temp.setUncheckmoney(usersStatus.getUsersInfo().getUncheckmoney());
				temp.setUnchecksave(usersStatus.getUsersInfo().getUncheckbalance());
				list.add(temp);
			}
			
			result.setData(list);
			result.setTotalProperty((long)list.size());
        }catch(Exception e){
			e.printStackTrace();			
			result.setSuccess(false);
			result.setMessage("failure");
		}
		return result;
	}
	
	
	/*获取用户*/	
	public ListCount listUser(Integer start,Integer limit,String query, Boolean all, Integer pjtid){
		log.debug("UserManagerService.listUser");
		ListCount listCount = new ListCount();
		List list = new ArrayList();
		String toadd = "";
		if (!all)
			toadd = " and u.pjtid is null";
		else
			if (pjtid != -1)
				toadd = " and u.pjtid = " + pjtid;
		
		String test = "select count(*) from users_info u where u.isvalid = 1"
				+" and (u.usrname like '%"+query+"%' or u.usertype like '%"+query+"%' or u.phone like '%"+query+"%')" + toadd;
		
		int count = this.getJdbcTemplate().queryForInt("select count(*) from users_info u where u.isvalid = 1"
				+" and (u.usrname like '%"+query+"%' or u.usertype like '%"+query+"%' or u.phone like '%"+query+"%')" + toadd);
		int max = this.getJdbcTemplate().queryForInt("select max(rownum) from users_info u where isvalid = 1"
				+" and (u.usrname like '%"+query+"%' or u.usertype like '%"+query+"%' or u.phone like '%"+query+"%')" + toadd);
		listCount.setTotalProperty((long)count);
		if(start>max){
			listCount.setData(list);
			return listCount;
		}else if(max-start < limit){
			limit = max - start;
		}
		String sql = "select * from (select rownum as num ,u.* ,p.pjtname,m.mchname,c.cmtname,b.bldname,un.untname from users_info u "+
				"left outer join project_info p on p.pjtid = u.pjtid "+
				"left outer join machineset_info m on m.mchid = u.mchid "+
				"left outer join community_info c on c.cmtid = u.cmtid "+
				"left outer join building_info b on b.bldid = u.bldid "+
				"left outer join unit_info un on un.untid = u.untid "+
				" where u.isvalid = 1 and (u.usrname like '%"+query+"%' or u.usertype like '%"+query+"%' or u.phone like '%"+query+"%')" + toadd + " order by u.usrid asc) where num > "+ start+" and num <= "+(start+limit);
		List rows = this.getJdbcTemplate().queryForList(sql
				);
		Iterator it = rows.iterator();
		while(it.hasNext()){
			Map map = (Map) it.next();
			UserInfo temp = new UserInfo();
			temp.setUsrid((Long)Global.toNullString(map, "usrid", "BigDecimal", "Long"));
			temp.setUsertype((String)Global.toNullString(map, "usertype", "String", "String"));
			temp.setAddress((String)Global.toNullString(map, "address", "String", "String"));
			temp.setUsrname((String)Global.toNullString(map, "usrname", "String", "String"));
			temp.setPhone((String)Global.toNullString(map, "phone", "String", "String"));
			temp.setStartdate((String)Global.toNullString(map, "startdate", "Timestamp", "String"));
			temp.setContractdate((String)Global.toNullString(map, "contractdate", "Timestamp", "String"));
			temp.setContracttype((String)Global.toNullString(map,"contracttype","String","String"));
			temp.setContractver((String)Global.toNullString(map,"contractver","String","String"));
			temp.setContractpic((String)Global.toNullString(map,"contractpic","String","String"));
			temp.setIdpic((String)Global.toNullString(map,"idpic","String","String"));
			temp.setHouseidpic((String)Global.toNullString(map,"houseidpic","String","String"));
			temp.setHousepic((String)Global.toNullString(map,"housepic","String","String"));
			temp.setDesp((String)Global.toNullString(map,"desp","String","String"));
			temp.setArea((Double)Global.toNullString(map, "area", "BigDecimal", "Double"));
			temp.setRealarea((Double)Global.toNullString(map, "realarea", "BigDecimal", "Double"));
			temp.setFeearea((Double)Global.toNullString(map, "feearea", "BigDecimal", "Double"));
			temp.setFeetype((String)Global.toNullString(map, "feetype", "String", "String"));
			temp.setFeerate((Double)Global.toNullString(map, "feerate", "BigDecimal", "Double"));
			temp.setDiscount((Double)Global.toNullString(map, "discount", "BigDecimal", "Double"));
			temp.setReducefee((Double)Global.toNullString(map, "reducefee", "BigDecimal", "Double"));
			temp.setHeatstate((String)Global.toNullString(map, "heatstate", "String", "String"));
			temp.setHeatbase((Double)Global.toNullString(map, "heatbase", "BigDecimal", "Double"));
			temp.setHeatrate((Double)Global.toNullString(map, "heatrate", "BigDecimal", "Double"));
			temp.setHousetype((String)Global.toNullString(map, "housetype", "String", "String"));
			temp.setBalance((Double)Global.toNullString(map, "balance", "BigDecimal", "Double"));
			temp.setMoney((Double)Global.toNullString(map, "money", "BigDecimal", "Double"));
			temp.setNeedmoney((Double)Global.toNullString(map, "needmoney", "BigDecimal", "Double"));
			temp.setLastdate((String)Global.toNullString(map, "lastdate", "Timestamp", "String"));
			temp.setPjtname((String)Global.toNullString(map, "pjtname", "String", "String"));
			temp.setCmtname((String)Global.toNullString(map, "cmtname", "String", "String"));
			temp.setBldname((String)Global.toNullString(map, "bldname", "String", "String"));
			temp.setUntname((String)Global.toNullString(map, "untname", "String", "String"));
			temp.setMchname((String)Global.toNullString(map, "mchname", "String", "String"));
			temp.setUpid((Long)Global.toNullString(map, "upid", "BigDecimal", "Long"));
			temp.setUncheckbalance((Double)Global.toNullString(map, "uncheckbalance", "BigDecimal", "Double"));
			temp.setUncheckmoney((Double)Global.toNullString(map, "uncheckmoney", "BigDecimal", "Double"));
			temp.setUnchecksave(temp.getUncheckbalance());
			list.add(temp);
		}
		listCount.setData(list);
		return listCount;
	}
	
	public ListCount listUser(Integer start,Integer limit,String query, Boolean all, Integer pjtid, StaffPermission sp){
		log.debug("UserManagerService.listUser");
		ListCount listCount = new ListCount();
		List list = new ArrayList();

		boolean is_all = false;
		
		if (sp != null)
		{
			for (Object o : sp.getPjtOpr())
			{
				OperationInfo one = (OperationInfo)o;
				if (one.getOprname().equals("oprallpjt"))
				{
					is_all = true;
					break;
				}
			}
		}
		
		if (is_all)
		{
			String toadd = "";
			if (!all)
				toadd = " and u.pjtid is null";
			else
				if (pjtid != -1)
					toadd = " and u.pjtid = " + pjtid;
			
			
			int count = this.getJdbcTemplate().queryForInt("select count(*) from users_info u where u.isvalid = 1"
					+" and (u.usrname like '%"+query+"%' or u.usertype like '%"+query+"%' or u.phone like '%"+query+"%')" + toadd);
			int max = this.getJdbcTemplate().queryForInt("select max(rownum) from users_info u where isvalid = 1"
					+" and (u.usrname like '%"+query+"%' or u.usertype like '%"+query+"%' or u.phone like '%"+query+"%')" + toadd);
			listCount.setTotalProperty((long)count);
			if(start>max){
				listCount.setData(list);
				return listCount;
			}else if(max-start < limit){
				limit = max - start;
			}
			String sql = "select * from (select rownum as num ,u.* ,p.pjtname,m.mchname,c.cmtname,b.bldname,un.untname from users_info u "+
					"left outer join project_info p on p.pjtid = u.pjtid "+
					"left outer join machineset_info m on m.mchid = u.mchid "+
					"left outer join community_info c on c.cmtid = u.cmtid "+
					"left outer join building_info b on b.bldid = u.bldid "+
					"left outer join unit_info un on un.untid = u.untid "+
					" where u.isvalid = 1 and (u.usrname like '%"+query+"%' or u.usertype like '%"+query+"%' or u.phone like '%"+query+"%')" + toadd + ") where num > "+ start+" and num <= "+(start+limit);
			List rows = this.getJdbcTemplate().queryForList(sql
					);
			Iterator it = rows.iterator();
			while(it.hasNext()){
				Map map = (Map) it.next();
				UserInfo temp = new UserInfo();
				temp.setUsrid((Long)Global.toNullString(map, "usrid", "BigDecimal", "Long"));
				temp.setUsertype((String)Global.toNullString(map, "usertype", "String", "String"));
				temp.setAddress((String)Global.toNullString(map, "address", "String", "String"));
				temp.setUsrname((String)Global.toNullString(map, "usrname", "String", "String"));
				temp.setPhone((String)Global.toNullString(map, "phone", "String", "String"));
				temp.setStartdate((String)Global.toNullString(map, "startdate", "Timestamp", "String"));
				temp.setContractdate((String)Global.toNullString(map, "contractdate", "Timestamp", "String"));
				temp.setContracttype((String)Global.toNullString(map,"contracttype","String","String"));
				temp.setContractver((String)Global.toNullString(map,"contractver","String","String"));
				temp.setContractpic((String)Global.toNullString(map,"contractpic","String","String"));
				temp.setIdpic((String)Global.toNullString(map,"idpic","String","String"));
				temp.setHouseidpic((String)Global.toNullString(map,"houseidpic","String","String"));
				temp.setHousepic((String)Global.toNullString(map,"housepic","String","String"));
				temp.setDesp((String)Global.toNullString(map,"desp","String","String"));
				temp.setArea((Double)Global.toNullString(map, "area", "BigDecimal", "Double"));
				temp.setRealarea((Double)Global.toNullString(map, "realarea", "BigDecimal", "Double"));
				temp.setFeearea((Double)Global.toNullString(map, "feearea", "BigDecimal", "Double"));
				temp.setFeetype((String)Global.toNullString(map, "feetype", "String", "String"));
				temp.setFeerate((Double)Global.toNullString(map, "feerate", "BigDecimal", "Double"));
				temp.setDiscount((Double)Global.toNullString(map, "discount", "BigDecimal", "Double"));
				temp.setReducefee((Double)Global.toNullString(map, "reducefee", "BigDecimal", "Double"));
				temp.setHeatstate((String)Global.toNullString(map, "heatstate", "String", "String"));
				temp.setHeatbase((Double)Global.toNullString(map, "heatbase", "BigDecimal", "Double"));
				temp.setHeatrate((Double)Global.toNullString(map, "heatrate", "BigDecimal", "Double"));
				temp.setHousetype((String)Global.toNullString(map, "housetype", "String", "String"));
				temp.setBalance((Double)Global.toNullString(map, "balance", "BigDecimal", "Double"));
				temp.setMoney((Double)Global.toNullString(map, "money", "BigDecimal", "Double"));
				temp.setNeedmoney((Double)Global.toNullString(map, "needmoney", "BigDecimal", "Double"));
				temp.setLastdate((String)Global.toNullString(map, "lastdate", "Timestamp", "String"));
				temp.setPjtname((String)Global.toNullString(map, "pjtname", "String", "String"));
				temp.setCmtname((String)Global.toNullString(map, "cmtname", "String", "String"));
				temp.setBldname((String)Global.toNullString(map, "bldname", "String", "String"));
				temp.setUntname((String)Global.toNullString(map, "untname", "String", "String"));
				temp.setMchname((String)Global.toNullString(map, "mchname", "String", "String"));
				temp.setLastnotpay((Long)Global.toNullString(map, "lastnotpay", "BigDecimal", "Long"));
				temp.setCurnotpay((Long)Global.toNullString(map, "curnotpay", "BigDecimal", "Long"));
				temp.setUpid((Long)Global.toNullString(map, "upid", "BigDecimal", "Long"));
				temp.setUncheckbalance((Double)Global.toNullString(map, "uncheckbalance", "BigDecimal", "Double"));
				temp.setUncheckmoney((Double)Global.toNullString(map, "uncheckmoney", "BigDecimal", "Double"));
				temp.setUnchecksave(temp.getUncheckbalance());
				list.add(temp);
			}
		}
		else
		{
			String toadd = "";
			if (!all)
				toadd = " and u.pjtid = null";
			else
				if (pjtid != -1)
					toadd = " and u.pjtid = " + pjtid;
			
			
			int count = this.getJdbcTemplate().queryForInt("select count(*) from users_info u, staff_info s, stf_grp_pjt_mapping m where s.stfid = m.stfid and u.pjtid = m.pjtid and u.isvalid = 1 and s.stfid = " + sp.getStaffInfo().getStfid()
					+" and (u.usrname like '%"+query+"%' or u.usertype like '%"+query+"%' or u.phone like '%"+query+"%')" + toadd);
			int max = this.getJdbcTemplate().queryForInt("select max(rownum) from users_info u, staff_info s, stf_grp_pjt_mapping m where s.stfid = m.stfid and u.pjtid = m.pjtid and u.isvalid = 1 and s.stfid = " + sp.getStaffInfo().getStfid()
					+" and (u.usrname like '%"+query+"%' or u.usertype like '%"+query+"%' or u.phone like '%"+query+"%')" + toadd);
			listCount.setTotalProperty((long)count);
			if(start>max){
				listCount.setData(list);
				return listCount;
			}else if(max-start < limit){
				limit = max - start;
			}
			String sql = "select * from (select rownum as num , u.*, p.pjtname,m.mchname,c.cmtname,b.bldname,un.untname from (select t.* from users_info t, staff_info s, stf_grp_pjt_mapping sm where sm.stfid = s.stfid and t.pjtid = sm.pjtid and t.isvalid = 1 and s.stfid = " + sp.getStaffInfo().getStfid()+ " ) u left outer join project_info p on p.pjtid = u.pjtid left outer join machineset_info m on m.mchid = u.mchid left outer join community_info c on c.cmtid = u.cmtid left outer join building_info b on b.bldid = u.bldid left outer join unit_info un on un.untid = u.untid " +
					"where (u.usrname like '%"+query+"%' or u.usertype like '%"+query+"%' or u.phone like '%"+query+"%')" + toadd + ") rs where num > "+ start+" and num <= "+(start+limit) ;
			List rows = this.getJdbcTemplate().queryForList(sql
					);
			Iterator it = rows.iterator();
			while(it.hasNext()){
				Map map = (Map) it.next();
				UserInfo temp = new UserInfo();
				temp.setUsrid((Long)Global.toNullString(map, "usrid", "BigDecimal", "Long"));
				temp.setUsertype((String)Global.toNullString(map, "usertype", "String", "String"));
				temp.setAddress((String)Global.toNullString(map, "address", "String", "String"));
				temp.setUsrname((String)Global.toNullString(map, "usrname", "String", "String"));
				temp.setPhone((String)Global.toNullString(map, "phone", "String", "String"));
				temp.setStartdate((String)Global.toNullString(map, "startdate", "Timestamp", "String"));
				temp.setContractdate((String)Global.toNullString(map, "contractdate", "Timestamp", "String"));
				temp.setContracttype((String)Global.toNullString(map,"contracttype","String","String"));
				temp.setContractver((String)Global.toNullString(map,"contractver","String","String"));
				temp.setContractpic((String)Global.toNullString(map,"contractpic","String","String"));
				temp.setIdpic((String)Global.toNullString(map,"idpic","String","String"));
				temp.setHouseidpic((String)Global.toNullString(map,"houseidpic","String","String"));
				temp.setHousepic((String)Global.toNullString(map,"housepic","String","String"));
				temp.setDesp((String)Global.toNullString(map,"desp","String","String"));
				temp.setArea((Double)Global.toNullString(map, "area", "BigDecimal", "Double"));
				temp.setRealarea((Double)Global.toNullString(map, "realarea", "BigDecimal", "Double"));
				temp.setFeearea((Double)Global.toNullString(map, "feearea", "BigDecimal", "Double"));
				temp.setFeetype((String)Global.toNullString(map, "feetype", "String", "String"));
				temp.setFeerate((Double)Global.toNullString(map, "feerate", "BigDecimal", "Double"));
				temp.setDiscount((Double)Global.toNullString(map, "discount", "BigDecimal", "Double"));
				temp.setReducefee((Double)Global.toNullString(map, "reducefee", "BigDecimal", "Double"));
				temp.setHeatstate((String)Global.toNullString(map, "heatstate", "String", "String"));
				temp.setHeatbase((Double)Global.toNullString(map, "heatbase", "BigDecimal", "Double"));
				temp.setHeatrate((Double)Global.toNullString(map, "heatrate", "BigDecimal", "Double"));
				temp.setHousetype((String)Global.toNullString(map, "housetype", "String", "String"));
				temp.setBalance((Double)Global.toNullString(map, "balance", "BigDecimal", "Double"));
				temp.setMoney((Double)Global.toNullString(map, "money", "BigDecimal", "Double"));
				temp.setNeedmoney((Double)Global.toNullString(map, "needmoney", "BigDecimal", "Double"));
				temp.setLastdate((String)Global.toNullString(map, "lastdate", "Timestamp", "String"));
				temp.setPjtname((String)Global.toNullString(map, "pjtname", "String", "String"));
				temp.setCmtname((String)Global.toNullString(map, "cmtname", "String", "String"));
				temp.setBldname((String)Global.toNullString(map, "bldname", "String", "String"));
				temp.setUntname((String)Global.toNullString(map, "untname", "String", "String"));
				temp.setMchname((String)Global.toNullString(map, "mchname", "String", "String"));
				temp.setLastnotpay((Long)Global.toNullString(map, "lastnotpay", "BigDecimal", "Long"));
				temp.setCurnotpay((Long)Global.toNullString(map, "curnotpay", "BigDecimal", "Long"));
				temp.setUpid((Long)Global.toNullString(map, "upid", "BigDecimal", "Long"));
				temp.setUncheckbalance((Double)Global.toNullString(map, "uncheckbalance", "BigDecimal", "Double"));
				temp.setUncheckmoney((Double)Global.toNullString(map, "uncheckmoney", "BigDecimal", "Double"));
				temp.setUnchecksave(temp.getUncheckbalance());
				list.add(temp);
			}
		}
		
		
		listCount.setData(list);
		return listCount;
	}
	
	//获取指定用户
	public UserInfo findById(Long usrid){ 
	    final UserInfo user=new UserInfo(); 
	    this.getJdbcTemplate().query("select usrid,contractpic,idpic,houseidpic,housepic from users_info " +
	    		"where isvalid =1 and usrid= "+usrid, new RowCallbackHandler(){ 
	                    public void processRow(ResultSet arg0)throws SQLException {
						user.setUsrid(arg0.getLong("usrid"));
						user.setContractpic(arg0.getString("contractpic"));
						user.setIdpic(arg0.getString("idpic"));
						user.setHouseidpic(arg0.getString("houseidpic"));
						user.setHousepic(arg0.getString("housepic"));}}); 
	    return user; 
	}
	
	/*获取总记录数*/	
	public Long countUser(){
		log.debug("UserManagerService.countUser");
		int count = getJdbcTemplate().queryForInt("select count(*) from users_info where isvalid = 1");
		return (long) count;
	}
	
	public String timeToString(Timestamp time){
		SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd");
		if(time == null)
			return "";
		else
			return sdf.format(time);
	}
	
	public String nullToString(String string){
		if(string == null)
			return "";
		else
			return string;
	}
	
	public Result listUserWithContionsHiber(
			Integer start,
			Integer limit,
			String queryStr, 
			Integer pjtid, 
			Integer cmtid, 
			Integer bldid, 
			StaffPermission sp,
			String date,
			String type,
			String stfid,
			HttpSession session){
		ServletContext servletContext = session.getServletContext();    
        ApplicationContext ctx = WebApplicationContextUtils.getWebApplicationContext(servletContext);
        SessionFactory sessionFactory = (SessionFactory) ctx.getBean("sessionFactory");
        Session hsession = SessionFactoryUtils.getSession(sessionFactory, true);
		Result result = new Result();
		Query query;
		String hql = "";
		String condition = "";
		List list = new ArrayList();
		try{
			query = hsession.createQuery("from HeatsysSetting");
			HeatsysSetting heatsysSetting = (HeatsysSetting)query.list().get(0);
			
			boolean is_all = false;
			if (sp != null)
			{
				for (Object o : sp.getPjtOpr())
				{
					OperationInfo one = (OperationInfo)o;
					if (one.getOprname().equals("oprallpjt"))
					{
						is_all = true;
						break;
					}
				}
			}
			
			if (is_all)
			{
				hql = "from UsersStatus u left outer join u.usersInfo.taskassignInfos ti where u.usid in (select max(u.usid) from UsersStatus u where statusdate<:statusdate group by u.usersInfo.usrid)";
				
				if (cmtid != null && cmtid != 0)
				{
					hql += " and u.usersInfo.communityInfo.cmtid=:cmtid";
				}
				
				if (bldid != null && bldid != 0)
				{
					hql += " and u.usersInfo.buildingInfo.bldid=:bldid";
				}
				
				if (!(stfid.equals(Global.STAFFALL))){
					hql += " and (ti.staffInfoByLaststaffid.stfid=:stfid or ti.staffInfoByCurstaffid.stfid=:stfid) and ti.dealname=:dealname";
				}
				
				if (!(type.equals(Global.USERALL))){
					hql += " and u.status=:status";
				}
				
				if (queryStr != null && !queryStr.trim().isEmpty())
					hql += " and (u.usersInfo.usrname like '%"+queryStr+"%' or u.usersInfo.usertype like '%"+queryStr+"%' or u.usersInfo.phone like '%"+queryStr+"%' or u.usersInfo.locationcode like '%"+queryStr+"%')";
				
				hql += " order by u.usersInfo.usrid asc";
				
				query = hsession.createQuery(hql).setFirstResult(start).setMaxResults(limit);
				Calendar cal = Calendar.getInstance();
			    cal.setTime(Global.nosecondformat.parse(date));
			    cal.add(cal.DATE, 1);
				query.setTimestamp("statusdate", cal.getTime());
				
				if (cmtid != null && cmtid != 0)
				{
					query.setLong("cmtid", cmtid);
				}
				
				if (bldid != null && bldid != 0)
				{
					query.setLong("bldid", bldid);
				}
				
				if(!(stfid.equals(Global.STAFFALL))){
					query.setLong("stfid", Long.parseLong(stfid));
					query.setString("dealname", heatsysSetting.getCurdealname());
				}
				
				if(!(type.equals(Global.USERALL))){
					query.setString("status", type);
				}
				
				List rows = query.list();
				for (int i = 0; i < rows.size(); i++)
				{
					Object[] obj = (Object[])rows.get(i);
					UsersStatus usersStatus = (UsersStatus)obj[0];
					UserInfo temp = new UserInfo();
					temp.setUsrid(usersStatus.getUsersInfo().getUsrid());
					temp.setUsertype(usersStatus.getUsersInfo().getUsertype());
					temp.setAddress(usersStatus.getUsersInfo().getAddress());
					temp.setUsrname(usersStatus.getUsersInfo().getUsrname());
					temp.setPhone(usersStatus.getUsersInfo().getPhone());
					//
					temp.setStartdate(usersStatus.getUsersInfo().getStartdate() == null ?
							null : Global.nosecondformat.format(usersStatus.getUsersInfo().getStartdate()));
					temp.setContractdate(usersStatus.getUsersInfo().getContractdate() == null ?
							null : Global.nosecondformat.format(usersStatus.getUsersInfo().getContractdate()));
					//
					temp.setContracttype(usersStatus.getUsersInfo().getContracttype());
					temp.setContractver(usersStatus.getUsersInfo().getContractver());
					temp.setContractpic(usersStatus.getUsersInfo().getContractpic());
					temp.setIdpic(usersStatus.getUsersInfo().getIdpic());
					temp.setHouseidpic(usersStatus.getUsersInfo().getHouseidpic());
					temp.setHousepic(usersStatus.getUsersInfo().getHousepic());
					temp.setDesp(usersStatus.getUsersInfo().getDesp());
					temp.setLocationcode(usersStatus.getUsersInfo().getLocationcode());
					temp.setArea(usersStatus.getUsersInfo().getArea());
					temp.setRealarea(usersStatus.getUsersInfo().getRealarea());
					temp.setFeearea(usersStatus.getUsersInfo().getFeearea());
					temp.setFeetype(usersStatus.getUsersInfo().getFeetype());
					temp.setFeerate(usersStatus.getUsersInfo().getFeerate());
					temp.setDiscount(usersStatus.getUsersInfo().getDiscount());
					temp.setReducefee(usersStatus.getUsersInfo().getReducefee());
					temp.setHeatstate(usersStatus.getUsersInfo().getHeatstate());
					temp.setHeatbase(usersStatus.getUsersInfo().getHeatbase());
					temp.setHeatrate(usersStatus.getUsersInfo().getHeatrate());
					temp.setHousetype(usersStatus.getUsersInfo().getHousetype());
					temp.setBalance(usersStatus.getUsersInfo().getBalance());
					temp.setMoney(usersStatus.getUsersInfo().getMoney());
					temp.setNeedmoney(usersStatus.getUsersInfo().getNeedmoney());
					//
					temp.setLastdate(usersStatus.getUsersInfo().getLastdate() == null ?
							null : Global.nosecondformat.format(usersStatus.getUsersInfo().getLastdate()));
					//
					temp.setPjtname(usersStatus.getUsersInfo().getProjectInfo() == null ? 
							null : usersStatus.getUsersInfo().getProjectInfo().getPjtname());
					temp.setCmtname(usersStatus.getUsersInfo().getCommunityInfo() == null ? 
							null : usersStatus.getUsersInfo().getCommunityInfo().getCmtname());
					temp.setBldname(usersStatus.getUsersInfo().getBuildingInfo() == null ? 
							null : usersStatus.getUsersInfo().getBuildingInfo().getBldname());
					temp.setUntname(usersStatus.getUsersInfo().getUnitInfo() == null ? 
							null : usersStatus.getUsersInfo().getUnitInfo().getUntname());
					temp.setMchname(usersStatus.getUsersInfo().getMachinesetInfo() == null ? 
							null : usersStatus.getUsersInfo().getMachinesetInfo().getMchname());
					
					temp.setLastnotpay(usersStatus.getUsersInfo().getLastnotpay());
					temp.setCurnotpay(usersStatus.getUsersInfo().getCurnotpay());
					temp.setUpid(usersStatus.getUsersInfo().getUnitPrice().getUpid());
					temp.setUncheckbalance(usersStatus.getUsersInfo().getUncheckbalance());
					temp.setUncheckmoney(usersStatus.getUsersInfo().getUncheckmoney());
					temp.setUnchecksave(usersStatus.getUsersInfo().getUncheckbalance());
					list.add(temp);
				}
				
				result.setData(list);
				
				hql = "select count(*) " + hql;
	    		
	    		query = hsession.createQuery(hql);
	    		cal = Calendar.getInstance();
			    cal.setTime(Global.nosecondformat.parse(date));
			    cal.add(cal.DATE, 1);
				query.setTimestamp("statusdate", cal.getTime());
				
				if (cmtid != null && cmtid != 0)
				{
					query.setLong("cmtid", cmtid);
				}
				
				if (bldid != null && bldid != 0)
				{
					query.setLong("bldid", bldid);
				}
				
				if(!(stfid.equals(Global.STAFFALL))){
					query.setLong("stfid", Long.parseLong(stfid));
					query.setString("dealname", heatsysSetting.getCurdealname());
				}
				
				if(!(type.equals(Global.USERALL))){
					query.setString("status", type);
				}
	    		
				rows = query.list();
				
				if (rows.size() > 0)
					result.setTotalProperty((Long)rows.get(0));
				else
					result.setTotalProperty((long)0);
			}
			else
			{
				hql = "from UsersStatus u left outer join u.usersInfo.taskassignInfos ti where u.usid in (select max(u.usid) from UsersStatus u where statusdate<:statusdate group by u.usersInfo.usrid)";
				
				if (cmtid != null && cmtid != 0)
				{
					hql += " and u.usersInfo.communityInfo.cmtid=:cmtid";
				}
				
				if (bldid != null && bldid != 0)
				{
					hql += " and u.usersInfo.buildingInfo.bldid=:bldid";
				}
				
				hql += " and (ti.staffInfoByLaststaffid.stfid=:stfid or ti.staffInfoByCurstaffid.stfid=:stfid) and ti.dealname=:dealname";
				
				if(!(type.equals(Global.USERALL))){
					hql += " and u.status=:status";
				}
				
				if (queryStr != null && !queryStr.trim().isEmpty())
					hql += " and (u.usersInfo.usrname like '%"+queryStr+"%' or u.usersInfo.usertype like '%"+queryStr+"%' or u.usersInfo.phone like '%"+queryStr+"%' or u.usersInfo.locationcode like '%"+queryStr+"%')";
				
				hql += " order by u.usersInfo.usrid asc";
				
				
				query = hsession.createQuery(hql).setFirstResult(start).setMaxResults(limit);
				Calendar cal = Calendar.getInstance();
			    cal.setTime(Global.nosecondformat.parse(date));
			    cal.add(cal.DATE, 1);
				query.setTimestamp("statusdate", cal.getTime());
				query.setString("dealname", heatsysSetting.getCurdealname());
				
				if (cmtid != null && cmtid != 0)
				{
					query.setLong("cmtid", cmtid);
				}
				
				if (bldid != null && bldid != 0)
				{
					query.setLong("bldid", bldid);
				}
				
				query.setLong("stfid", sp.getStaffInfo().getStfid());
				
				if(!(type.equals(Global.USERALL))){
					query.setString("status", type);
				}
				
				List rows = query.list();
				for (int i = 0; i < rows.size(); i++)
				{
					Object[] obj = (Object[])rows.get(i);
					UsersStatus usersStatus = (UsersStatus)obj[0];
					UserInfo temp = new UserInfo();
					temp.setUsrid(usersStatus.getUsersInfo().getUsrid());
					temp.setUsertype(usersStatus.getUsersInfo().getUsertype());
					temp.setAddress(usersStatus.getUsersInfo().getAddress());
					temp.setUsrname(usersStatus.getUsersInfo().getUsrname());
					temp.setPhone(usersStatus.getUsersInfo().getPhone());
					//
					temp.setStartdate(usersStatus.getUsersInfo().getStartdate() == null ?
							null : Global.nosecondformat.format(usersStatus.getUsersInfo().getStartdate()));
					temp.setContractdate(usersStatus.getUsersInfo().getContractdate() == null ?
							null : Global.nosecondformat.format(usersStatus.getUsersInfo().getContractdate()));
					//
					temp.setContracttype(usersStatus.getUsersInfo().getContracttype());
					temp.setContractver(usersStatus.getUsersInfo().getContractver());
					temp.setContractpic(usersStatus.getUsersInfo().getContractpic());
					temp.setIdpic(usersStatus.getUsersInfo().getIdpic());
					temp.setHouseidpic(usersStatus.getUsersInfo().getHouseidpic());
					temp.setHousepic(usersStatus.getUsersInfo().getHousepic());
					temp.setDesp(usersStatus.getUsersInfo().getDesp());
					temp.setArea(usersStatus.getUsersInfo().getArea());
					temp.setRealarea(usersStatus.getUsersInfo().getRealarea());
					temp.setFeearea(usersStatus.getUsersInfo().getFeearea());
					temp.setFeetype(usersStatus.getUsersInfo().getFeetype());
					temp.setFeerate(usersStatus.getUsersInfo().getFeerate());
					temp.setDiscount(usersStatus.getUsersInfo().getDiscount());
					temp.setReducefee(usersStatus.getUsersInfo().getReducefee());
					temp.setHeatstate(usersStatus.getUsersInfo().getHeatstate());
					temp.setHeatbase(usersStatus.getUsersInfo().getHeatbase());
					temp.setHeatrate(usersStatus.getUsersInfo().getHeatrate());
					temp.setHousetype(usersStatus.getUsersInfo().getHousetype());
					temp.setBalance(usersStatus.getUsersInfo().getBalance());
					temp.setMoney(usersStatus.getUsersInfo().getMoney());
					temp.setNeedmoney(usersStatus.getUsersInfo().getNeedmoney());
					//
					temp.setLastdate(usersStatus.getUsersInfo().getLastdate() == null ?
							null : Global.nosecondformat.format(usersStatus.getUsersInfo().getLastdate()));
					//
					temp.setPjtname(usersStatus.getUsersInfo().getProjectInfo() == null ? 
							null : usersStatus.getUsersInfo().getProjectInfo().getPjtname());
					temp.setCmtname(usersStatus.getUsersInfo().getCommunityInfo() == null ? 
							null : usersStatus.getUsersInfo().getCommunityInfo().getCmtname());
					temp.setBldname(usersStatus.getUsersInfo().getBuildingInfo() == null ? 
							null : usersStatus.getUsersInfo().getBuildingInfo().getBldname());
					temp.setUntname(usersStatus.getUsersInfo().getUnitInfo() == null ? 
							null : usersStatus.getUsersInfo().getUnitInfo().getUntname());
					temp.setMchname(usersStatus.getUsersInfo().getMachinesetInfo() == null ? 
							null : usersStatus.getUsersInfo().getMachinesetInfo().getMchname());
					
					temp.setLastnotpay(usersStatus.getUsersInfo().getLastnotpay());
					temp.setCurnotpay(usersStatus.getUsersInfo().getCurnotpay());
					temp.setUpid(usersStatus.getUsersInfo().getUnitPrice().getUpid());
					temp.setUncheckbalance(usersStatus.getUsersInfo().getUncheckbalance());
					temp.setUncheckmoney(usersStatus.getUsersInfo().getUncheckmoney());
					temp.setUnchecksave(usersStatus.getUsersInfo().getUncheckbalance());
					list.add(temp);
				}
				
				result.setData(list);
				
				hql = "select count(*) " + hql;
				query = hsession.createQuery(hql);
				
				cal = Calendar.getInstance();
			    cal.setTime(Global.nosecondformat.parse(date));
			    cal.add(cal.DATE, 1);
				query.setTimestamp("statusdate", cal.getTime());
				query.setString("dealname", heatsysSetting.getCurdealname());
				
				if (cmtid != null && cmtid != 0)
				{
					query.setLong("cmtid", cmtid);
				}
				
				if (bldid != null && bldid != 0)
				{
					query.setLong("bldid", bldid);
				}
				
				query.setLong("stfid", sp.getStaffInfo().getStfid());
				
				if(!(type.equals(Global.USERALL))){
					query.setString("status", type);
				}
				
				rows = query.list();
				
				if (rows.size() > 0)
					result.setTotalProperty((Long)rows.get(0));
				else
					result.setTotalProperty((long)0);
			}
		}catch(Exception e){
			e.printStackTrace();			
			result.setSuccess(false);
			result.setMessage("failure");
		}
		return result;
			
	}
	
	public ListCount listUserWithContions(
			Integer start,
			Integer limit,
			String query, 
			Boolean all, 
			Integer pjtid, 
			Integer cmtid, 
			Integer bldid, 
			StaffPermission sp,
			String date,
			String type,
			String stfid){
		log.debug("UserManagerService.listUser");
		ListCount listCount = new ListCount();
		List list = new ArrayList();
		boolean is_all = false;
		if (sp != null)
		{
			for (Object o : sp.getPjtOpr())
			{
				OperationInfo one = (OperationInfo)o;
				if (one.getOprname().equals("oprallpjt"))
				{
					is_all = true;
					break;
				}
			}
		}
		
		if (is_all)
		{
			String toadd = "";
			if (!all)
				toadd = " and u.pjtid is null";
			else
				if (pjtid != -1)
					toadd = " and u.pjtid = " + pjtid;
			String sql = "with "+
			"A as "+
			"( "+
			"select max(statusdate) as statusdate,usrid from users_status where statusdate<=to_date('"+date+" 23:59:59','yyyy-MM-dd HH24:mi:ss') group by usrid "+
			") "+
			", "+
			"B as "+
			"( "+
			"select u.*, ti.curstaffid,p.pjtname,m.mchname,c.cmtname,b.bldname,un.untname from users_info u "+
			"left outer join taskassign_info ti "+
			"on u.usrid = ti.usrid "+
			"left outer join project_info p on p.pjtid = u.pjtid  "+
			"left outer join machineset_info m on m.mchid = u.mchid "+
			"left outer join community_info c on c.cmtid = u.cmtid "+
			"left outer join building_info b on b.bldid = u.bldid "+
			"left outer join unit_info un on un.untid = u.untid "+
			"where u.isvalid=1 and (u.usrname like '%"+query+"%' or u.usertype like '%"+query+"%' or u.phone like '%"+query+"%') "+toadd;
			if(!(stfid.equals(Global.STAFFALL))){
				sql =sql +" and (ti.curstaffid= "+stfid + " or ti.laststaffid="+stfid+")";
			}
			if (cmtid != null && cmtid != 0)
			{
				sql =sql +" and c.cmtid= "+cmtid;
			}
			if (bldid != null && bldid != 0)
			{
				sql =sql +" and b.bldid= "+bldid;
			}
			sql = sql+ ") "+
			", "+
			"C as "+
			"( "+
			"select us.* from A  "+
			"left outer join users_status us "+
			"on A.statusdate = us.statusdate and A.usrid = us.usrid "+
			") "+
			"select B.* from B  "+
			"left outer join C "+
			"on B.usrid=C.usrid ";
			
			if(!(type.equals(Global.USERALL))){
				sql=sql+"where C.status='"+type+"'  ";
			}
			sql = sql + " order by B.usrid asc";
			List rows = this.getJdbcTemplate().queryForList(sql);
			Integer size = rows.size();
			listCount.setTotalProperty(size.longValue());
			if(start+1>size){
				listCount.setData(list);
				return listCount;
			}else if(start+limit>size){
				limit = size-start;
			}
			int count = 0;
			while(count<limit){
				Map map = (Map) (rows.get(start+count));
				UserInfo temp = new UserInfo();
				temp.setUsrid((Long)Global.toNullString(map, "usrid", "BigDecimal", "Long"));
				temp.setUsertype((String)Global.toNullString(map, "usertype", "String", "String"));
				temp.setAddress((String)Global.toNullString(map, "address", "String", "String"));
				temp.setUsrname((String)Global.toNullString(map, "usrname", "String", "String"));
				temp.setPhone((String)Global.toNullString(map, "phone", "String", "String"));
				temp.setStartdate((String)Global.toNullString(map, "startdate", "Timestamp", "String"));
				temp.setContractdate((String)Global.toNullString(map, "contractdate", "Timestamp", "String"));
				temp.setContracttype((String)Global.toNullString(map,"contracttype","String","String"));
				temp.setContractver((String)Global.toNullString(map,"contractver","String","String"));
				temp.setContractpic((String)Global.toNullString(map,"contractpic","String","String"));
				temp.setIdpic((String)Global.toNullString(map,"idpic","String","String"));
				temp.setHouseidpic((String)Global.toNullString(map,"houseidpic","String","String"));
				temp.setHousepic((String)Global.toNullString(map,"housepic","String","String"));
				temp.setDesp((String)Global.toNullString(map,"desp","String","String"));
				temp.setArea((Double)Global.toNullString(map, "area", "BigDecimal", "Double"));
				temp.setRealarea((Double)Global.toNullString(map, "realarea", "BigDecimal", "Double"));
				temp.setFeearea((Double)Global.toNullString(map, "feearea", "BigDecimal", "Double"));
				temp.setFeetype((String)Global.toNullString(map, "feetype", "String", "String"));
				temp.setFeerate((Double)Global.toNullString(map, "feerate", "BigDecimal", "Double"));
				temp.setDiscount((Double)Global.toNullString(map, "discount", "BigDecimal", "Double"));
				temp.setReducefee((Double)Global.toNullString(map, "reducefee", "BigDecimal", "Double"));
				temp.setHeatstate((String)Global.toNullString(map, "heatstate", "String", "String"));
				temp.setHeatbase((Double)Global.toNullString(map, "heatbase", "BigDecimal", "Double"));
				temp.setHeatrate((Double)Global.toNullString(map, "heatrate", "BigDecimal", "Double"));
				temp.setHousetype((String)Global.toNullString(map, "housetype", "String", "String"));
				temp.setBalance((Double)Global.toNullString(map, "balance", "BigDecimal", "Double"));
				temp.setMoney((Double)Global.toNullString(map, "money", "BigDecimal", "Double"));
				temp.setNeedmoney((Double)Global.toNullString(map, "needmoney", "BigDecimal", "Double"));
				temp.setLastdate((String)Global.toNullString(map, "lastdate", "Timestamp", "String"));
				temp.setPjtname((String)Global.toNullString(map, "pjtname", "String", "String"));
				temp.setCmtname((String)Global.toNullString(map, "cmtname", "String", "String"));
				temp.setBldname((String)Global.toNullString(map, "bldname", "String", "String"));
				temp.setUntname((String)Global.toNullString(map, "untname", "String", "String"));
				temp.setMchname((String)Global.toNullString(map, "mchname", "String", "String"));
				temp.setLastnotpay((Long)Global.toNullString(map, "lastnotpay", "BigDecimal", "Long"));
				temp.setCurnotpay((Long)Global.toNullString(map, "curnotpay", "BigDecimal", "Long"));
				temp.setUpid((Long)Global.toNullString(map, "upid", "BigDecimal", "Long"));
				temp.setUncheckbalance((Double)Global.toNullString(map, "uncheckbalance", "BigDecimal", "Double"));
				temp.setUncheckmoney((Double)Global.toNullString(map, "uncheckmoney", "BigDecimal", "Double"));
				temp.setUnchecksave(temp.getUncheckbalance());
				list.add(temp);
				count++;
			}
			listCount.setData(list);
			return listCount;
		}
		else
		{
			String toadd = "";
			if (!all)
				toadd = " and u.pjtid = null";
			else
				if (pjtid != -1)
					toadd = " and u.pjtid = " + pjtid;
			
			String sql = "with "+
			"A as "+
			"( "+
			"select max(statusdate) as statusdate,usrid from users_status where statusdate<to_date('"+date+" 23:59:59','yyyy-MM-dd HH24:mi:ss') group by usrid "+
			") "+
			", "+
			"B as "+
			"( "+
			"select u.*, ti.curstaffid,p.pjtname,m.mchname,c.cmtname,b.bldname,un.untname from users_info u "+
			"left outer join taskassign_info ti "+
			"on u.usrid = ti.usrid "+
			"left outer join project_info p on p.pjtid = u.pjtid  "+
			"left outer join machineset_info m on m.mchid = u.mchid "+
			"left outer join community_info c on c.cmtid = u.cmtid "+
			"left outer join building_info b on b.bldid = u.bldid "+
			"left outer join unit_info un on un.untid = u.untid "+
			"where u.isvalid=1 and (u.usrname like '%"+query+"%' or u.usertype like '%"+query+"%' or u.phone like '%"+query+"%') and si.stfid="+sp.getStaffInfo().getStfid()+toadd;
			if(!(stfid.equals(Global.STAFFALL))){
				sql =sql +" and (ti.curstaffid= "+stfid + " or ti.laststaffid="+stfid+")";
			}
			if (cmtid != 0)
			{
				sql =sql +" and c.cmtid= "+cmtid;
			}
			if (bldid != 0)
			{
				sql =sql +" and b.bldid= "+bldid;
			}
			sql = sql+ ") "+
			", "+
			"C as "+
			"( "+
			"select us.* from A  "+
			"left outer join users_status us "+
			"on A.statusdate = us.statusdate and A.usrid = us.usrid "+
			") "+
			"select B.* from B  "+
			"left outer join C "+
			"on B.usrid=C.usrid ";
			
			if(!(type.equals(Global.USERALL))){
				sql=sql+"where C.status='"+type+"'  ";
			}
			sql = sql + " order by B.usrid asc";
			List rows = this.getJdbcTemplate().queryForList(sql);
			Integer size = rows.size();
			listCount.setTotalProperty(size.longValue());
			if(start+1>size){
				listCount.setData(list);
				return listCount;
			}else if(start+limit>size){
				limit = size-start;
			}
			int count = 0;
			while(count<limit){
				Map map = (Map) (rows.get(start+count));
				UserInfo temp = new UserInfo();
				temp.setUsrid((Long)Global.toNullString(map, "usrid", "BigDecimal", "Long"));
				temp.setUsertype((String)Global.toNullString(map, "usertype", "String", "String"));
				temp.setAddress((String)Global.toNullString(map, "address", "String", "String"));
				temp.setUsrname((String)Global.toNullString(map, "usrname", "String", "String"));
				temp.setPhone((String)Global.toNullString(map, "phone", "String", "String"));
				temp.setStartdate((String)Global.toNullString(map, "startdate", "Timestamp", "String"));
				temp.setContractdate((String)Global.toNullString(map, "contractdate", "Timestamp", "String"));
				temp.setContracttype((String)Global.toNullString(map,"contracttype","String","String"));
				temp.setContractver((String)Global.toNullString(map,"contractver","String","String"));
				temp.setContractpic((String)Global.toNullString(map,"contractpic","String","String"));
				temp.setIdpic((String)Global.toNullString(map,"idpic","String","String"));
				temp.setHouseidpic((String)Global.toNullString(map,"houseidpic","String","String"));
				temp.setHousepic((String)Global.toNullString(map,"housepic","String","String"));
				temp.setDesp((String)Global.toNullString(map,"desp","String","String"));
				temp.setArea((Double)Global.toNullString(map, "area", "BigDecimal", "Double"));
				temp.setRealarea((Double)Global.toNullString(map, "realarea", "BigDecimal", "Double"));
				temp.setFeearea((Double)Global.toNullString(map, "feearea", "BigDecimal", "Double"));
				temp.setFeetype((String)Global.toNullString(map, "feetype", "String", "String"));
				temp.setFeerate((Double)Global.toNullString(map, "feerate", "BigDecimal", "Double"));
				temp.setDiscount((Double)Global.toNullString(map, "discount", "BigDecimal", "Double"));
				temp.setReducefee((Double)Global.toNullString(map, "reducefee", "BigDecimal", "Double"));
				temp.setHeatstate((String)Global.toNullString(map, "heatstate", "String", "String"));
				temp.setHeatbase((Double)Global.toNullString(map, "heatbase", "BigDecimal", "Double"));
				temp.setHeatrate((Double)Global.toNullString(map, "heatrate", "BigDecimal", "Double"));
				temp.setHousetype((String)Global.toNullString(map, "housetype", "String", "String"));
				temp.setBalance((Double)Global.toNullString(map, "balance", "BigDecimal", "Double"));
				temp.setMoney((Double)Global.toNullString(map, "money", "BigDecimal", "Double"));
				temp.setNeedmoney((Double)Global.toNullString(map, "needmoney", "BigDecimal", "Double"));
				temp.setLastdate((String)Global.toNullString(map, "lastdate", "Timestamp", "String"));
				temp.setPjtname((String)Global.toNullString(map, "pjtname", "String", "String"));
				temp.setCmtname((String)Global.toNullString(map, "cmtname", "String", "String"));
				temp.setBldname((String)Global.toNullString(map, "bldname", "String", "String"));
				temp.setUntname((String)Global.toNullString(map, "untname", "String", "String"));
				temp.setMchname((String)Global.toNullString(map, "mchname", "String", "String"));
				temp.setLastnotpay((Long)Global.toNullString(map, "lastnotpay", "BigDecimal", "Long"));
				temp.setCurnotpay((Long)Global.toNullString(map, "curnotpay", "BigDecimal", "Long"));
				temp.setUpid((Long)Global.toNullString(map, "upid", "BigDecimal", "Long"));
				temp.setUncheckbalance((Double)Global.toNullString(map, "uncheckbalance", "BigDecimal", "Double"));
				temp.setUncheckmoney((Double)Global.toNullString(map, "uncheckmoney", "BigDecimal", "Double"));
				temp.setUnchecksave(temp.getUncheckbalance());
				list.add(temp);
				count++;
			}
			listCount.setData(list);
			return listCount;
		}
	}
}
