library(survival)
options(scipen=999) # remove scientific notation

toDays <- function(ml) {
  return (ml / 1000 / 60 / 60 / 24);
}

mvc <- read.csv("0-all-mvc.csv", header=T)
mvc$days <- as.numeric(toDays(mvc$lastseen) - toDays(mvc$started))
mvc$diffcommits <- mvc$lastseen_number - mvc$started_number
mvc <- mvc[mvc$days > 1 & mvc$diffcommits > 1,]

# total number of instances per smell
table(mvc$smell)

# median of the 'days' for each smell
aggregate(mvc[, 9], list(mvc$smell), mean)

# survivors per smell
table(mvc$alive, mvc$smell)

# survival analysis
surv <- survfit(Surv(mvc$days, mvc$alive)~1)
summary(surv)
plot(surv, xlab="Time", ylab="Survival Probability")

# survival analysis per smell
# repated it per smell
for(smell in unique(mvc$smell)) {
  png(file=paste("../img/survival/", gsub(" ", "-", smell), ".pdf", sep=""))
  mvc2 <- mvc[mvc$smell == smell,]
  surv <- survfit(Surv(mvc2$days, mvc2$alive)~1)
  summary(surv)
  plot(surv, xlab="Time", ylab="Survival Probability")
  dev.off()
}