library(effsize)
library(sqldf)

# ------------------------------------------------
# Configuration
# ------------------------------------------------
role_ps <- c(1,1,4,4,4,2)
smells_ps <- c("Promiscuous Controller", "Smart Controller", "Fat Repository", "Multiple queries", "Smart Repository", "DB Querying service")
# ------------------------------------------------

compare <- function(dat,smell,pmd) {
  
  non_smelly_with_d <- nrow(sqldf("select * from dat d where smell = 'no' and bugs > 0 and exists (select 1 from pmd p where p.file = d.file and p.release = d.release and p.smell = 'no')"))
  non_smelly_with_non_d <- nrow(sqldf("select * from dat d where smell = 'no' and bugs = 0 and exists (select 1 from pmd p where p.file = d.file and p.release = d.release and p.smell = 'no')"))
  
  smelly_with_d <- nrow(dat[dat$smell==smell & dat$bugs > 0,])
  smelly_with_non_d <- nrow(dat[dat$smell==smell & dat$bugs == 0,])
  
  m <- matrix(c(smelly_with_d,smelly_with_non_d,non_smelly_with_d,non_smelly_with_non_d),
              nrow = 2,
              dimnames =
                list(c("Defect", "Non Defect"),
                     c("Smelly", "Non Smelly")))
  
  test <- fisher.test(m)
  return(c(smelly_with_d, smelly_with_non_d,non_smelly_with_d,non_smelly_with_non_d, test$p.value, test$estimate, test$conf.int[1], test$conf.int[2]))
}

file_output <- "output/results.csv"
cat("smell,loc,a,b,c,d,fisher,estimate,confint1,confint2\n", file=file_output)

cmp <- read.csv("../data/smell-per-release-nice-sample.csv")
pmd <- read.csv("../data/pmd-smell-per-release-nice-sample.csv")

for(s in 1:length(role_ps)) {
  role <- role_ps[s]
  smell <- smells_ps[s]
  
  print(smell)
  
  loc_stats <- summary(cmp[cmp$role == role,]$loc)
  mins <- c(           1,loc_stats[2],loc_stats[3],loc_stats[5])
  maxs <- c(loc_stats[2],loc_stats[3],loc_stats[5],9999999)
  
  for(i in 1:length(mins)) {
    min <- mins[i]
    max <- maxs[i]
    
    filtered_by_loc <- cmp[cmp$loc >= min & cmp$loc < max & cmp$role == role,]
    
    test <- compare(filtered_by_loc,smell,pmd)
    cat(paste(smell,i,test[1],test[2],test[3], test[4], test[5], test[6], test[7], test[8], sep=","), file=file_output,append=TRUE)     
    cat("\n",file=file_output,append=TRUE)
  }
}
