library(effsize)
library(sqldf)

# ------------------------------------------------
# Configuration
# ------------------------------------------------
role_ps <- c(1,2,4)
smells_ps <- c("GodClass", "CouplingBetweenObjects", "CyclomaticComplexity", "ExcessiveClassLength", "ExcessiveMethodLength", "ExcessiveParameterList")
# ------------------------------------------------

compare <- function(dat,smell,specific) {
  
  non_smelly_with_d <- nrow(sqldf("select * from dat d where smell = 'no' and bugs > 0 and exists (select 1 from specific p where p.file = d.file and p.release = d.release and p.smell = 'no')"))
  non_smelly_with_non_d <- nrow(sqldf("select * from dat d where smell = 'no' and bugs = 0 and exists (select 1 from specific p where p.file = d.file and p.release = d.release and p.smell = 'no')"))
  
  smelly_with_d <- nrow(dat[dat$smell==smell & dat$bugs > 0,])
  smelly_with_non_d <- nrow(dat[dat$smell==smell & dat$bugs == 0,])
  
  m <- matrix(c(smelly_with_d,smelly_with_non_d,non_smelly_with_d,non_smelly_with_non_d),
              nrow = 2,
              dimnames =
                list(c("Defect", "Non Defect"),
                     c("Smelly", "Non Smelly")))
  
  test <- fisher.test(m)
  return(c(smelly_with_d, smelly_with_non_d,non_smelly_with_d,non_smelly_with_non_d, test$p.value, test$estimate, test$conf.int[1], test$conf.int[2]))
}

file_output <- "output/results.csv"
cat("role,smell,a,b,c,d,fisher,estimate,confint1,confint2\n", file=file_output)

cmp <- read.csv("../data/pmd-smell-per-release-nice-sample.csv")
specific <- read.csv("../data/smell-per-release-nice-sample.csv")

for(role in role_ps) {
  print(role)
  for(smell in smells_ps) {
    print(smell)
    
    filtered_by_loc <- cmp[cmp$role == role,]
    
    test <- compare(filtered_by_loc,smell,specific)
    cat(paste(role,smell,test[1],test[2],test[3], test[4], test[5], test[6], test[7], test[8], sep=","), file=file_output,append=TRUE)     
    cat("\n",file=file_output,append=TRUE)
  }
}